/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMPackets.h
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2002 iXQ-Team. All rights reserved.
//

#import <Foundation/Foundation.h>


@interface SMPackets : NSObject
{
    //here is what we need to describe an ICQ packet :
    //an Id (not used for now, just in case)
    unsigned long packetID;
    //the data within the packet
    unsigned char data[9000];
    //the length of the data
    unsigned short length;
    //when a packet is Final, it means that it will be sent to the server and we have then to calculate a header
    BOOL isFinal;
    //because of the structure of an ICQ Packet, there are subpart in the packet identified by TLV
    int currentTLVPosition;
}

//convenience methods
+ (SMPackets *) packet;
+ (SMPackets *) packetForChannel:(unsigned char)channel withParams:(int)seqNum;
+ (SMPackets *) packetWithBytes:(void*)bytes andSize:(unsigned short)size;

    //Initialisation of a packet
- (SMPackets *) init;
- (SMPackets *) initForChannel:(unsigned char)channel withParams:(int)seqNum;
- (SMPackets *) initWithBytes:(void*)bytes andSize:(unsigned short)size;


    //Access Data
- (unsigned char*) bytes;
- (NSData*) dataBytes;
- (unsigned short) length;
    //Access specific data
- (unsigned char) getByteAtPosition:(int)position;
- (unsigned short) get2BytesAtPosition:(int)position;
- (unsigned long) get4BytesAtPosition:(int)position;
- (void*) getBytesFrom:(unsigned long)position length:(unsigned short)size;
- (unsigned char) extractChannel;
    //Access TLV-related data
- (int) getCurrentTLV;
- (int) getCurrentTLVSize;
- (void*) getDataOfCurrentTLV;
- (int) getTLVPosition;
- (int) goToNextTLV;
- (int) setTLVPosition:(int)newPosition;
- (void) debugCurrentTLV;
- (void) debugPacket;


    //Modify the packet
- (void) setByte:(unsigned char) unsignedChar atPosition:(int)position;
- (void) set2Bytes:(unsigned short) unsignedShort atPosition:(int)position;
- (void) set4Bytes:(unsigned long) unsignedLong atPosition:(int)position;
- (void) setString:(NSString*) aString atPosition:(int)position;
- (void) setLNTString:(NSString*) aString atPosition:(int)position;
- (void) setBWString:(NSString*) aString atPosition:(int)position;
- (void) setUINString:(NSString*) aString atPosition:(int)position;
- (void) appendPacket:(SMPackets*) otherPacket;

    //Calulate the size and add it within the packet
- (int) setSizeInPacket;
    //Add a protocol related command (SNAC)
- (int) addSnacCommandOfFamily:(unsigned short)family withType:(unsigned short)type;
- (int) addSnacCommandOfFamily:(unsigned short)family withType:(unsigned short)type andFlags:(unsigned long)flags;

    //Managing TLV
- (int) addTLV:(unsigned short)tlvNumber withSize:(short)size;
/*
- (void)encodeWithCoder:(NSCoder *)coder;
- (id) initWithCoder:(NSCoder *)decoder;
*/
@end
