/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMPackets.m
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2002 iXQ-Team. All rights reserved.
//

#import "SMPackets.h"


@implementation SMPackets


//convenience methods
+ (SMPackets *) packet
{
    return [[[SMPackets alloc] init] autorelease];
}

+ (SMPackets *) packetForChannel:(unsigned char)channel withParams:(int)seqNum
{
    return [[[SMPackets alloc] initForChannel:channel withParams:seqNum] autorelease];
}

+ (SMPackets *) packetWithBytes:(void*)bytes andSize:(unsigned short)size
{
    return [[[SMPackets alloc] initWithBytes:bytes andSize:size] autorelease];
}


//Packet initialisation
- (SMPackets *) init
{
    packetID=0;
    length=0;
    isFinal=NO;
    currentTLVPosition=0;

    return self;
}

//because it has an header, this packet is considered as final
- (SMPackets *) initForChannel:(unsigned char)channel withParams:(int)seqNum;
{
    //Standard initialisation
    [self init];

    //Adding the header with given parameters
    //(command byte - channel - sequence number - size)
    [self setByte:((unsigned char)0x2A) atPosition:length];
    [self setByte:((unsigned char)channel) atPosition:length];
    [self set2Bytes:((unsigned short)seqNum) atPosition:length];
    [self set2Bytes:((unsigned short)0) atPosition:length];

    isFinal=YES;
    return self;
}


- (SMPackets *) initWithBytes:(void*)bytes andSize:(unsigned short)size
{
    //standard initialisation
    [self init];

    //copying the original packet
    length=size;
    memcpy(data, bytes , length);

    return self;
}


//Accessing data
- (unsigned char*) bytes
{
    return data;
}

- (NSData*) dataBytes
{
    NSData* dataBytes = [NSData dataWithBytes:data length:length];

    return dataBytes;
}

- (unsigned short) length
{
    return length;
}

//Accessing specific data
//Accessing an unsigned char
- (unsigned char) getByteAtPosition:(int)position
{
    unsigned char result;
    if (length<position+1)
        return 0;
    memcpy(&result, data+position , sizeof(unsigned char));
    return result;
}

//Accessing an unsigned short
- (unsigned short) get2BytesAtPosition:(int)position
{
    unsigned short result;
    if (length<position+2)
        return 0;
    memcpy(&result, data+position , sizeof(unsigned short));
    return result;
}

//Accessing an unsigned long
- (unsigned long) get4BytesAtPosition:(int)position
{
    unsigned long result;
    if (length<position+4)
        return 0;
    memcpy(&result, data+position , sizeof(unsigned long));
    return result;
}

- (void*) getBytesFrom:(unsigned long)position length:(unsigned short)size
{
    if (length<position+size)
        return nil;
    else
    {
        void* bytes=malloc(size*sizeof(unsigned char));
        memcpy(bytes, data+position, size*sizeof(unsigned char));
        return bytes;
    }
}

//Accessing the packet channel
//(there are 4 channels : connection/data/errors/disconnection)
- (unsigned char) extractChannel
{
    return [self getByteAtPosition:1];
}

//Accessing TLV related data
//Accesing the ID of the current TLV
- (int) getCurrentTLV
{
    return [self get2BytesAtPosition:currentTLVPosition];
}

- (void) debugPacket
{
    int c;
    NSMutableString* dump = [NSMutableString string];

    for(c = 0; c < length; c++) {
        if(c != 0 && (c % 4) == 0) [dump appendString:@" "];
        [dump appendFormat:@"%02X", (unsigned char)data[c]];
    }

    NSLog(@"Size: 0x%02X Data: 0x%@", length, dump);
}

- (void) debugCurrentTLV
{
    int c;
    int tlv = [self getCurrentTLV];
    int size = [self getCurrentTLVSize];
    char* rawdata = [self getDataOfCurrentTLV];
    NSMutableString* dump = [NSMutableString string];

    for(c = 0; c < size; c++) {
        if(c != 0 && (c % 4) == 0) [dump appendString:@" "];
        [dump appendFormat:@"%02X", (unsigned char)rawdata[c]];
    }

    NSLog(@"TLV: 0x%02X Size: 0x%02X Data: 0x%@", tlv, size, dump);
}

//Accessing the length of the data in the TLV
- (int) getCurrentTLVSize
{
    return [self get2BytesAtPosition:currentTLVPosition+2];
}

- (int) getTLVPosition
{
    return currentTLVPosition;
}

//Accessing the data within the TLV
- (void*) getDataOfCurrentTLV
{
    return [self getBytesFrom:(currentTLVPosition+4) length:[self getCurrentTLVSize]];
}

//Jumping to next TLV
//it returns the ID of the TLV
//If the end of the packet is reached, it returns 0
- (int) goToNextTLV
{
    int size = [self getCurrentTLVSize];
    int tlv;
    //new position=current position + data size + ID size + data size size
    currentTLVPosition += size+4;
    if (currentTLVPosition<length&&size!=0)
        tlv = [self get2BytesAtPosition:currentTLVPosition];
    else
        tlv=0;

    return tlv;
}
//Determining the position of the current TLV
- (int) setTLVPosition:(int)newPosition
{
    currentTLVPosition=newPosition;
    return 0;
}


//Modify the packet
//= Modify some data at a specified position
//Modify the size of the packet if needed
//Modify the header if the packet has one
- (void) setByte:(unsigned char) unsignedChar atPosition:(int)position
{
    if (position+1>length)
        length=position+1;
    if (isFinal)
        [self setSizeInPacket];
    (*(unsigned char*)(data+position))=unsignedChar;
}

- (void) set2Bytes:(unsigned short) unsignedShort atPosition:(int)position
{
    if (position+2>length)
        length=position+2;
    if (isFinal)
        [self setSizeInPacket];
    (*(unsigned short*)(data+position))=unsignedShort;
}

- (void) set4Bytes:(unsigned long) unsignedLong atPosition:(int)position
{
    if (position+4>length)
        length=position+4;
    if (isFinal)
        [self setSizeInPacket];
    (*(unsigned long*)(data+position))=unsignedLong;
}

//Including a string within the packet
- (void) setString:(NSString*) aString atPosition:(int)position
{
    int counter;
    int stringLength=[aString length];

    for (counter=0;counter<stringLength;counter++)
    {
        [self setByte:(unsigned char)([aString characterAtIndex:counter]) atPosition:(position+counter)];
    }
}

//LNTS is a word (LE) preceeded NullTerminatedString: the word indicates the length of the NTS (null char included)
- (void) setLNTString:(NSString*) aString atPosition:(int)position
{
    [self set2Bytes:NSSwapShort((unsigned short)([aString length]+1)) atPosition:position];
    [self setString:aString atPosition:position+2];
    [self setByte:0 atPosition:position+2+[aString length]];    
}

// BWS is a word (BE) preceeded STRING: the word indicates the length of the STRING
- (void) setBWString:(NSString*) aString atPosition:(int)position
{
    [self set2Bytes:((unsigned short)([aString length])) atPosition:position];
    [self setString:aString atPosition:position+2];
}

// B-UIN is a BYTE preceded STRING: the byte indicates the length of the string and the string report an uin number
- (void) setUINString:(NSString*) aString atPosition:(int)position
{
    int stringLength=[aString length];

    [self setByte:((unsigned short)(stringLength)) atPosition:position];
    [self setString:aString atPosition:position+1];
}

//Appending a packet
//where appending each byte of the other packet at the end of the current packet
- (void) appendPacket:(SMPackets*) otherPacket
{
    int i;
    for (i=0;i<[otherPacket length];i++)
    {
        [self setByte:[otherPacket getByteAtPosition:i] atPosition:length];
    }
}

//Adding the size of the packet within the header
- (int) setSizeInPacket
{
    (*(unsigned short*)(data+4))=length-6;
    return 0;
}


//Adding a protocol related command
- (int) addSnacCommandOfFamily:(unsigned short)family withType:(unsigned short)type
{
    //NSLog([NSString stringWithFormat:@"New SNAC Command : %X/%X", family, type]);
  [self addSnacCommandOfFamily:family withType:type andFlags:0];

    return 0;
}

- (int) addSnacCommandOfFamily:(unsigned short)family withType:(unsigned short)type andFlags:(unsigned long)flags
{
//    NSLog([NSString stringWithFormat:@"New SNAC Command : %X/%X", family, type]);
    [self set2Bytes:family atPosition:length];
    [self set2Bytes:type atPosition:length];
    [self set4Bytes:flags atPosition:length];
    [self set2Bytes:type atPosition:length];

    return 0;
}

//Managing TLV : adding it at the end of the packet
//Add the size of the TLV if this size is not -1
- (int) addTLV:(unsigned short)tlvNumber withSize:(short)size
{
    [self set2Bytes:tlvNumber atPosition:length];
    if (size!=-1)
        [self set2Bytes:size atPosition:length];
    return 0;
}
/*
- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeValueOfObjCType:@encode(unsigned short) at:&length];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&isFinal];
    [coder encodeArrayOfObjCType:@encode(unsigned char) count:length at:data];
}

- (id) initWithCoder:(NSCoder *)decoder
{
    [decoder decodeValueOfObjCType:@encode(unsigned short) at:&length];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&isFinal];
    [decoder decodeArrayOfObjCType:@encode(unsigned char) count:length at:data];
    packetID=0;
    currentTLVPosition=0;
    
    return self;
}*/


@end
