/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMContactListItem.m
//  iXQ
//
//  Created by starman on Tue Oct 23 2001.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMContactListItem.h"


@implementation SMContactListItem

static const NSString* statusTable[] = {
    @"Offline", @"Available", @"Away", @"N/A", @"Occupied", @"DND", @"Free For Chat", @"Invisible"
};

+ (SMContactListItem*) newItemWithName:(NSString*)newItemName andUIN:(int)newItemUIN
{
    return [[[SMContactListItem alloc] initItemWithName:newItemName andUIN:newItemUIN] autorelease];
}

- (SMContactListItem*) initItemWithName:(NSString*)newItemName andUIN:(int)newItemUIN
{
    [children removeAllObjects];
    [children release];
    [nickname release];
    children=[[NSMutableArray alloc] init];
    [nickname release];
    nickname=@"";
    [nickname retain];
    [name release];
    [firstname release];
    [lastname release];
    [email release];
    [smsCountry release];
    [smsArea release];
    [smsNumber release];
    smsCountry=@"";
    smsArea=@"";
    smsNumber=@"";
    [smsCountry retain];
    [smsArea retain];
    [smsNumber retain];
    [capabilities release];
    capabilities=@"";
    [capabilities retain];
    name=[newItemName copy];
    uin=newItemUIN;
    status=0;
    convert=NO;
    isExpanded=NO;
    return self;
}

+ (SMContactListItem*) newItemWithName:(NSString*)newItemName uin:(int)newItemUIN firstname:(NSString*)first lastname:(NSString*)last email:(NSString*)mail
{
    return [[[SMContactListItem alloc] initItemWithName:newItemName uin:newItemUIN firstname:first lastname:last email:mail] autorelease];
}

- (SMContactListItem*) initItemWithName:(NSString*)newItemName uin:(int)newItemUIN firstname:(NSString*)first lastname:(NSString*)last email:(NSString*)mail
{
    [children removeAllObjects];
    [children release];
    children=[[NSMutableArray alloc] init];
    [name release];
    name=[newItemName copy];
    uin=newItemUIN;
    [firstname release];
    firstname=[first copy];
    [lastname release];
    lastname=[last copy];
    [email release];
    email=[mail copy];
    if (smsCountry==nil)
    {
        smsCountry=@"";
        [smsCountry retain];
    }
    if (smsArea==nil)
    {
        smsArea=@"";
        [smsArea retain];
    }
    if (smsNumber==nil)
    {
        smsNumber=@"";
        [smsNumber retain];
    }
    
    if (capabilities==nil)
    {
        capabilities=@"";
        [capabilities retain];
    }
    if (nickname==nil)
    {
	nickname=@"";
        [nickname retain];
    }
    return self;
}

- (void) dealloc
{
    [children removeAllObjects];
    [name release];
    [capabilities release];
    [children release];
    [nickname release];
    [lastname release];
    [firstname release];
    [email release];
    [smsCountry release];
    [smsArea release];
    [smsNumber release];
}

- (int) setAwaitingMessage:(BOOL)isIt
{
    awaitingMessage=isIt;
    
    return 0;
}

- (BOOL) isExpanded
{
    return isExpanded;
}

- (int) setExpand:(BOOL)itIs
{
    isExpanded=itIs;
    return 0;
}

- (BOOL) hasAnAwaitingMessage
{
    return awaitingMessage;
}

- (NSComparisonResult) orderByStatus:(id)comparedObject
{
    int status1=(status+7)%8;
    int status2=([comparedObject getStatusNb]+7)%8;
    if (status1==status2)
        return [[self getNickname] compare:[comparedObject getNickname]];
    if (status1>status2)
        return NSOrderedDescending;
    return NSOrderedAscending;
}

- (int) sortByStatus
{
    [children sortUsingSelector:@selector(orderByStatus:)];
    
    return 0;
}

- (int) addUser:(id)user
{
    int userUIN=[user getUIN];
    int counter;
    [user retain];
    for (counter=0;counter<[children count];counter++)
    {
	if ([[children objectAtIndex:counter] getUIN]==userUIN)
	    return -1;
    }
    [children addObject:user];
    [user release];
    return 0;
}

- (int) addUser:(id)user atIndex:(int)index
{
    int userUIN=[user getUIN];
    int counter;
    [user retain];
    for (counter=0;counter<[children count];counter++)
    {
	if ([[children objectAtIndex:counter] getUIN]==userUIN)
	    return -1;
    }
    [children insertObject:user atIndex:index];
    [user release];
    return 0;
}

- (int) removeUserAtPosition:(int)position
{
    [children removeObjectAtIndex:position];
    return 0;
}

- (int) removeUser:(id)user
{
    [children removeObject:user];
    return 0;
}

- (NSString*) getName
{
    return name;
}

- (int) setName:(NSString*)newName
{
    [name release];
    name=[newName copy];
    
    return 0;
}

- (NSString*) getNickname
{
    if (nickname==nil)
        return name;
    if ([nickname isEqualToString:@""])
	return name;
    else
	return nickname;
}

- (int) setNickname:(NSString*)newNickname
{
    [nickname release];
    nickname=[newNickname copy];
    
    return 0;
}

- (NSString*) getCapabilities
{
    if (capabilities==nil)
        return @"";
    return capabilities;
}

- (int) setCapabilities:(NSString*)newCapabilities
{
    [newCapabilities retain];
    //[capabilities release];
    capabilities=[newCapabilities copy];
    [newCapabilities release];
    return 0;
}

- (NSString*) getStatus
{
    return (NSString*) statusTable[status];
}

- (int) getStatusNb
{
    return status;
}

- (int) setSMSNumber:(NSString*)newSMSNumber forCountry:(NSString*)newSMSCountry andArea:(NSString*)newSMSArea
{
    [newSMSNumber retain];
    [newSMSCountry retain];
    [newSMSArea retain];

    [smsArea release];
    [smsNumber release];
    [smsCountry release];

    smsNumber=[newSMSNumber copy];
    smsCountry=[newSMSCountry copy];
    smsArea=[newSMSArea copy];
    
    [newSMSNumber release];
    [newSMSCountry release];
    [newSMSArea release];
    return 0;
}

- (NSDictionary*) getSMSInfos
{
    NSMutableDictionary* smsInfos=[NSMutableDictionary dictionary];
    [smsInfos setObject:smsNumber forKey:@"number"];
    [smsInfos setObject:smsArea forKey:@"area"];
    [smsInfos setObject:smsCountry forKey:@"country"];

    return smsInfos;
}

- (int) setStatus:(int)userStatus
{
    status=userStatus;
    //NSLog(name);
    return 0;
}

- (int) setLastname:(NSString*)newLast
{
    [lastname release];
    lastname=[newLast retain];
    return 0;
}

- (int) setFirstname:(NSString*)newFirst
{
    [firstname release];
    firstname=[newFirst retain];
    return 0;
}

- (int) setIPAddress:(NSString*)newIP
{
    [ipadress release];
    ipadress=[newIP retain];
    return 0;
}

- (int) setMail:(NSString*)newMail
{
    [email release];
    email=[newMail retain];
    return 0;
}

- (NSString*) getLastname
{
    return lastname;
}

- (NSString*) getFirstname
{
    return firstname;
}

- (NSString*) getIPAdress
{
    return ipadress;
}

- (NSString*) getMail
{
    return email;
}

- (int) getUIN
{
    return uin;
}

- (NSSet*) getContacts
{
    NSMutableSet* contacts=[NSMutableSet set];
    int counter;
    for (counter=0;counter<[children count];counter++)
    {
	[contacts addObject:[NSString stringWithFormat:@"%d", [[children objectAtIndex:counter] getUIN]]];
    }
    return contacts;
}

- (int) getOnlineUsersNb
{
    int result=0;
    int counter;
    for (counter=0;counter<[children count];counter++)
    {
	if ([[children objectAtIndex:counter] getStatusNb]!=0||[[children objectAtIndex:counter] hasAnAwaitingMessage])
	    result++;
    }
    return result;
}

- (NSSet*) getContactsObjects
{
    NSMutableSet* contacts=[NSMutableSet set];
    int counter;
    for (counter=0;counter<[children count];counter++)
    {
	[contacts addObject:[children objectAtIndex:counter]];
    }
    return contacts;
}

- (id) getChildAtPosition:(int)position
{
    return [children objectAtIndex:position];
}

- (id) getOnlineChildAtPosition:(int)position
{
    int counter=0;
    int counterPosition=-1;
    for (counter=0;counter<[children count];counter++)
    {
        if ([[children objectAtIndex:counter] getStatusNb]>0||[[children objectAtIndex:counter] hasAnAwaitingMessage])
            counterPosition++;
        if (counterPosition==position)
            return [children objectAtIndex:counter];
    }
    return nil;
}

- (int) getNumberOfChildren
{
    return [children count];
}

- (int) goOffline
{
    int counter;
    for (counter=0;counter<[children count];counter++)
    {
	[[children objectAtIndex:counter] setStatus:0];
    }
    
    return 0;
}

- (int) toggleConversion
{
    convert=!convert;
    
    return 0;
}
- (BOOL) getConversion
{
    return convert;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    int version=4;
    [coder encodeValueOfObjCType:@encode(int) at:&version];
    [coder encodeObject:name];
    [coder encodeObject:children];
    [coder encodeObject:email];
    [coder encodeObject:firstname];
    [coder encodeObject:lastname];
    [coder encodeValueOfObjCType:@encode(int) at:&uin];
    [coder encodeObject:nickname];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&convert];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&isExpanded];
    [coder encodeObject:smsNumber];
    [coder encodeObject:smsArea];
    [coder encodeObject:smsCountry];
}

- (id) initWithCoder:(NSCoder *)decoder
{
    int version;
    convert=NO;
    [decoder decodeValueOfObjCType:@encode(int) at:&version];
    name = [[decoder decodeObject] retain];
    children = [[decoder decodeObject] retain];
    email = [[decoder decodeObject] retain];
    firstname = [[decoder decodeObject] retain];
    lastname = [[decoder decodeObject] retain];
    [decoder decodeValueOfObjCType:@encode(int) at:&uin];
    nickname = [[decoder decodeObject] retain];
    status=0;
    smsNumber = @"";
    smsArea = @"";
    smsCountry = @"";

    [smsCountry retain];
    [smsArea retain];
    [smsNumber retain];
    if (version==1)
	return self;
        
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&convert];
    if (version==2)
	return self;
        
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&isExpanded];
    if (version==3)
	return self;

    [smsCountry release];
    [smsArea release];
    [smsNumber release];
    smsNumber = [[decoder decodeObject] retain];
    smsArea = [[decoder decodeObject] retain];
    smsCountry = [[decoder decodeObject] retain];

    if (smsCountry==nil)
    {
        smsCountry=@"";
        [smsCountry retain];
    }
    if (smsArea==nil)
    {
        smsArea=@"";
        [smsArea retain];
    }
    if (smsNumber==nil)
    {
        smsNumber=@"";
        [smsNumber retain];
    }
    
    if (version==4)
        return self;
    
    return self;
}

@end
