/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMUINPrefs.h
//  iXQ
//
//  Created by starman on Mon Oct 22 2001.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>

@interface SMUINPrefs : NSObject {
    NSMutableArray * groupList;
    NSString* password;
    int version;
    BOOL isPasswordSaved;
    BOOL autoAway;
    BOOL defaultClose;
    BOOL ignoreMessages;
    BOOL messagesDate;
    BOOL spellChecking;
    BOOL utilityContactList;
    BOOL hideOffline;
    BOOL sortByStatus;
    BOOL bouncingIcon;
    BOOL displayMessageWhenContactOnline;
    BOOL speakMessageWhenContactOnline;
    BOOL speakMessageWhenNewMessage;
    BOOL playSoundWhenNewMessage;
    BOOL visibleToolbar;
    BOOL mute;
    BOOL autoConnect;
    int autoAwayTime;
    float alphaLevel;
    NSString* nickName;
    NSString* firstName;
    NSString* lastName;
    NSString* soundPath;
    NSSound* defaultSound;
    id encryptedPassword;
    id usersDetails;
    int status;
    int oldStatus;
    NSColor* listColor;
    NSColor* contactListTextColor;
    NSColor* contactListSelectionColor;
    NSColor* historySelfColor;
    NSColor* historyContactColor;
    NSString* theme;
    NSMutableSet* unknownContacts;
    NSRect contactListWindowFrame;
    
    NSDate *loginDate;
}

+ sharedInstance;

- (id) init;
- (int) initForPath:(NSString*)path;
- (int) getVersion;
- (NSString*) getPassword;
- (BOOL) isPasswordSaved;
- (NSString*) getNickName;
- (NSString*) getFirstName;
- (NSString*) getLastName;
- (id) getGroups;

- (BOOL) getAutoAway;
- (int) getAutoAwayTime;
- (BOOL) getDefaultClose;
- (BOOL) getIgnoreMessages;
- (BOOL) getMessagesDate;
- (BOOL) getSpellChecking;
- (BOOL) getUtilityContactList;
- (BOOL) getBouncingIcon;
- (BOOL) getMute;

- (BOOL) getHideOffline;
- (BOOL) getToolbarVisible;
- (int) setToolbarVisible:(BOOL)itIs;
- (BOOL) getSortByStatus;
- (int) setAutoConnect:(BOOL)itIs;
- (BOOL) getAutoConnect;
- (NSString*) getTheme;
- (float) getAlphaLevel;
- (int) setAlphaLevel:(float)newLevel;
- (NSColor*) getListColor;
- (int) setListColor:(NSColor*)newColor;
- (NSColor*) contactListTextColor;
- (int) setContactListTextColor:(NSColor*)newColor;
- (NSColor*) contactListSelectionColor;
- (int) setContactListSelectionColor:(NSColor*)newColor;
- (NSColor*) historySelfColor;
- (int) setHistorySelfColor:(NSColor*)newColor;
- (NSColor*) historyContactColor;
- (int) setHistoryContactColor:(NSColor*)newColor;
- (int) setTheme:(NSString*)newTheme;
- (int) setSortByStatus:(BOOL)itIs;
- (int) setHideOffline:(BOOL)itIs;
- (int) setAutoAwayTime:(int)time;
- (int) setAutoAway:(BOOL)itIs;
- (int) setMute:(BOOL)itIs;
- (int) setBouncingIcon:(BOOL)itIs;
- (int) setDefaultClose:(BOOL)itIs;
- (int) setIgnoreMessages:(BOOL)itIs;
- (int) setMessagesDate:(BOOL)itIs;
- (int) setSpellChecking:(BOOL)itIs;
- (int) setUtilityContactList:(BOOL)itIs;

- (int) setPassword:(NSString*)newPassword;
- (int) setIsPasswordSaved:(BOOL)isIt;
- (int) setNickName:(NSString*)newNickName;
- (int) setFirstName:(NSString*)newFirstName;
- (int) setLastName:(NSString*)newLastName;

- (int) setStatus:(int)newStatus;
- (int) getStatus;
- (int) getOldStatus;

- (int) setCLWFrame:(NSRect)frame;
- (NSRect) getCLWFrame;

- (id) getUnknownContactWithUIN:(int)uin;
- (int) addUnknownContact:(id)contact;
- (int) addUser:(id)user toGroup:(int)group;
- (int) addUser:(id)user toGroup:(int)group atIndex:(int)index;
- (int) removeUserAtPosition:(int)position fromGroup:(int)group;
- (int) removeUser:(id)user fromGroup:(int)group;
- (int) createGroupWithName:(NSString*)groupName;
- (int) deleteGroup:(int)group;
- (int) removeGroup:(id)group;
- (int) findGroupWithName:(NSString*)groupName;
- (id) getDefaultSound;
- (NSString*) getSoundPath;
- (int) setDefaultSound:(NSString*)newSoundPath;
- (int) getGroupPosition:(id)group;

- (int) sortByStatus;
- (int) goOffline;
- (int) getNumberOfGroups;
- (id) getGroupAtPosition:(int)position;
- (NSArray*) getGroupList;
- (id) getUserInGroup:(int)group atPosition:(int)position;
- (NSSet*) contactList;
- (NSSet*) contactListObjects;
- (id) getContactWithUIN:(int)contactUIN;

- (int) createConfigFilesForUIN:(int)uin;
- (int) savePrefsForUIN:(int)uin;

- (void) dealloc;

-(BOOL)displayMessageWhenContactOnline;
-(void)setDisplayMessageWhenContactOnline:(BOOL)value;
-(BOOL)speakMessageWhenContactOnline;
-(void)setSpeakMessageWhenContactOnline:(BOOL)value;
-(BOOL)speakMessageWhenNewMessage;
-(void)setSpeakMessageWhenNewMessage:(BOOL)value;

-(void)setLoginDate:(NSDate*)date;
-(NSDate*)loginDate;

+ (NSArray*) getContactsForFile:(NSString*)path;

@end
