/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMUINPrefs.m
//  iXQ
//
//  Created by starman on Mon Oct 22 2001.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMUINPrefs.h"
#import "SMContactListItem.h"


@implementation SMUINPrefs
    
+ sharedInstance
{

    static id sInst;
    
    if (!sInst) {
        sInst = [[SMUINPrefs alloc] init];
    }
    return sInst;
}

- (id) init
{
    [groupList removeAllObjects];
    [groupList release];
    groupList=[[NSMutableArray alloc] init];
    
    [self createGroupWithName:NSLocalizedString(@"Default", @"")];
    [self createGroupWithName:NSLocalizedString(@"Friends", @"")];
    [self createGroupWithName:NSLocalizedString(@"Co-Workers", @"")];
    [soundPath release];
    soundPath=[[NSString alloc] initWithString:@"/System/Library/Sounds/Glass.aiff"];
    [defaultSound release];
    defaultSound=[[NSSound alloc] initWithContentsOfFile:soundPath byReference:YES];
    
    [password release];
    [nickName release];
    [firstName release];
    [lastName release];
    [theme release];
    listColor=[NSColor whiteColor];
    contactListTextColor=[NSColor blackColor];
    contactListSelectionColor=[NSColor grayColor];
    historySelfColor=[NSColor redColor];
    historyContactColor=[NSColor blueColor];
    
    theme=[[NSString alloc] initWithString:@""];
    password=[[NSString alloc] initWithString:@""];
    nickName=[[NSString alloc] initWithString:@""];
    firstName=[[NSString alloc] initWithString:@""];
    lastName=[[NSString alloc] initWithString:@""];
    status=0;
    oldStatus=0;
    isPasswordSaved=NO;
    
    autoAwayTime=5;
    autoAway=NO;
    defaultClose=NO;
    ignoreMessages=NO;
    hideOffline=NO;
    sortByStatus=NO;
    messagesDate=NO;
    spellChecking=NO;
    utilityContactList=NO;
    visibleToolbar=YES;
    bouncingIcon=NO;
    mute=NO;
    autoConnect=NO;
    alphaLevel=1.0;
    
    // notifications
    displayMessageWhenContactOnline=NO;
    speakMessageWhenContactOnline=NO;
    
    [unknownContacts release];
    unknownContacts=[[NSMutableSet alloc] init];
    contactListWindowFrame=NSMakeRect(200, 500, 120, 150);
    
    return self;
}

- (int) initForPath:(NSString*)path
{
    //NSString* homeDir=NSHomeDirectory();
    NSString* prefsDir=path;
    SMUINPrefs* prefs;
    if ([[NSFileManager defaultManager] fileExistsAtPath:prefsDir])
    {
	prefs=[[NSUnarchiver unarchiveObjectWithFile:prefsDir] retain];
    }
    else
    {
        prefs=[[SMUINPrefs alloc] init];
    }
    
    oldStatus=0;
    
    [self setPassword:[prefs getPassword]];
    [self setNickName:[prefs getNickName]];
    [self setFirstName:[prefs getFirstName]];
    [self setLastName:[prefs getLastName]];

    [groupList removeAllObjects];
    [groupList release];
    groupList = [[NSMutableArray alloc] initWithArray:[prefs getGroups]];
    
    contactListWindowFrame=[prefs getCLWFrame];
    
    
    if (![password isEqualToString:@""])
	isPasswordSaved=YES;
    else 
	isPasswordSaved=NO;
    
    if ([prefs getVersion]>=2)
    {
    	[self setAutoAwayTime:[prefs getAutoAwayTime]];
    	[self setAutoAway:[prefs getAutoAway]];
    	[self setMute:[prefs getMute]];
    	[self setBouncingIcon:[prefs getBouncingIcon]];
    	[self setDefaultClose:[prefs getDefaultClose]];
    	[self setIgnoreMessages:[prefs getIgnoreMessages]];
    	[self setMessagesDate:[prefs getMessagesDate]];
    	[self setSpellChecking:[prefs getSpellChecking]];
    	[self setUtilityContactList:[prefs getUtilityContactList]];
        [self setDefaultSound:[prefs getSoundPath]];
    }
    
    if ([prefs getVersion]>=3)
    {
    	[self setHideOffline:[prefs getHideOffline]];
    	[self setSortByStatus:[prefs getSortByStatus]];
    }
    
    if ([prefs getVersion]>=4)
    {
    	[self setToolbarVisible:[prefs getToolbarVisible]];
    }
    
    if ([prefs getVersion]>=5)
    {
    	[self setTheme:[prefs getTheme]];
        [self setAutoConnect:[prefs getAutoConnect]];
    }
    
    if ([prefs getVersion]>=6)
    {
    	[self setListColor:[prefs getListColor]];
        [self setAlphaLevel:[prefs getAlphaLevel]];
    }
    
    if ([prefs getVersion]>=7)
    {
    	[self setContactListTextColor:[prefs contactListTextColor]];
        [self setContactListSelectionColor:[prefs contactListSelectionColor]];
        [self setHistorySelfColor:[prefs historySelfColor]];
        [self setHistoryContactColor:[prefs historyContactColor]];
    }
    if ([prefs getVersion]>=8)
    {        // notifications
        [self setDisplayMessageWhenContactOnline:[prefs displayMessageWhenContactOnline]];
        [self setSpeakMessageWhenContactOnline:[prefs speakMessageWhenContactOnline]];
    }
    if ([prefs getVersion]>=9)
    {
        [self setStatus:[prefs getStatus]];
    }    
    
    [prefs release];
    return 0;
}

- (int) getVersion
{
    return version;
}

- (int) setDefaultSound:(NSString*) newSoundPath
{
    if ([soundPath isEqualToString:newSoundPath])
        return 0;
    [soundPath release];
    soundPath=[newSoundPath copy];
    [defaultSound release];
    defaultSound=[[NSSound alloc] initWithContentsOfFile:soundPath byReference:YES];
    return 0;
}

- (NSString*) getSoundPath
{
    return soundPath;
}

- (void) dealloc
{
    [groupList removeAllObjects];
    [groupList release];
    [password release];
    [nickName release];
    [firstName release];
    [soundPath release];
    [lastName release];
    [defaultSound release];
    [encryptedPassword release];
    [usersDetails release];
    [unknownContacts release];
}

- (BOOL) isPasswordSaved
{
    return isPasswordSaved;
}

- (NSString*) getPassword
{
    return password;
}

- (int) setStatus:(int)newStatus
{
    oldStatus=status;
    status=newStatus;
    return 0;
}

- (NSColor*) getListColor
{
    return listColor;
}
- (int) setListColor:(NSColor*)newColor
{
    [newColor retain];
    [listColor release];
    listColor=newColor;
    return 0;
}
- (NSColor*) contactListTextColor {
    return contactListTextColor;
}
- (int) setContactListTextColor:(NSColor*)newColor {
    [contactListTextColor release];
    contactListTextColor=[newColor retain];
    return 0;
}
- (NSColor*) contactListSelectionColor {
    return contactListSelectionColor;
}
- (int) setContactListSelectionColor:(NSColor*)newColor {
    [contactListSelectionColor release];
    contactListSelectionColor=[newColor retain];
    return 0;
}
- (NSColor*) historySelfColor {
    return historySelfColor;
}
- (int) setHistorySelfColor:(NSColor*)newColor {
    [historySelfColor release];
    historySelfColor=[newColor retain];
    return 0;
}
- (NSColor*) historyContactColor {
    return historyContactColor;
}
- (int) setHistoryContactColor:(NSColor*)newColor {
    [historyContactColor release];
    historyContactColor=[newColor retain];
    return 0;
}

- (float) getAlphaLevel
{
    return alphaLevel;
}

- (int) setAlphaLevel:(float)newLevel
{
    alphaLevel=newLevel;
    return 0;
}

- (int) getStatus
{
    return status;
}

- (int) getOldStatus
{
    return oldStatus;
}

- (int) setCLWFrame:(NSRect)frame
{
    //NSLog(@"sauvegarde fenetre");
    contactListWindowFrame = frame;
    return 0;
}

- (NSRect) getCLWFrame
{
    //NSLog(@"recup fenetre");
    return contactListWindowFrame;
}

- (BOOL) getAutoAway
{
    return autoAway;
}

- (int) getAutoAwayTime
{
    return autoAwayTime;
}

- (BOOL) getMute
{
    return !mute;
}

- (BOOL) getDefaultClose
{
    return defaultClose;
}

- (BOOL) getIgnoreMessages
{
    return ignoreMessages;
}

- (BOOL) getMessagesDate
{
    return messagesDate;
}

- (BOOL) getSpellChecking
{
    return spellChecking;
}

- (BOOL) getBouncingIcon
{
    return bouncingIcon;
}

- (BOOL) getUtilityContactList
{
    return utilityContactList;
}

- (BOOL) getHideOffline
{
    return hideOffline;
}

- (BOOL) getSortByStatus
{
    return sortByStatus;
}

- (BOOL) getToolbarVisible
{
    return visibleToolbar;
}

- (NSString*) getTheme
{
    return theme;
}

- (int) setTheme:(NSString*)newTheme
{
    if ([newTheme isEqualToString:theme])
        return 0;
    [theme release];
    theme=[newTheme retain];
    return 0;
}

- (int) setToolbarVisible:(BOOL)itIs
{
    visibleToolbar=itIs;
    return 0;
}

- (int) setSortByStatus:(BOOL)itIs
{
    sortByStatus=itIs;
    return 0;
}

- (int) setHideOffline:(BOOL)itIs
{
    hideOffline=itIs;
    return 0;
}

- (int) setAutoAway:(BOOL)itIs
{
    autoAway=itIs;
    return 0;
}

- (int) setMute:(BOOL)itIs
{
    mute=!itIs;
    return 0;
}

- (int) setAutoAwayTime:(int)time
{
    autoAwayTime=time;
    return 0;
}

- (int) setDefaultClose:(BOOL)itIs
{
    defaultClose=itIs;
    return 0;
}

- (int) setBouncingIcon:(BOOL)itIs
{
    bouncingIcon=itIs;
    return 0;
}

- (int) setIgnoreMessages:(BOOL)itIs
{
    ignoreMessages=itIs;
    return 0;
}

- (int) setMessagesDate:(BOOL)itIs
{
    messagesDate=itIs;
    return 0;
}

- (int) setSpellChecking:(BOOL)itIs
{
    spellChecking=itIs;
    return 0;
}

- (int) setUtilityContactList:(BOOL)itIs
{
    utilityContactList=itIs;
    return 0;
}


- (id) getDefaultSound
{
    return defaultSound;
}

- (NSString*) getNickName
{
    return nickName;
}

- (NSString*) getFirstName
{
    return firstName;
}

- (NSString*) getLastName
{
    return lastName;
}

- (id) getGroups
{
    return groupList;
}

- (int) getGroupPosition:(id)group
{
    return [groupList indexOfObject:group];
}

- (int) setAutoConnect:(BOOL)itIs
{
    autoConnect=itIs;
    return 0;
}

- (BOOL) getAutoConnect
{
    return autoConnect;
}

- (int) setIsPasswordSaved:(BOOL)isIt
{
    isPasswordSaved=isIt;
    return 0;
}

- (int) setPassword:(NSString*)newPassword
{
    [password release];
    password=[newPassword copy];
    return 0;
}

- (int) setNickName:(NSString*)newNickName
{
    [nickName release];
    nickName=[newNickName copy];
    return 0;
}

- (int) setFirstName:(NSString*)newFirstName
{
    [firstName release];
    firstName=[newFirstName copy];
    return 0;
}

- (int) setLastName:(NSString*)newLastName
{
    [lastName release];
    lastName=[newLastName copy];
    return 0;
}

- (id) getUnknownContactWithUIN:(int)uin
{
    NSEnumerator *enumerator = [unknownContacts objectEnumerator];
    id value;

    while ((value = [enumerator nextObject])) {
    	if ([value getUIN]==uin)
	{
	    [unknownContacts removeObject:value];
	    return value;
	}
    }
    
    return nil;

}

- (int) addUnknownContact:(id)contact
{
    [contact retain];
    [unknownContacts addObject:contact];
    [contact release];
    return 0;
}

- (int) addUser:(id)user toGroup:(int)group
{
    return [[groupList objectAtIndex:group] addUser:user];
}

- (int) addUser:(id)user toGroup:(int)group atIndex:(int)index
{
    return [[groupList objectAtIndex:group] addUser:user atIndex:index];
}

- (int) removeUserAtPosition:(int)position fromGroup:(int)group
{
    [[groupList objectAtIndex:group] removeUserAtPosition:position];
    return 0;
}

- (int) removeUser:(id)user fromGroup:(int)group
{
    [[groupList objectAtIndex:group] removeUser:user];
    return 0;
}

- (int) createGroupWithName:(NSString*)group
{
    SMContactListItem* newGroup=[[SMContactListItem alloc] initItemWithName:group andUIN:0];
    [groupList addObject:newGroup];
    [newGroup release];
    return 0;
}

- (int) removeGroup:(id)group
{
    [groupList removeObject:group];
    return 0;
}

- (int) findGroupWithName:(NSString*)groupName
{
    int counter=0;
    [groupName retain];
    for (counter=0; counter<[self getNumberOfGroups];counter++)
    {
	if ([[[self getGroupAtPosition:counter] getName] isEqualToString:groupName])
        {
            [groupName release];
	    return counter;
        }
    }

    [groupName release];
    return -1;
}

- (int) deleteGroup:(int)group
{
    [groupList removeObjectAtIndex:group];
    return 0;
}

- (int) getNumberOfGroups
{
    return [groupList count];
}

- (id) getGroupAtPosition:(int)position
{
    return [groupList objectAtIndex:position];
}

- (NSArray*) getGroupList
{
    int counter=0;
    NSMutableArray* groupsList = [NSMutableArray array];
    for (counter=0;counter<[self getNumberOfGroups];counter++)
    {
	[groupsList addObject:[[self getGroupAtPosition:counter] getName]];
    }
    return groupsList;
}

- (int) goOffline
{
    int counter=0;
    for (counter=0;counter<[self getNumberOfGroups];counter++)
    {
	[[self getGroupAtPosition:counter] goOffline];
    }
    
    return 0;
}

- (NSSet*) contactList
{
    NSMutableSet* contacts=[NSMutableSet set];
    int counter=0;
    for (counter=0;counter<[self getNumberOfGroups];counter++)
    {
	[contacts unionSet:[[self getGroupAtPosition:counter] getContacts]];
    }
    return contacts;
}

- (NSSet*) contactListObjects
{
    NSMutableSet* contacts=[NSMutableSet set];
    int counter=0;
    for (counter=0;counter<[self getNumberOfGroups];counter++)
    {
	[contacts unionSet:[[self getGroupAtPosition:counter] getContactsObjects]];
    }
    return contacts;
}

- (int) sortByStatus
{
    int counter=0;
    for (counter=0;counter<[self getNumberOfGroups];counter++)
    {
	[[self getGroupAtPosition:counter] sortByStatus];
    }
    return 0;
}

- (id) getContactWithUIN:(int)contactUIN
{
    NSEnumerator *enumerator = [[self contactListObjects] objectEnumerator];
    id value;

    while ((value = [enumerator nextObject])) {
	if ([value getUIN]==contactUIN)
	    return value;
    }
    return nil;
}

- (id) getUserInGroup:(int)group atPosition:(int)position
{
    return [[groupList objectAtIndex:group] getChildAtPosition:position];
}

- (int) savePrefsForUIN:(int)uin
{
    NSString* homeDir=NSHomeDirectory();
    NSString* prefsDir=[homeDir stringByAppendingPathComponent:[NSString stringWithFormat:@"/Library/Application Support/iXQ/%d/%d.details", uin, uin]];
    [self createConfigFilesForUIN:uin];
    [NSArchiver archiveRootObject:self toFile:prefsDir];
    return 0;
}


- (int) createConfigFilesForUIN:(int)uin
{
    NSString* iXQDir=NSHomeDirectory();
    NSString* uinHome;
    NSString* historyHome;
    
    iXQDir=[iXQDir stringByAppendingPathComponent:@"Library/Application Support/iXQ/"];
    uinHome=[iXQDir stringByAppendingPathComponent:[NSString stringWithFormat:@"%d",uin]];
    historyHome=[uinHome stringByAppendingPathComponent:@"/history/"];
    

    if (![[NSFileManager defaultManager] fileExistsAtPath:iXQDir])
	[[NSFileManager defaultManager] createDirectoryAtPath:iXQDir attributes:nil];
	
    if (![[NSFileManager defaultManager] fileExistsAtPath:uinHome])
	[[NSFileManager defaultManager] createDirectoryAtPath:uinHome attributes:nil];
	
    if (![[NSFileManager defaultManager] fileExistsAtPath:historyHome])
	[[NSFileManager defaultManager] createDirectoryAtPath:historyHome attributes:nil];
	
    return 0;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    NSString* passTMP=password;
    
    version=9;
    [coder encodeValueOfObjCType:@encode(int) at:&version];
    
    if (!isPasswordSaved)
	password=@"";
    encryptedPassword=[password dataUsingEncoding:NSUTF8StringEncoding];
    
    [coder encodeObject:groupList];
    [coder encodeObject:encryptedPassword];
    [coder encodeObject:nickName];
    [coder encodeObject:firstName];
    [coder encodeObject:lastName];
    [coder encodeRect:contactListWindowFrame];
    
    [coder encodeValueOfObjCType:@encode(BOOL) at:&autoAway];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&defaultClose];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&ignoreMessages];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&messagesDate];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&spellChecking];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&utilityContactList];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&bouncingIcon];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&mute];
    [coder encodeValueOfObjCType:@encode(int) at:&autoAwayTime];
    [coder encodeObject:soundPath];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&hideOffline];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&sortByStatus];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&visibleToolbar];
    [coder encodeObject:theme];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&autoConnect];
    [coder encodeObject:listColor];
    [coder encodeValueOfObjCType:@encode(float) at:&alphaLevel];
    [coder encodeObject:contactListTextColor];
    [coder encodeObject:contactListSelectionColor];
    [coder encodeObject:historySelfColor];
    [coder encodeObject:historyContactColor];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&displayMessageWhenContactOnline];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&speakMessageWhenContactOnline];
    [coder encodeValueOfObjCType:@encode(int) at:&status];
    
    password=passTMP;
}

- (id) initWithCoder:(NSCoder *)decoder
{
    [decoder decodeValueOfObjCType:@encode(int) at:&version];
    groupList = [[decoder decodeObject] retain];
    encryptedPassword = [[decoder decodeObject] retain];
    nickName = [[decoder decodeObject] retain];
    firstName = [[decoder decodeObject] retain];
    lastName = [[decoder decodeObject] retain];
    contactListWindowFrame = [decoder decodeRect];

    if (![encryptedPassword isKindOfClass:[NSString class]]) {
        password = [[NSString alloc] initWithData:encryptedPassword encoding: NSUTF8StringEncoding];
    }
    else
	password= encryptedPassword;

    if (version==1)
	return self;
        
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&autoAway];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&defaultClose];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&ignoreMessages];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&messagesDate];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&spellChecking];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&utilityContactList];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&bouncingIcon];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&mute];
    [decoder decodeValueOfObjCType:@encode(int) at:&autoAwayTime];
    soundPath=[[decoder decodeObject] retain];
    
    if (version==2)
        return self;
        
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&hideOffline];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&sortByStatus];
    
    if (version==3)
        return self;
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&visibleToolbar];
    
    if (version==4)
        return self;
    theme = [[decoder decodeObject] retain];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&autoConnect];
    
    if (version==5)
        return self;
    
    listColor = [[decoder decodeObject] retain];
    [decoder decodeValueOfObjCType:@encode(float) at:&alphaLevel];
    
    if (version==6)
        return self;
    
    contactListTextColor = [[decoder decodeObject] retain];
    contactListSelectionColor = [[decoder decodeObject] retain];
    historySelfColor = [[decoder decodeObject] retain];
    historyContactColor = [[decoder decodeObject] retain];
    
    if (version==7)
        return self;
    
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&displayMessageWhenContactOnline];
    [decoder decodeValueOfObjCType:@encode(BOOL) at:&speakMessageWhenContactOnline];
    
    if (version==8)
        return self;
        
     [decoder decodeValueOfObjCType:@encode(int) at:&status];
     
    if (version==9)
        return self;    
           
    return self;
}


-(void)setLoginDate:(NSDate*)date {
    if (date!=loginDate) {
        [loginDate release];
        loginDate=[date retain];
    }
}

-(NSDate*)loginDate {
    return loginDate;
}



// notifications
- (void)setDisplayMessageWhenContactOnline:(BOOL)value
{	displayMessageWhenContactOnline=value;
}
- (BOOL)displayMessageWhenContactOnline
{	return displayMessageWhenContactOnline;
}
- (void)setSpeakMessageWhenContactOnline:(BOOL)value
{	speakMessageWhenContactOnline=value;
}
- (BOOL)speakMessageWhenContactOnline
{	return speakMessageWhenContactOnline;
}
- (void)setSpeakMessageWhenNewMessage:(BOOL)value
{	speakMessageWhenNewMessage=value;
}
- (BOOL)speakMessageWhenNewMessage
{	return speakMessageWhenNewMessage;
}

+ (NSArray*) getContactsForFile:(NSString*)path
{
  [[SMUINPrefs sharedInstance] initForPath:path];
  return [[[SMUINPrefs sharedInstance] contactList] allObjects];
}

@end
