/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//AboutBox.m

#import "SMAboutBox.h"

@implementation SMAboutBox

static SMAboutBox *sharedInstance = nil;

+ (SMAboutBox *)sharedInstance {
    if (!sharedInstance) {
        sharedInstance=[[SMAboutBox alloc] init];
    }
    return sharedInstance;
}

- (id)init
{
         
    self=[super init];
    if (self) {
        NSString *creditsPath;
        NSMutableAttributedString *creditsString;
        NSString *versionString;
        NSString *copyrightString;
        NSDictionary *infoDictionary;
        CFBundleRef localInfoBundle;
        NSDictionary *localInfoDict;
        if (![NSBundle loadNibNamed:@"AboutBox" owner:self])
        {
            // int NSRunCriticalAlertPanel(NSString *title,
            // NSString *msg, NSString *defaultButton,
            // NSString *alternateButton, NSString *otherButton, ...);
            NSLog( @"Failed to load AboutBox.nib" );
            NSBeep();
            return nil;
        }
    
        window = [[NSWindow alloc] initWithContentRect:[[[versionField window] contentView] frame] styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:YES];
        
        [window setContentView:[[[versionField window] contentView] retain]];
            
        [window setHasShadow:YES ];
        // Get the info dictionary (Info.plist)
        infoDictionary = [[NSBundle mainBundle] infoDictionary];
    
        // Get the localized info dictionary (InfoPlist.strings)
        localInfoBundle = CFBundleGetMainBundle();
        localInfoDict = (NSDictionary *)
        CFBundleGetLocalInfoDictionary( localInfoBundle );
            
        // Setup the version field
        versionString = [infoDictionary objectForKey:@"CFBundleVersion"];
        [versionField setStringValue:[NSString stringWithFormat:NSLocalizedString(@"Version %@",@"Version %@"), versionString]];
    
        // Setup our credits
        creditsPath = [[NSBundle mainBundle] pathForResource:@"Credits" ofType:@"rtf"];
        creditsString = [[NSMutableAttributedString alloc] initWithPath:creditsPath documentAttributes:nil];
        
        [creditsField replaceCharactersInRange:NSMakeRange( 0, 0 ) withRTF:[creditsString RTFFromRange: NSMakeRange( 0, [creditsString length] )
                        documentAttributes:nil]];
    
        // Setup the copyright field
        copyrightString = [localInfoDict objectForKey:@"NSHumanReadableCopyright"];
        [copyrightField setStringValue:copyrightString];
        
        // Prepare some scroll info
        maxScrollHeight = [[creditsField string] length];
        
        // Setup the window
        [window setExcludedFromWindowsMenu:YES];
        [window setMenu:nil];
        [window center];
        [window setDelegate:self];

    }
    return self;
}

- (void)showPanel
{    

    if (![[versionField window] isVisible])
    {
        currentPosition = 0;
        restartAtTop = NO;
        startTime = [NSDate timeIntervalSinceReferenceDate] + 3.0;
        [creditsField scrollPoint:NSMakePoint( 0, 0 )];
    }
    // Show the window
  [window makeKeyAndOrderFront:nil];
}

- (void)windowDidBecomeKey:(NSNotification *)notification
{
  scrollTimer = [[NSTimer scheduledTimerWithTimeInterval:2/4 target:self selector:@selector(scrollCredits:) userInfo:nil repeats:YES] retain];
}

- (void)windowDidResignKey:(NSNotification *)notification
{
  [scrollTimer invalidate];
  [scrollTimer release];
  scrollTimer = nil;
}

- (void)scrollCredits:(NSTimer *)timer
{
    if ([NSDate timeIntervalSinceReferenceDate] >= startTime)
    {
        if (restartAtTop)
        {
            // Reset the startTime
            startTime = [NSDate timeIntervalSinceReferenceDate] + 3.0;
            restartAtTop = NO;
            // Set the position
            [creditsField scrollPoint:NSMakePoint( 0, 0 )]; return;
        }

        if (currentPosition >= maxScrollHeight)
        {
            // Reset the startTime
            startTime = [NSDate timeIntervalSinceReferenceDate] + 3.0;
            // Reset the position
            currentPosition = 0;
            restartAtTop = YES;
        }
        else
        {
            // Scroll to the position
            [creditsField scrollPoint:NSMakePoint( 0, currentPosition )];
            // Increment the scroll position
            currentPosition += 0.01;
        }
    }
}

// Get the Stuff of your eyes kidddies - Az on the place ; the king of comments :p
- (IBAction)killPanel:(id)sender
{
    [window orderOut:sender];
}

@end

@implementation NSWindow (SMAboutBox)

- (BOOL)canBecomeKeyWindow
{
  return YES;
}

@end
