/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMContactListController.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMContactListController.h"
#import "SMAccountDocument.h"
#import "NSArray_Extensions.h"
#import "NSOutlineView_Extensions.h"
#import "SMAlert.h"
#import "SMThemeManager.h"


// Conveniences for accessing nodes, or the data in the node.
#define NODE(n)			((SMTreeNode*)n)
#define NODE_DATA(n) 		((SMNodeData*)[NODE((n)) nodeData])
#define SAFENODE(n) 		((SMTreeNode*)((n)?(n):(treeRoot)))

#define DragDropSimplePboardType 	@"MyCustomOutlineViewPboardType"
#define MENU_SIZE 17

@interface SMContactListController (Private)
- (void)setupToolbar;
- (void)_addNewDataToSelection:(SMTreeNode *)newChild;

@end

@implementation SMContactListController



// initialisations and such

-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(id)sender{
    self=[super init];
  if (self) {
	int tFilter = 0;
        accountPreferences=prefs;
        (SMAccountDocument*)document = sender;
        
        [self setShouldCloseDocument:YES];
        
        // outlineview initialisation
        treeRoot = [[SMTreeNode treeFromDictionary: nil] retain];
		if ([accountPreferences hideOfflineContacts]) tFilter+=1;
		if ([accountPreferences sortByName]) tFilter+=2;
		if ([accountPreferences sortByStatus]) tFilter+=4;
		[treeRoot setFilter:tFilter];
    
        [NSBundle loadNibNamed:@"SMContactList" owner:self];
        
        [progress retain];
        [progress removeFromSuperview];
        [outlines setAccountPreferences:accountPreferences];
        [outlines setAccountDocument:document];
        [outlines setContactList:self];
        
        /*for (i=0;i<MENU_SIZE;i++) {
            imageTable[i]=[[NSImage imageNamed:(NSString*)imageNameTable[i]] retain];
        }*/

        pbUsers = [[NSMutableArray alloc] init];
       

        [self initMenus];

        doCloseDocument=NO;
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newContactAdded:) name:@"newContactAdded" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newGroupAdded:) name:@"newGroupAdded" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionDeleted:) name:@"selectionDeleted" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionRenamed:) name:@"selectionRenamed" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageReceived:) name:@"newMessageReceived" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(readMessageFromUIN:) name:@"readMessageFromUIN" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(redisplayContactList:) name:@"redisplayContactList" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(statusChanged:) name:@"statusChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactChangedStatus:) name:@"contactChangedStatus" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"preferencesChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactChanged:) name:@"contactChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loginError:) name:@"loginError" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewMessageArchive:) name:@"viewMessageArchive" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:) name:@"NSApplicationWillResignActiveNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillBecomeActive:) name:@"NSApplicationWillBecomeActiveNotification" object:nil];
    }
    return self;
}

-(void)initMenus {
    // Process the main menu items
    int i;
    [ixqMenu removeAllItems];
    [ixqMenu addItemWithTitle:NSLocalizedString(@"Add Users",@"Add Users") action:@selector(newSearch:) keyEquivalent:@"f"];
    [ixqMenu addItemWithTitle:NSLocalizedString(@"Create Group",@"Create Group") action:@selector(createGroup:) keyEquivalent:@"G"];
    
    // show/hide
    [ixqMenu addItem:[NSMenuItem separatorItem]];
    if ([accountPreferences hideOfflineContacts]) {
            [ixqMenu addItemWithTitle:NSLocalizedString(@"Show offline contacts",@"Show offline contacts") action:@selector(togleVisibleContacts:) keyEquivalent:@""];
            
        } else {
            [ixqMenu addItemWithTitle:NSLocalizedString(@"Hide offline contacts",@"Hide offline contacts") action:@selector(togleVisibleContacts:) keyEquivalent:@""];
               
        }
    [ixqMenu addItem:[NSMenuItem separatorItem]];
    [ixqMenu addItemWithTitle:NSLocalizedString(@"History Manager",@"History Manager") action:@selector(openHistoryManager:) keyEquivalent:@"H"];
    [ixqMenu addItemWithTitle:NSLocalizedString(@"Lists Manager",@"Lists Manager") action:@selector(openListsManager:) keyEquivalent:@"L"];
    [ixqMenu addItemWithTitle:NSLocalizedString(@"Account Preferences",@"Account Preferences") action:@selector(openAccountPreferences:) keyEquivalent:@""];

    // Process the status menu items
    [statusMenu removeAllItems];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Available",@"Available (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Away", @"Away (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"N/A", @"N/A (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Occupied", @"Occupied (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"DND", @"DND (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Free For Chat", @"Free For Chat (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Invisible", @"Invisible (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    [statusMenu addItemWithTitle:NSLocalizedString(@"Offline", @"Offline (status)") action:@selector(changeStatus:) keyEquivalent:@""];
    for (i=0;i<7;i++){
        [[statusMenu itemAtIndex:i] setImage:[[accountPreferences currentTheme] smallImageForStatus:i+1]];
    }
    [[statusMenu itemAtIndex:7] setImage:[[accountPreferences currentTheme] smallImageForStatus:0]];    
    
    [outlines createMenu];
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [treeRoot release];
    [progress release];
    [super dealloc];
}

- (void)awakeFromNib {
    NSTableColumn *tableColumn = nil;
    ImageAndTextCell *imageAndTextCell = nil;
 
    
    
    tableColumn = [outlines tableColumnWithIdentifier: @"nickname"];
    imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
    [imageAndTextCell setEditable: YES];
    [tableColumn setDataCell:imageAndTextCell];    
    
    // Register to geet our custom type, strings, and filenames.... try dragging each into the view!
    [outlines registerForDraggedTypes:[NSArray arrayWithObjects:@"FoundUsers", DragDropSimplePboardType, NSStringPboardType, NSFilenamesPboardType, nil]];
    
    // window initialisations
    [[self window] setTitle:[accountPreferences UIN]];
        
    //outlineview
    //[outlines setDataSource:self];
    //[outlines expandItem:treeRoot];
    
    [outlines setIndentationPerLevel:([outlines indentationPerLevel]/2)];
    [outlines setTarget:self];
    [outlines setDoubleAction:@selector(onDoubleClick)];    
    
    [self redisplay];           
}


-(NSDictionary*)toDict {
    return [treeRoot toDict];
}

-(void)fromDict:(NSDictionary*)dict {
  int tFilter = 0;
  [treeRoot fromDict:dict];
  if ([accountPreferences hideOfflineContacts]) tFilter+=1;
  if ([accountPreferences sortByName]) tFilter+=2;
  if ([accountPreferences sortByStatus]) tFilter+=4;
  [treeRoot setFilter:tFilter];
  [treeRoot updateTempChildren:YES];
//  [treeRoot recursiveSortChildren];
    [outlines reloadData];
    [treeRoot selfExpand:outlines];
    
}

// accessors
-(NSArray*)pbUsers {
    return pbUsers;
}


// actions

- (void)connect {	
    NSRect r=[progress frame];
    [progress setFrameSize:NSMakeSize(([[self window] frame]).size.width-r.origin.x-43 , r.size.height)];
    [[[self window] contentView] addSubview:progress];
    [progress startAnimation:self];
    [document requestLogin];


}

- (void)disconnect {	
    NSDictionary *dict;
    if([accountPreferences currentStatus]==0)
    {
        NSLog(@"Already offline");
    }
    else{
        [progress startAnimation:self];
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
            [accountPreferences UIN], @"uin",
            [NSNumber numberWithInt:0], @"status",
            nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];   
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
            [accountPreferences UIN], @"uin",
            [NSNumber numberWithInt:0], @"status",
            nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"statusChanged" object:self userInfo:dict];        
    }
}


- (IBAction)sheetOK:(id)sender {
    [NSApp endSheet:connectPanel];
    [connectPanel orderOut:nil];
    [progress stopAnimation:self];
    [progress removeFromSuperview];
}

- (IBAction)changeStatus:(id)sender {
    int status;
    status=[sender tag]+1;
    if (status==8) {
        [self disconnect];
    } else if (![accountPreferences currentStatus]) {
        [accountPreferences setSessionLoginStatus:status];
        [self connect];
    } else {
        NSDictionary *dict;
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:status], @"status",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
   }   
}

- (void) newSearch:(id)sender {
    [document newSearch];
    
    /*if (!searchWindow) {
        searchWindow = [[SMSearchWindowController alloc] initWithPreferences:accountPreferences treeRoot:treeRoot];
        [document addWindowController:searchWindow];
    }
    [searchWindow makeKeyAndOrderFront];*/
}

-(void)createGroup:(id)sender {
    NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"newGroupAdded" object:self userInfo:dict];
    
}
-(void)togleVisibleContacts:(id)sender {
    NSDictionary *dict;
    
    [accountPreferences setHideOfflineContacts:![accountPreferences hideOfflineContacts]];
    
    if ([accountPreferences hideOfflineContacts]) {
        [[ixqMenu itemAtIndex:3] setTitle:NSLocalizedString(@"Show offline contacts",@"Show offline contacts")];    
    } else {
        [[ixqMenu itemAtIndex:3] setTitle:NSLocalizedString(@"Hide offline contacts",@"Hide offline contacts")];   
    }
    
    dict=[NSDictionary dictionaryWithObjectsAndKeys:
        [accountPreferences UIN], @"uin",
        nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"preferencesChanged" object:self userInfo:dict];
}


-(void)openAccountPreferences:(id)sender {
     [document addWindowController:accountPreferences];
     [accountPreferences makeKeyAndOrderFront:self];    
}
-(void)openHistoryManager:(id)sender {
     [document openHistoryManager:sender uin:nil];    
}

- (int) onDoubleClick {
    SMTreeNode *item=[outlines itemAtRow:[outlines selectedRow]];
    if ([NODE_DATA(item) isLeaf]) {
        NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NODE_DATA(item) contactUIN], @"contactUIN",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"openMessageWindow" object:self userInfo:dict];
    }
    return 0;
}



// observers


- (void)newGroupAdded:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];


    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [self _addNewDataToSelection: [SMTreeNode treeNodeWithData: [SMNodeData groupDataWithName:@"New Group"]]];
        [document saveContactList];
    }
    
}
- (void)viewMessageArchive:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];


    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [self _addNewDataToSelection: [SMTreeNode treeNodeWithData: [SMNodeData groupDataWithName:@"New Group"]]];
        [document saveContactList];
    }
    
}

- (void)selectionDeleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        
        NSBeginAlertSheet(NSLocalizedString(@"Delete Selection",@"Delete Selection"),NSLocalizedString(@"Cancel",@"Cancel"),NSLocalizedString(@"Ok", @"Ok"),
        nil, 
        [self window], self, nil,@selector(doDelete:returnCode:contextInfo:), nil,NSLocalizedString(@"Are you sure you want to delete the selected contacts ?",@"Are you sure you want to delete the selected contacts ?"));
        
    }
}

- (void)selectionRenamed:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        NSArray *selection = [self selectedNodes];
        if ([selection count]) {
            int row = [outlines rowForItem:[selection objectAtIndex:0]];
            
            [outlines editColumn:0 row:row withEvent:nil select:YES];

            if (![NODE_DATA([selection objectAtIndex:0]) isLeaf])
            {
                int tEditedRow = [outlines editedRow];
                id tCell;
                NSText* tEditor;

                if (tEditedRow != -1)
                {
                    tCell = [[outlines tableColumnWithIdentifier:@"nickname"] dataCellForRow:tEditedRow];
                    tEditor = [[outlines window] fieldEditor:NO forObject:tCell];
                    [tEditor setString:[[NODE_DATA([selection objectAtIndex:0]) contactInfos] preferedNickname]];
                }
            }
        }
        
    }
}

- (void)newMessageReceived:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
	SMMessage *tempMsg = [[notification userInfo] objectForKey:@"message"];
    SMMessage *msg=[[SMMessage alloc] initWithData:[tempMsg data] srcUIN:[tempMsg srcUIN] destUIN:[tempMsg destUIN]];
    [msg setDate:[tempMsg date]];
        if (![document isContactInIgnores:[msg srcUIN]]) {
            SMTreeNode *item=[SMTreeNode childOf:treeRoot forUIN:[msg srcUIN]];
            if (!item)
            {
                NSDictionary *dict;
                SMMessage *msg = [[notification userInfo] objectForKey:@"message"];
                SMUserInfos *user=[[SMUserInfos alloc] initWithUIN:[msg srcUIN] nickname:[msg srcUIN]];
                item = [SMTreeNode treeNodeWithData: [SMNodeData leafDataWithInfos:user]];

                [treeRoot insertChild: item atIndex:0];

                // tell the kernel
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    user, @"SMUserInfos",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"contactAdded" object:self userInfo:dict];
                [user setIsSearchingForInfos:YES];
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    user, @"SMUserInfos",
                    [NSNumber numberWithBool:[accountPreferences autoOpenNewContactDetails]], @"openPanel",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"viewUserDetailsForUIN" object:self userInfo:dict];

                [document requestFullInfoForUIN:[user UIN]];
                [user release];
            }
            if (![NODE_DATA(item) actionStatus])
                [self incrementParentsStatusForItem:item];
            [NODE_DATA(item) setActionStatus:8];
            [outlines reloadData];
        }
    }
}

- (void)readMessageFromUIN:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        NSString *contactUIN = [[notification userInfo] objectForKey:@"contactUIN"];
        if (contactUIN) {
            SMTreeNode *item=[SMTreeNode childOf:treeRoot forUIN:contactUIN];
            if (item) {
                if ([NODE_DATA(item) actionStatus])
                    [self decrementParentsStatusForItem:item];
                [NODE_DATA(item) setActionStatus:0];
                [outlines reloadData];
            }
        }
    }
}

- (void) decrementParentsStatusForItem:(SMTreeNode*)inItem
{
    SMTreeNode* tCurrentNode = [inItem nodeParent];
    while (tCurrentNode != treeRoot && tCurrentNode != nil)
    {
        [NODE_DATA(tCurrentNode) setActionStatus:[NODE_DATA(tCurrentNode) actionStatus]-1];
        if ([NODE_DATA(tCurrentNode) actionStatus] < 0)
            [NODE_DATA(tCurrentNode) setActionStatus:0];
        tCurrentNode = [tCurrentNode nodeParent];
    }
}

- (void) incrementParentsStatusForItem:(SMTreeNode*)inItem
{
    SMTreeNode* tCurrentNode = [inItem nodeParent];
    while (tCurrentNode != treeRoot && tCurrentNode != nil)
    {
        [NODE_DATA(tCurrentNode) setActionStatus:[NODE_DATA(tCurrentNode) actionStatus]+1];
        tCurrentNode = [tCurrentNode nodeParent];
    }
}

-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        //[[self window] makeKeyAndOrderFront:self];
        //[statusMenu setEnabled:YES];
        //[ixqMenu setEnabled:YES];
        
        [progress stopAnimation:self];
        [progress removeFromSuperview];
        /*[infos setStringValue:@"Connected !"];
        [NSApp endSheet:connectPanel];
        [connectPanel orderOut:nil];*/
    }
}

-(void)statusChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
     

    if ([uin isEqualToString:[accountPreferences UIN]]) {
        int i;
        int status = [[[notification userInfo] objectForKey:@"status"] intValue];
        if (status>=0) [statusMenu  setImage:[[accountPreferences currentTheme] smallImageForStatus:status]];
        if (!status) {
            NSArray* contacts=[SMTreeNode allLeafChildrenOf:treeRoot];
            for (i=0;i<[contacts count];i++) {
                [[contacts objectAtIndex:i] setContactStatus:0];
            }
            [treeRoot updateTempChildren:YES];
//			[treeRoot recursiveSortChildren];
            [outlines reloadData];
            [progress stopAnimation:self];
                        
            if (doCloseDocument) {
                [document close];
            }
        }
    }
}

-(void)contactChangedStatus:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
  NSString *ip = [[notification userInfo] objectForKey:@"ipAddress"];
  NSString *localIP = [[notification userInfo] objectForKey:@"localIP"];
  int cookie = [[notification userInfo] objectForKey:@"cookie"];
  int port = [[notification userInfo] objectForKey:@"port"];
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMTreeNode	*contact = [SMTreeNode childOf:treeRoot forUIN:[[notification userInfo] objectForKey:@"contactUIN"]];
        
        if (contact) {
            
            if (![NODE_DATA(contact) contactStatus] && [[[notification userInfo] objectForKey:@"contactStatus"] intValue]) {
            NSDictionary *dict;
            
            
            [[NODE_DATA(contact) contactInfos] setCapabilities:[[notification userInfo] objectForKey:@"capabilities"]];
            if (ip != NULL)
            {
                if (![ip isEqualToString:@"0.0.0.0"])
                {
                    [[NODE_DATA(contact) contactInfos] setIP:ip];
                }
            }
            if ([[[notification userInfo] objectForKey:@"clientversion"] intValue]!= kUnknownVers)
                [[NODE_DATA(contact) contactInfos] setClientVers:[[[notification userInfo] objectForKey:@"clientversion"] intValue]];
        
            if ([[NODE_DATA(contact) contactInfos] autoEncoding]) {
                if ([[NODE_DATA(contact) contactInfos] clientVers]) {
                    if ([[NODE_DATA(contact) contactInfos] isMacClient]) {
                        [[NODE_DATA(contact) contactInfos] setEncoding:@"NSMacOSRomanStringEncoding"];
//                        NSLog([[[NODE_DATA(contact) contactInfos] UIN] stringByAppendingString: @" : selected NSMacOSRomanStringEncoding encoding"]);
                    } else {
                        [[NODE_DATA(contact) contactInfos] setEncoding:@"NSISOLatin1StringEncoding"];
//                        NSLog([[[NODE_DATA(contact) contactInfos] UIN] stringByAppendingString: @" : selected NSISOLatin1StringEncoding encoding"]);
                    }
                }
            }
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NODE_DATA(contact) contactInfos], @"SMUserInfos",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"contactIsOnline" object:self userInfo:dict];
            
            
            }
            
            

            
            
            
            [NODE_DATA(contact) setContactStatus:[[[notification userInfo] objectForKey:@"contactStatus"] intValue]];

		  [[contact nodeParent] updateTempChildren:NO];
        }
//		[[contact nodeParent] recursiveSortChildren];
//		[treeRoot recursiveSortChildren];
        [outlines reloadData];
    }
}

-(void)preferencesChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
	  [self redisplay];
    }
}
-(void)contactChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [outlines reloadData];
    }
}
-(void)loginError:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [self openConnectSheet:NO infos:[[notification userInfo] objectForKey:@"message"]];
    }
}

-(void)redisplayContactList:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [outlines reloadData];
    }
}

- (void)outlineViewItemWillCollapse:(NSNotification *)notification {
    [NODE_DATA([[notification userInfo] objectForKey:@"NSObject"]) setIsExpanded:NO];
   
}

- (void)outlineViewItemWillExpand:(NSNotification *)notification {
    [NODE_DATA([[notification userInfo] objectForKey:@"NSObject"]) setIsExpanded:YES];
   
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification {
    if ([[self selectedNodes] count]==1) {
        if ([NODE_DATA([[self selectedNodes] objectAtIndex:0]) isLeaf]) {
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NODE_DATA([[self selectedNodes] objectAtIndex:0]) contactInfos], @"SMUserInfos",
                [NSNumber numberWithBool:NO], @"openPanel",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"viewUserDetailsForUIN" object:self userInfo:dict];
        }
    }
}


- (void)windowWillClose:(NSNotification *)aNotification {
    [accountPreferences setContactListPosition:[[self window] frame]];
    [document saveDocument:self];
    if ([accountPreferences currentStatus]>0) {
        doCloseDocument=YES;
        [self disconnect];
    }
}

- (void)windowDidResize:(NSNotification *)aNotification {
    if (!inOpening) {
        [self checkScrollBar];
    }
}


    

-(void)applicationWillResignActive:(NSNotification *)notification {
    if ([accountPreferences globalAlphaLevelWhenNotInFront]!=1.0) {
        [[self window] setAlphaValue:[accountPreferences globalAlphaLevelWhenNotInFront]];
    }
}
-(void)applicationWillBecomeActive:(NSNotification *)notification {
    [[self window] setAlphaValue:[accountPreferences contactListAlphaLevel]];
}

// ================================================================
//  NSOutlineView data source methods. (The required ones)
// ================================================================

// Required methods.
- (id)outlineView:(NSOutlineView *)olv child:(int)index ofItem:(id)item {
//    if ([accountPreferences hideOfflineContacts]) {
        return [SAFENODE(item) tempChildAtIndex:index];
//    } else {
//        return [SAFENODE(item) childAtIndex:index];
//    }
}
- (BOOL)outlineView:(NSOutlineView *)olv isItemExpandable:(id)item {
    return [NODE_DATA(item) isGroup];
}
- (int)outlineView:(NSOutlineView *)olv numberOfChildrenOfItem:(id)item {
//    if ([accountPreferences hideOfflineContacts]) {
       return [SAFENODE(item) numberOfTempChildren];
//    } else {
//        return [SAFENODE(item) numberOfChildren];
//    }
}
- (id)outlineView:(NSOutlineView *)olv objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item {
    id objectValue = nil;


    if([[tableColumn identifier] isEqualToString: @"nickname"]) {
    
        float alpha;
	NSString *str;
        NSColor *color;
        str = [[NODE_DATA(item) contactInfos] preferedNickname];
        if ([accountPreferences blendOfflineContacts] && ![document isContactOnline:[[NODE_DATA(item) contactInfos] UIN]]) {
            alpha=0.3;
        } else {
            alpha=1.0;
        }
        if ([NODE_DATA(item) isLeaf]) {
            color=[[accountPreferences contactListContactTextColor] colorWithAlphaComponent:alpha];
        } else {
            color=[accountPreferences contactListGroupTextColor];
            str = [NSString stringWithFormat:@"%@ (%d/%d)", str, [SAFENODE(item) numberOfTempChildren], [SAFENODE(item) numberOfChildren]];
        }
        
        
        objectValue=[[NSAttributedString alloc]initWithString:str attributes:[NSDictionary  dictionaryWithObjectsAndKeys:color,  NSForegroundColorAttributeName, nil]];

    } 
    
    return objectValue;
}

// Optional method: needed to allow editing.
- (void)outlineView:(NSOutlineView *)olv setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item  {
    if([[tableColumn identifier] isEqualToString: @"nickname"]) {
        
	if (![object isEqualToString:@""]) {
            [[NODE_DATA(item) contactInfos] setPreferedNickname: object];
        } else {
            [[NODE_DATA(item) contactInfos] setPreferedNickname: nil];
        }

	  if ([NODE_DATA(item) isLeaf]) {
		NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
		  [accountPreferences UIN], @"uin",
		  [[NODE_DATA(item) contactInfos] UIN], @"contactUIN",
		  nil];
		[[NSNotificationCenter defaultCenter] postNotificationName:@"contactChanged" object:self userInfo:dict];
	  }
	  [document saveContactList];
    }

}

// ================================================================
//  NSOutlineView delegate methods.
// ================================================================

- (BOOL)outlineView:(NSOutlineView *)olv shouldExpandItem:(id)item {
    return [NODE_DATA(item) isExpandable];
}

- (void)outlineView:(NSOutlineView *)olv willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item {    
    if ([[tableColumn identifier] isEqualToString: @"nickname"]) {
	
	if ([NODE_DATA(item) isLeaf]) {
            if ([NODE_DATA(item) actionStatus]) {
                [(ImageAndTextCell*)cell setImage: [[accountPreferences currentTheme] imageNamed:@"message"]];
            } else {
                [(ImageAndTextCell*)cell setImage: [[accountPreferences currentTheme] smallImageForStatus:[NODE_DATA(item) contactStatus]]];
            }
        } else {
            if ([NODE_DATA(item) actionStatus]==0) {
                [(ImageAndTextCell*)cell setImage:  [[accountPreferences currentTheme] imageNamed:@"folder"]];
            } else {
                [(ImageAndTextCell*)cell setImage: [[accountPreferences currentTheme] imageNamed:@"DossierMsg"]];
            }
        }
    } 
}

// ================================================================
//  NSOutlineView data source methods. (dragging related)
// ================================================================

- (BOOL)outlineView:(NSOutlineView *)olv writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard {
    int i;
    draggedNodes = items; 
    
    
    [pboard declareTypes:[NSArray arrayWithObjects: DragDropSimplePboardType, NSStringPboardType, @"account", @"SMUsersInfos", nil] owner:self];
    
    [pbUsers release];
    pbUsers = [[NSMutableArray alloc] initWithCapacity:[items count]];
    
    for (i=0;i<[items count];i++) {
        [pbUsers addObject:[NODE_DATA([items objectAtIndex:i]) contactInfos]];
    }
    
    [pboard setData:[NSData data] forType:DragDropSimplePboardType]; 
    [pboard setString:[accountPreferences UIN] forType:@"account"];
    [pboard setString:@"df" forType:@"SMUsersInfos"];
    
    
    [pboard setString: [draggedNodes description] forType: NSStringPboardType];

    return YES;
}

- (unsigned int)outlineView:(NSOutlineView*)olv validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)childIndex {
    
    SMTreeNode *targetNode = item;
    BOOL targetNodeIsValid = YES;

    if (NO) {
	targetNode = nil;
	childIndex = NSOutlineViewDropOnItemIndex;
    } else {
	BOOL isOnDropTypeProposal = childIndex==NSOutlineViewDropOnItemIndex;
	
	// Refuse if: dropping "on" the view itself unless we have no data in the view.
	if (targetNode==nil ) 
	    targetNodeIsValid = YES;
    
        if ([NODE_DATA(targetNode) isLeaf ] && isOnDropTypeProposal==YES)
	    targetNodeIsValid = NO;
	    
	// Check to make sure we don't allow a node to be inserted into one of its descendants!
	if (targetNodeIsValid && ([info draggingSource]==outlines) && [[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObject: DragDropSimplePboardType]] != nil) {
	    NSArray *_draggedNodes = [[[info draggingSource] dataSource] draggedNodes];
	    targetNodeIsValid = ![targetNode isDescendantOfNodeInArray: _draggedNodes];
	}
        
        if (!([[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObjects: DragDropSimplePboardType, @"FoundUsers", nil]] != nil)) {
	    targetNodeIsValid = NO;
	}
        
        
        
        if (![[[info draggingPasteboard] stringForType:@"account"] isEqualToString:[accountPreferences UIN]]) {
	    targetNodeIsValid = NO;
	}
        
    }
    
    // Set the item and child index in case we computed a retargeted one.
    [outlines setDropItem:targetNode dropChildIndex:childIndex];
    
    return targetNodeIsValid ? NSDragOperationGeneric : NSDragOperationNone;
}

- (void)_performDropOperation:(id <NSDraggingInfo>)info onNode:(TreeNode*)parentNode atIndex:(int)childIndex {
    // Helper method to insert dropped data into the model. 
    NSPasteboard * pboard = [info draggingPasteboard];
    NSMutableArray * itemsToSelect = nil;
    
    // Do the appropriate thing depending on wether the data is DragDropSimplePboardType or NSStringPboardType.
    if ([pboard availableTypeFromArray:[NSArray arrayWithObjects:DragDropSimplePboardType, nil]] != nil) {
        SMContactListController *dragDataSource = [[info draggingSource] dataSource];
        NSArray *_draggedNodes = [TreeNode minimumNodeCoverFromNodesInArray: [dragDataSource draggedNodes]];
        NSEnumerator *draggedNodesEnum = [_draggedNodes objectEnumerator];
        SMTreeNode *_draggedNode = nil, *_draggedNodeParent = nil;
        
	itemsToSelect = [NSMutableArray arrayWithArray:[self selectedNodes]];
	
        if ([accountPreferences hideOfflineContacts]) {
            childIndex=[parentNode indexOfChild: [parentNode tempChildAtIndex:childIndex]];
        }
        
        while ((_draggedNode = [draggedNodesEnum nextObject])) {
            _draggedNodeParent = (SMTreeNode*)[_draggedNode nodeParent];
            if (parentNode==_draggedNodeParent && [parentNode indexOfChild: _draggedNode]<childIndex) childIndex--;
            [_draggedNodeParent removeChild: _draggedNode];
        }
        [parentNode insertChildren: _draggedNodes atIndex: childIndex];
        [document saveContactList];
    } 
    else if ([pboard availableTypeFromArray:[NSArray arrayWithObject: @"FoundUsers"]]) {
        NSMutableArray *users = [[[document searchWindow] pbUsers] retain];        
        int i;
        
        for (i=0;i<[users count];i++) {
            NSDictionary *dict;
            SMUserInfos *user=[[users objectAtIndex:i] copy];
            SMTreeNode *newItem = [SMTreeNode treeNodeWithData: [SMNodeData leafDataWithInfos:user]];
	
            itemsToSelect = [NSMutableArray arrayWithObject: newItem];
            [parentNode insertChild: newItem atIndex:childIndex++];
            
            // tell the kernel
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                user, @"SMUserInfos",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"contactAdded" object:self userInfo:dict];
            [user release];
        }
        //[parent updateChangeCount:NSChangeDone];
        [users release];
        [document saveContactList];
    }
    [treeRoot updateTempChildren:YES];
//    [treeRoot recursiveSortChildren];
    [outlines reloadData];
    //[outlines selectItems: itemsToSelect byExtendingSelection: NO];
}

- (BOOL)outlineView:(NSOutlineView*)olv acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(int)childIndex {
    TreeNode * 		parentNode = nil;
    
    // Determine the parent to insert into and the child index to insert at.
    if ([NODE_DATA(targetItem) isLeaf]) {
        parentNode = (SMTreeNode*)(childIndex==NSOutlineViewDropOnItemIndex ? [targetItem nodeParent] : targetItem);
        childIndex = (childIndex==NSOutlineViewDropOnItemIndex ? [[targetItem nodeParent] indexOfChild: targetItem]+1 : 0);
        if ([NODE_DATA(parentNode) isLeaf]) [NODE_DATA(parentNode) setIsLeaf:NO];
    } else {            
        parentNode = SAFENODE(targetItem);
	childIndex = (childIndex==NSOutlineViewDropOnItemIndex?0:childIndex);
    }
    
    [self _performDropOperation:info onNode:parentNode atIndex:childIndex];
        
    return YES;
}




// utils
-(void)openConnectSheet:(BOOL)autoConnect infos:(NSString*)str {
    [infos setStringValue:str];
    [NSApp     beginSheet: connectPanel
           modalForWindow: [self window]
            modalDelegate: NULL
           didEndSelector: NULL
              contextInfo: nil];
    
}

- (void)doDelete:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo 
{
    if (returnCode != NSAlertDefaultReturn) {
            // Tell all of the selected nodes to remove themselves from the model.
            NSArray *selection = [self selectedNodes];
            [selection makeObjectsPerformSelector: @selector(removeFromParent)];
            [outlines deselectAll:nil];
            [outlines reloadData];
            [document saveContactList];
        }
}


- (NSArray*)draggedNodes   { return draggedNodes; }
- (NSArray *)selectedNodes { return [outlines allSelectedItems]; }

-(NSRect)position {
    return [[self window] frame];
    
}
-(void)setPosition:(NSRect)position {
    [[self window] setFrame:position display:YES];
}

- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    return [accountPreferences UIN];
}

-(void)changeUIN {
    [[self window] setTitle:[accountPreferences UIN]];
}

-(void)redisplay {
  int i;
  int tFilter = 0;
    for (i=0;i<7;i++){
        [[statusMenu itemAtIndex:i] setImage:[[accountPreferences currentTheme] smallImageForStatus:i+1]];
    }
    [[statusMenu itemAtIndex:7] setImage:[[accountPreferences currentTheme] smallImageForStatus:0]];  
    [statusMenu  setImage:[[accountPreferences currentTheme] smallImageForStatus:[accountPreferences currentStatus]]];
    [ixqMenu  setImage:[[accountPreferences currentTheme] imageNamed:@"ixqmenu"]];
    
    [outlines setBackgroundColor:[accountPreferences contactListBGColor]];
    if ([[NSApplication sharedApplication] isActive]) {
        [[self window] setAlphaValue:[accountPreferences contactListAlphaLevel]];
    } else {
        [[self window] setAlphaValue:[accountPreferences globalAlphaLevelWhenNotInFront]];
    }
    if ([accountPreferences contactListStayOnTop]) {
        [[self window] setLevel:NSStatusWindowLevel-1];
    } else {
        [[self window] setLevel:NSNormalWindowLevel];
    }//
	if ([accountPreferences hideOfflineContacts]) tFilter+=1;
	if ([accountPreferences sortByName]) tFilter+=2;
	if ([accountPreferences sortByStatus]) tFilter+=4;
	[treeRoot setFilter:tFilter];
    [treeRoot updateTempChildren:YES];
//    [treeRoot recursiveSortChildren];
	
    [outlines reloadData];
    [self checkScrollBar];
}

-(void)makeKeyAndOrderFront:(id)sender {
    [[self window] makeKeyAndOrderFront:sender];
}

-(void)openAndResize {
    NSRect r = [[self window] frame];
    NSRect oldRect = [[self window] frame];
    
    float diff = NSHeight(r) - 38.0 - 28.0;
    r.origin.y += diff;
    r.size.height = 38.0 + 28.0;
    
    [[self window] setFrame:r display:YES];
    
    [ovSC setHasVerticalScroller:NO];
    
    
    [[self window] makeKeyAndOrderFront:self];
    
    inOpening = YES;
    [[self window] setFrame:oldRect display:YES animate:YES];
    inOpening = NO;
    
    [self checkScrollBar];
}

-(void)sendContacts {
        NSArray* contacts=[SMTreeNode allChildrenOf:treeRoot];
        int i;
        for (i=0;i<[contacts count];i++) {
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [contacts objectAtIndex:i], @"SMUserInfos",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"contactAdded" object:self userInfo:dict];
        }
}

- (BOOL)windowShouldClose:(id)sender {
    
    
    return YES;
}



- (NSArray*)contacts {
    return [SMTreeNode allChildrenOf:treeRoot];
}

-(SMUserInfos*)contactOfUIN:(NSString*)uin {
    return [NODE_DATA([SMTreeNode childOf:treeRoot forUIN:uin]) contactInfos];
}
-(BOOL)isContactOnline:(NSString*)uin {
    return [NODE_DATA([SMTreeNode childOf:treeRoot forUIN:uin])  contactStatus]>0;
}


-(void)checkScrollBar {
    int n=[outlines numberOfRows];
    if (n*19.0 < [ovSC frame].size.height) {
        [ovSC setHasVerticalScroller:NO];
    } else {
        [ovSC setHasVerticalScroller:YES];
    }
}
@end


@implementation SMContactListController (Private)

- (void) setupToolbar {
  /*  // Create a new toolbar instance, and attach it to our document window 
    NSToolbar *toolbar = [[[NSToolbar alloc] initWithIdentifier: MyProjectToolbarIdentifier] autorelease];
    
    // Set up toolbar properties: Allow customization, give a default display mode, and remember state in user defaults 
    [toolbar setAllowsUserCustomization: YES];
    [toolbar setAutosavesConfiguration: YES];
    [toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
    
    // We are the delegate
    [toolbar setDelegate: self];
    
    // Attach the toolbar to the document window 
    [[self window] setToolbar: toolbar];*/
}


- (void)_addNewDataToSelection:(SMTreeNode *)newChild {
   int childIndex = 0, newRow = 0;
    NSArray *selectedNodes = [self selectedNodes];
    SMTreeNode *selectedNode = ([selectedNodes count] ? [selectedNodes objectAtIndex:0] : treeRoot);
    TreeNode *parentNode = nil;

    if ([NODE_DATA(selectedNode) isGroup]) { 
	parentNode = selectedNode; 
	childIndex = 0; 
    }
    else { 
	parentNode = [selectedNode nodeParent]; 
	childIndex = [parentNode indexOfChildIdenticalTo:selectedNode]+1; 
    }
    
    [parentNode insertChild: newChild atIndex: childIndex];
    [outlines reloadData];
    
    newRow = [outlines rowForItem: newChild];
    if (newRow>=0) {
        [outlines selectRow: newRow byExtendingSelection: NO];
    }
    if (![NODE_DATA(newChild) isLeaf]) {
        [outlines editColumn:0 row:newRow withEvent:nil select:YES];

	  {
		int tEditedRow = [outlines editedRow];
		id tCell;
		NSText* tEditor;

		if (tEditedRow != -1)
		{
		  tCell = [[outlines tableColumnWithIdentifier:@"nickname"] dataCellForRow:tEditedRow];
		  tEditor = [[outlines window] fieldEditor:NO forObject:tCell];
		  [tEditor setString:[[NODE_DATA(newChild) contactInfos] preferedNickname]];
		}
	  }
    }
    
    
}


@end


