/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SDOutlineView.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMOutlineView.h"
#import "SMUserInfos.h"
#import "SMTreeNode.h"
#import "SMContactListController.h"
#import "SMAccountDocument.h"

// Conveniences for accessing nodes, or the data in the node.
#define NODE(n)			((SMTreeNode*)n)
#define NODE_DATA(n) 		((SMNodeData*)[NODE((n)) nodeData])
#define SAFENODE(n) 		((SMTreeNode*)((n)?(n):(treeRoot)))

@implementation SMOutlineView


- (void)createMenu {
        listsMenu=[[NSMenu alloc] initWithTitle:@"Lists Menu"];
        
        
        
        contextMenu=[[NSMenu alloc] initWithTitle:@"Contextual Menu"];
        
        [contextMenu addItemWithTitle:NSLocalizedString(@"Message",@"Message") action:@selector(newMessage:) keyEquivalent:@"n"];
        [contextMenu addItemWithTitle:NSLocalizedString(@"Mail",@"Mail") action:@selector(newMail:) keyEquivalent:@"M"];
        [contextMenu addItemWithTitle:NSLocalizedString(@"View Message Archive",@"View Message Archive") action:@selector(openHistory:) keyEquivalent:@""];
        [contextMenu addItem:[NSMenuItem separatorItem]];
        [contextMenu addItemWithTitle:NSLocalizedString(@"User Details",@"User Details") action:@selector(userDetails:) keyEquivalent:@"i"];    
        [contextMenu addItemWithTitle:NSLocalizedString(@"Notify when online",@"Notify when online") action:@selector(notifyWhenOnline:) keyEquivalent:@""];   
        
        
        [contextMenu addItemWithTitle:NSLocalizedString(@"Lists",@"Lists") action:nil keyEquivalent:@""]; 
        [contextMenu addItem:[NSMenuItem separatorItem]];
        [contextMenu addItemWithTitle:NSLocalizedString(@"Rename",@"Rename") action:@selector(renameSelections:) keyEquivalent:@""];   
        [contextMenu addItemWithTitle:NSLocalizedString(@"Delete",@"Delete") action:@selector(deleteSelections:) keyEquivalent:[NSString stringWithFormat:@"%c", 8]];    
        [contextMenu addItemWithTitle:NSLocalizedString(@"New Group",@"New Group") action:@selector(newGroup:) keyEquivalent:@"G"];
        
        
        [contextMenu setSubmenu:listsMenu forItem:[contextMenu itemAtIndex:6]];

    [contextMenu setAutoenablesItems:NO];
    
    [self setMenu:contextMenu];
}
-(void)dealloc {
    [super dealloc];
}

- (NSMenu *)menuForEvent:(NSEvent *)theEvent {
    SMUserInfos *item;

    
    if ([self  numberOfSelectedRows]  <2) {
        NSPoint p=[self convertPoint:[theEvent locationInWindow] fromView:nil];
        [self selectRow:[self rowAtPoint:p]byExtendingSelection:NO];
    }
    

    
    while ([listsMenu numberOfItems]) {
        [listsMenu removeItemAtIndex:0];
    }
    addVis=YES;
    addInv=YES;
    addIgn=YES;
    item=[NODE_DATA([self itemAtRow:[self selectedRow]]) contactInfos];
    
    
    
    
    
    
    if ([[contactList selectedNodes] count]>1) {
        [listsMenu addItemWithTitle:@"Multiple Selection" action:nil keyEquivalent:@""];
        [[contextMenu itemAtIndex:5 ] setEnabled:NO];
    } else {
        // notify
        [[contextMenu itemAtIndex:5 ] setEnabled:YES];
        [[contextMenu itemAtIndex:5 ]setState:[item notifyWhenOnline]];

        
        
        // visibility
        if ([accountDocument isContactInVisibles:[item UIN]]) {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Remove From Visible List",@"Remove From Visible List") action:@selector(changeVisible:) keyEquivalent:@""];
            addVis=NO;
        } else {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Add To Visible List",@"Add To Visible List") action:@selector(changeVisible:) keyEquivalent:@""];
        }
        if ([accountDocument isContactInInvisibles:[item UIN]]) {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Remove From invisible List",@"Remove From invisible List") action:@selector(changeInvisible:) keyEquivalent:@""];
            addInv=NO;
        } else {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Add To invisible List",@"Add To invisible List") action:@selector(changeInvisible:) keyEquivalent:@""];
        }
    if ([accountDocument isContactInIgnores:[item UIN]]) {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Remove From Ignore List",@"Remove From Ignore List") action:@selector(changeIgnore:) keyEquivalent:@""];
            addIgn=NO;
        } else {
            [listsMenu addItemWithTitle:NSLocalizedString(@"Add To Ignore List",@"Add To Ignore List") action:@selector(changeIgnore:) keyEquivalent:@""];
        }

    }
       
        
    
    
    return contextMenu;
    
}

- (void)keyDown:(NSEvent *)theEvent {
    if ([[theEvent charactersIgnoringModifiers] characterAtIndex:0] == 13 || [[theEvent charactersIgnoringModifiers] characterAtIndex:0] == 3) {
        [self newMessage:self];
        return;
    }
    [super keyDown:theEvent];
    return;
}


-(void)deleteSelections:(id)sender {
     NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"selectionDeleted" object:self userInfo:dict];
}
-(void)renameSelections:(id)sender {
     NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"selectionRenamed" object:self userInfo:dict];
}

-(void)newGroup:(id)sender {
     NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"newGroupAdded" object:self userInfo:dict];
}

-(void)notifyWhenOnline:(id)sender {
     SMUserInfos *item=[NODE_DATA([self itemAtRow:[self selectedRow]]) contactInfos];
     [item setNotifyWhenOnline:![(NSMenuItem*)sender state]]; 
     [(NSMenuItem*)sender setState:![(NSMenuItem*)sender state]];
}

-(void)newMessage:(id)sender {
    SMTreeNode *item=[self itemAtRow:[self selectedRow]];
    if ([NODE_DATA(item) isLeaf]) {
        NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NODE_DATA(item) contactUIN], @"contactUIN",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"openMessageWindow" object:self userInfo:dict];
    }
}

-(void)newMail:(id)sender {
 
    NSArray *contacts=[contactList selectedNodes];
    if ([contacts count]) {
        int i;
        NSString *addrs=@"mailto:";
        addrs=[addrs stringByAppendingString:[[NODE_DATA([[contactList selectedNodes] objectAtIndex:0]) contactInfos] mail]?[[NODE_DATA([[contactList selectedNodes] objectAtIndex:0]) contactInfos] mail]:[[NODE_DATA([[contactList selectedNodes] objectAtIndex:0]) contactInfos] preferedNickname]];
        for (i=1;i<[contacts count];i++) {
            addrs=[[addrs 	stringByAppendingString:@";"]
                                stringByAppendingString:[[NODE_DATA([[contactList selectedNodes] objectAtIndex:i]) contactInfos] mail]?[[NODE_DATA([[contactList selectedNodes] objectAtIndex:i]) contactInfos] mail]:[[NODE_DATA([[contactList selectedNodes] objectAtIndex:i]) contactInfos] preferedNickname]];
        }
        
        [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:addrs ]];
    }
}

-(void)openHistory:(id)sender {
     SMTreeNode *item=[self itemAtRow:[self selectedRow]];
     [accountDocument viewMessageArchiveForUIN:[NODE_DATA(item) contactUIN]]; 
}
-(void)userDetails:(id)sender {
     SMTreeNode *item=[self itemAtRow:[self selectedRow]];
    if ([NODE_DATA(item) isLeaf]) {
        NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NODE_DATA(item) contactInfos], @"SMUserInfos",
                [NSNumber numberWithBool:YES], @"openPanel",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"viewUserDetailsForUIN" object:self userInfo:dict];
    }
}

-(void)changeVisible:(id)sender {
     SMTreeNode *item=[self itemAtRow:[self selectedRow]];
     [accountDocument setVisible:[NODE_DATA(item) contactUIN] state:addVis]; 
}
-(void)changeInvisible:(id)sender {
     SMTreeNode *item=[self itemAtRow:[self selectedRow]];
     [accountDocument setInvisible:[NODE_DATA(item) contactUIN] state:addInv];  
}
-(void)changeIgnore:(id)sender {
     SMTreeNode *item=[self itemAtRow:[self selectedRow]];
     [accountDocument setIgnore:[NODE_DATA(item) contactUIN] state:addIgn]; 
}
// accessors
-(SMAccountPreferences*)accountPreferences {
    return accountPreferences;
}
-(void)setAccountPreferences:(SMAccountPreferences*)prefs {
        accountPreferences=prefs;
}
-(void)setAccountDocument:(SMAccountDocument*)doc {
        (SMAccountDocument*)accountDocument=doc;
}
-(void)setContactList:(SMContactListController*)cl{
        (SMContactListController*)contactList=cl;
}

@end
