/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMTreeNode.h
//
//  Copyright (c) 2002 Apple. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "TreeNode.h"
#import "SMUserInfos.h"

@interface SMNodeData : TreeNodeData {
@private
    NSString *name;
    NSString *contactUIN;
    SMUserInfos *contactInfos;
    
    int contactStatus;
    int actionStatus;
    BOOL isLeaf;
    BOOL isExpandable;
    BOOL isExpanded;
    
}

- (id)initWithInfos:(SMUserInfos*)infos isLeaf:(BOOL)leaf;

+ (id)leafDataWithName:(NSString*)name;
    // Convenience method to return a leaf node with its name set.

+ (id)leafDataWithInfos:(SMUserInfos*)infos;
    // Convenience method to return a leaf node with its name set.
    
+ (id)groupDataWithName:(NSString*)name;
    // Convenience method to return a branch node with its name set.
    
- (void)setName:(NSString*)name;
- (NSString*)name;
- (void)setContactUIN:(NSString*)uin;
- (NSString*)contactUIN;
- (void)setContactInfos:(SMUserInfos*)infos;
- (SMUserInfos*)contactInfos;
- (void)setContactStatus:(int)status;
- (int)contactStatus;
- (void)setActionStatus:(int)status;
- (int)actionStatus;
    // Set and get the name.

- (void)setIsLeaf:(BOOL)isLeaf;
- (BOOL)isLeaf;
- (BOOL)isGroup;
    // Set and determine the type of this item (leaf or group).


- (void)setIsExpandable: (BOOL)checked;
- (BOOL)isExpandable;
- (void)setIsExpanded: (BOOL)checked;
- (BOOL)isExpanded;
    // Set and get the expandability of this item.



@end

@interface SMTreeNode : TreeNode {
}

-(void)selfExpand:(NSOutlineView*)ov;

+ (id) treeFromDictionary:(NSDictionary*)dict;
    // This is a convenience method to return a tree root of a tree derived from an input dictionary. (see InitInfo.dict)
    
+(SMTreeNode*)childOf:(SMTreeNode*)item forUIN:(NSString*)uin;

// utils
+(NSArray*)allChildrenOf:(SMTreeNode*)item;
+(NSArray*)allLeafChildrenOf:(SMTreeNode*)item;
@end

