/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#import "SMTreeNode.h"

#define KEY_GROUPNAME	@"Group"
#define KEY_ENTRIES	@"Entries"

#define NODE(n)			((SMTreeNode*)n)
#define NODE_DATA(n) 		((SMNodeData*)[NODE((n)) nodeData])
#define SAFENODE(n) 		((SMTreeNode*)((n)?(n):(nodeData)))

@implementation SMNodeData

- (id)initWithName:(NSString*)str isLeaf:(BOOL)leaf {
    self = [super init];
    if (self==nil) return nil;
    name = [str retain];
    contactInfos=[[SMUserInfos alloc] initWithUIN:@"0" nickname:[str copy]];
    isLeaf = leaf;
    isExpandable = !isLeaf;
    contactStatus=0;
    actionStatus=0;
    return self;
}

- (id)initWithInfos:(SMUserInfos*)infos isLeaf:(BOOL)leaf {
    self = [super init];
    if (self) {
        contactUIN = [[infos UIN] retain];
        name = [[infos nickname] retain];
        contactInfos=[infos retain];
    
        isLeaf = leaf;
        isExpandable = !isLeaf;
        if (!isLeaf) isExpanded = YES;
    }
    return self;
}

+ (id)leafDataWithInfos:(SMUserInfos*)infos {
    // Convenience method to return a leaf node with its name set.
    return [[[SMNodeData alloc] initWithInfos:infos isLeaf:YES] autorelease];
}


+ (id)leafDataWithName:(NSString*)str {
    // Convenience method to return a leaf node with its name set.
    return [[[SMNodeData alloc] initWithName:str isLeaf:YES] autorelease];
}

+ (id)groupDataWithName:(NSString*)str {
    // Convenience method to return a branch node with its name set.
    return [[[SMNodeData alloc] initWithName:str isLeaf:NO] autorelease];
}

- (void)dealloc {
    [contactInfos release];
    [name release];
    [contactUIN release];
    [super dealloc];
}

- (void)setName:(NSString *)str { 
    if (!name || ![name isEqualToString: str]) {
	[name release]; 
	name = [str retain]; 
    }
}

- (NSString*)name { 
    return name; 
}



- (void)setContactUIN:(NSString*)uin {
    if (!contactUIN || ![contactUIN isEqualToString: uin]) {
	[contactUIN release]; 
	contactUIN = [uin retain]; 
    }
}

- (NSString*)contactUIN {
    return contactUIN;
}

- (void)setContactInfos:(SMUserInfos*)infos {
    if (contactInfos!=infos) {
	[contactInfos release]; 
	contactInfos = [infos retain]; 
    }
}

- (SMUserInfos*)contactInfos {
    return contactInfos;
}

- (void)setContactStatus:(int)status {
    contactStatus=status;
}
- (int)contactStatus {
    return contactStatus;
}
- (void)setActionStatus:(int)status {
    actionStatus=status;
}
- (int)actionStatus {
    return actionStatus;
}

- (void)setIsLeaf:(BOOL)leaf { 
    isLeaf = leaf; 
}

- (BOOL)isLeaf { 
    return isLeaf; 
}

- (BOOL)isGroup { 
    return !isLeaf; 
}



- (void)setIsExpandable: (BOOL)expandable {
    isExpandable = expandable;
}

- (BOOL)isExpandable {
    return isExpandable;
}

- (void)setIsExpanded: (BOOL)expandable {
    isExpanded = expandable;
}

- (BOOL)isExpanded {
    return isExpanded;
}


- (NSString*)description { 
    if (isLeaf) {
        return [contactInfos description]; 
    } else {
        return @"(Folder)";
    }
}

- (NSComparisonResult)compare:(TreeNodeData*)other {
    // We want the data to be sorted by name, so we compare [self name] to [other name]
    SMNodeData *_other = other;
    return [name compare: [_other name]];
}

- (NSComparisonResult)compareAlphabetically:(TreeNodeData*)other {
  // We want the data to be sorted by name, so we compare [self name] to [other name]
  SMNodeData *_other = other;
  return [[contactInfos preferedNickname] compare: [[_other contactInfos] preferedNickname] options:NSCaseInsensitiveSearch];
}

- (NSComparisonResult)compareByStatus:(TreeNodeData *)inNodeData
{
  if (!isLeaf && ![inNodeData isLeaf])
  {
	return NSOrderedSame;
  }
  else if (contactStatus == [inNodeData contactStatus])
  {
	return NSOrderedSame;
  }
  else
  {
	if (((contactStatus+7)%8)<(((int)[inNodeData contactStatus]+7)%8))
	{
	  return NSOrderedAscending;
	}
	else
	{
	  return NSOrderedDescending;
	}
  }
}


@end

@implementation SMTreeNode


-(void)selfExpand:(NSOutlineView*)ov {
  int i;
  if ([NODE_DATA(self) isExpanded]) {
	[ov expandItem:self];
  }
    for (i=0; i<[[self children] count]; i++) {
            [NODE([self childAtIndex:i]) selfExpand:ov];
    }
}


- (id) initFromDictionary:(NSDictionary*)dict {
    // This is a convenience init method to return a tree root of a tree derived from an input dictionary.
    // The input dictionary for this example app is InitInfo.dict.  Look at that file to understand the format.
    SMNodeData *data;
    //NSEnumerator *entryEnum;
    //id entry;
    //SMTreeNode *child = nil;
    
    if (dict==nil) {
        data = [[SMNodeData alloc] initWithName:@"OVRoot" isLeaf:NO];
        //[data setIsExpanded:YES];
        self = [super initWithData:data parent:nil children:[NSArray array]];
        if (self==nil) return nil;
    
    } else {
   	/*data = [SMNodeData fromDict: [dict objectForKey: @"Data"]];
        entryEnum = [[dict objectForKey: @"Children"] objectEnumerator];
    
    
    
        self = [super initWithData:data parent:nil children:[NSArray array]];
        if (self==nil) return nil;
    
        while ((entry=[entryEnum nextObject])) {
            child = [SMTreeNode fromDict: entry];
            [self insertChild: child atIndex: [self numberOfChildren]];
        }*/
    }
    return self;
}

+ (id) treeFromDictionary:(NSDictionary*)dict {
    return [[[SMTreeNode alloc] initFromDictionary:dict] autorelease];
}


+(SMTreeNode*)childOf:(SMTreeNode*)item forUIN:(NSString*)uin {
    if (![NODE_DATA(item) isLeaf]) {
        SMTreeNode *result=nil;
        int i=0;
        while (i<[[item children] count] && result==nil) {
            result=[SMTreeNode childOf:NODE([item childAtIndex:i]) forUIN:uin];
            i++;
        }
        return result;
    } else {
        if ([[NODE_DATA(item) contactUIN] isEqualToString:uin]) {
            return item;
        } else {
            return nil;
        }
    }
}

// utils
+(NSArray*)allChildrenOf:(SMTreeNode*)item {
    NSMutableArray *found=[[NSMutableArray alloc] init];
    
    if (![NODE_DATA(item) isLeaf]) {
        int i;
        for (i=0;i<[[item children] count];i++) {
            [found addObjectsFromArray:[SMTreeNode allChildrenOf:NODE([item childAtIndex:i])]];
        }
    } else {
        if ([NODE_DATA(item) contactInfos]) {
            [found addObject:[NODE_DATA(item) contactInfos] ];
        }
    }
    return found;
}
+(NSArray*)allLeafChildrenOf:(SMTreeNode*)item {
    NSMutableArray *found=[[NSMutableArray alloc] init];
    
    if (![NODE_DATA(item) isLeaf]) {
        int i;
        for (i=0;i<[[item children] count];i++) {
            [found addObjectsFromArray:[SMTreeNode allLeafChildrenOf:NODE([item childAtIndex:i])]];
        }
    } else {
        if ([NODE_DATA(item) contactInfos]) {
            [found addObject:NODE_DATA(item) ];
        }
    }
    return found;
}



@end

