/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#import "SMHistoryManagerController.h"
#import "SMAccountPreferences.h"
#import "SMAccountDocument.h"


@implementation SMHistoryManagerController


// init and such
-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(SMAccountDocument*)sender{
    self=[super init];
    if (self) {
        int i;
        accountPreferences=prefs ;
        (SMAccountDocument*)document = sender;
        
        selfNick=[[[document userInfos] nickname] retain];
        selfUIN=[[accountPreferences UIN] retain];
        fileHandles=[[NSMutableDictionary alloc] init];
        contacts=[[NSArray alloc] initWithArray:[(SMAccountDocument*)document contacts]];
        //contacts=[(SMAccountDocument*)document contacts];
        
        
        
        [NSBundle loadNibNamed:@"SMHistoryManagerController" owner:self];
        
        if ([accountPreferences historyManagerPosition].size.height>0) {
            [[self window] setFrameOrigin:[accountPreferences historyManagerPosition].origin ];
        }

        
        if (floor(NSAppKitVersionNumber) > 620) {        
            [progress setStyle:NSProgressIndicatorSpinningStyle];
            [progress setDisplayedWhenStopped:NO]; 
        }
        [progress setIndeterminate:YES];
        [progress setNeedsDisplay:YES];
        [progress startAnimation:self];
        
        
        [scrollView setHasHorizontalScroller:NO];
        [scrollView setHasVerticalScroller:YES];
        [scrollView setContentView:clipView]; 
        
        historyView=[[SMHistoryView alloc] initWithFrame:NSMakeRect(0,0,[clipView frame].size.width,0)];
        
        [historyView setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
        [clipView setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
        [clipView setDocumentView:historyView];
        [historyView setSelfUIN:selfUIN];
        [historyView release];

        
        [togleButton setAction:@selector(togleSize:) target:self];
        
        [bigSplitView retain];
        [scrollView retain];
        [popup retain];
        [popup removeFromSuperview];
        [popup removeAllItems];
        for (i=0;i<[self lastContact];i++) {
            [popup addItemWithTitle:[[contacts objectAtIndex:i] preferedNickname]];
            [[popup itemAtIndex:[popup numberOfItems]-1] setRepresentedObject:[contacts objectAtIndex:i]];
        }
        
        
        [contactList reloadData];
        /*[self selectContact:0];
        [dateList selectRow:[self lastChat] byExtendingSelection:NO];*/
        [self adjustScrollers];
        
        [progress stopAnimation:self];
        
       
        
        [contactList reloadData];
        
        
        // read encodings
        encodings=[[NSDictionary alloc] initWithContentsOfFile:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"/Contents/Resources/encodings.txt"]];
        
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageReceived:) name:@"newMessageReceived" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageSent:) name:@"newMessageSent" object:nil];
        
        
        
    }
    return self;
}

-(void)dealloc {
     
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    //[document removeWindowController:self];
    [selfNick release];
    [selfUIN release];
    [contactNick release];
    [fileHandles release];
    [contacts release];
    [super dealloc];
}

- (void)awakeFromNib {
}

-(void)closeHistories {
    int i;
    NSArray	*fhKeys=[fileHandles allKeys];
    for (i=0;i<[fhKeys count];i++) {
        NSString	*path, *tmpPath;


        NSFileHandle	*fh=(NSFileHandle*)[fileHandles objectForKey:[fhKeys objectAtIndex:i]];
        path=[[[document fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:[[fhKeys objectAtIndex:i] stringByAppendingPathExtension:@"plist"]];
        tmpPath=[path stringByAppendingPathExtension:@"tmp"];
        
       [fh release];
        [fileHandles removeObjectForKey:[fhKeys objectAtIndex:i]];
        
       
        
        [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];
        [[NSFileManager defaultManager] movePath:tmpPath toPath:path handler:nil];
        NSLog([@"Closing history for uin : " stringByAppendingString:[fhKeys objectAtIndex:i]]);
    }

}

-(void)openHistory:(NSString*)contactUIN {
// open global history
        NSFileHandle	*globalHistory;
        NSString *path=[[[[document fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:contactUIN] stringByAppendingPathExtension:@"plist"];
        NSString *tmpPath=[path stringByAppendingPathExtension:@"tmp"];
       	
        if (![[NSFileManager defaultManager] fileExistsAtPath:path]) {
            [[NSFileManager defaultManager] createFileAtPath:path contents:[[NSString stringWithString:@""] dataUsingEncoding:nil] attributes:nil];
            [[NSFileManager defaultManager] copyPath:path toPath:tmpPath handler:nil];
            globalHistory=[[NSFileHandle fileHandleForWritingAtPath:tmpPath] retain];
            [globalHistory writeData:[[NSString stringWithString:@"()"] dataUsingEncoding:nil]];
            [fileHandles setObject:globalHistory forKey:contactUIN];
        } else {
            //int l;
            NSString *test;
            [[NSFileManager defaultManager] copyPath:path toPath:tmpPath handler:nil];
            
            // check if beginning of file is ok
            globalHistory=[NSFileHandle fileHandleForUpdatingAtPath:tmpPath];
            test = [[NSString alloc] initWithData:[globalHistory readDataOfLength:1] encoding:nil];
            if (![test isEqualToString:@"("]) {
                NSData *data;
                [globalHistory seekToFileOffset:0];
                data=[globalHistory readDataToEndOfFile];
                [globalHistory writeData:[[NSString stringWithString:@"("] dataUsingEncoding:nil]];
                [globalHistory writeData:data];
                [globalHistory closeFile];
                [[NSFileManager defaultManager] copyPath:tmpPath toPath:path handler:nil];
            }
            [test release];
            

            globalHistory=[[NSFileHandle fileHandleForWritingAtPath:tmpPath] retain];
            [globalHistory seekToEndOfFile];
            /*l=[globalHistory offsetInFile];
            [globalHistory truncateFileAtOffset:l-1];*/
            [fileHandles setObject:globalHistory forKey:contactUIN];
        }
        [globalHistory seekToEndOfFile];
        
    
    NSLog([@"Opening history for uin : " stringByAppendingString:contactUIN]);
}


// tableview delegates

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {
    if (aTableView==contactList) {
        return [contacts count];
    } else {
        return [keys count]+1;
    }
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {
    NSString *col = [aTableColumn identifier];
    if ([col isEqualToString:@"contacts"]) {
        return [[contacts objectAtIndex:rowIndex] preferedNickname];
    } else if ([col isEqualToString:@"dates"]) {
        if (!rowIndex) {
            return [NSString stringWithFormat:NSLocalizedString(@"All (%d)", @"All (a number)"), [keys count]];
        } else {
            return [keys objectAtIndex:rowIndex-1];
        }
    } else {
        return nil;
    }
}



- (void)tableViewSelectionDidChange:(NSNotification *)notification {
    if ([notification object]==contactList && [contactList selectedRow]>=0 )
    {
        int	selectedDateRow=[dateList selectedRow];
        int row=[contactList selectedRow];
        NSString *uin=[[contacts objectAtIndex:row] UIN];
        
        if (![fileHandles objectForKey:uin]) {
            [self openHistory:uin];
        }

        selectingContact=YES;
        [self selectContact:row];        
        [popup selectItemAtIndex:row];
        selectingContact=NO;
        if ([dateList selectedRow]!=[self lastChat]) {
		  [dateList selectRow:[self lastChat] byExtendingSelection:NO];
        } else {
            [self selectChat:[self lastChat]];
        }
        [dateList scrollRowToVisible:[self lastChat]];
        


    } else {//if ([dateList selectedRow]>=0) {
        if (!selectingContact) {
            int row=[dateList selectedRow];
            [self selectChat:row];
            [dateList scrollRowToVisible:row];
        }
    }

}




// actions


// search field
- (IBAction)search:(id)sender {
    NSString *selected;
    int selectedRow=[dateList selectedRow];
    
    if (selectedRow) {
        selected=[[keys objectAtIndex:selectedRow-1] retain];
    }
    
    if ([[search stringValue] length]) {   
        [historyView setFilter:[search stringValue]];
        [self updateMessagesWithFilter:[search stringValue]];    
    } else {
        [self updateMessagesWithFilter:nil];
    }
    
    if (selectedRow) {
        int newRow=[keys indexOfObject:selected]+1;
        if (newRow>0) {
            [dateList selectRow:newRow byExtendingSelection:NO];
            if (newRow==selectedRow) {
                [self selectChat:newRow];
            }
        } else {
            [dateList selectRow:[self lastChat] byExtendingSelection:NO];
        }
        [selected release];
    } else {
        [dateList selectRow:0 byExtendingSelection:NO];
        [self selectChat:0];
    }
    
    if (![[search stringValue] length]) {
        NSRange range=[[historyView string] rangeOfString:oldSearch options:NSCaseInsensitiveSearch];
        if (range.length>0) {
            
            [historyView scrollRangeToVisible:range];
            [historyView setFilter:NULL];
        }
    } else {
        [oldSearch release];
        oldSearch=[[search stringValue] retain];
    }
    
    
    [[self window] makeFirstResponder:search];
}

// bottom right button
-(IBAction)nextChat:(id)sender {
    int row=[dateList selectedRow];
    if (row<[keys count]) {
        [dateList selectRow:row+1 byExtendingSelection:NO];
    }
}

// bottom right button
-(IBAction)previousChat:(id)sender {
    int row=[dateList selectedRow];
    if (row>0) {
        [dateList selectRow:row-1 byExtendingSelection:NO];
    }
}


// contact selection via popup, when browseer is hidden
-(IBAction)contactMenu:(id)sender {
    [contactList selectRow:[sender indexOfSelectedItem] byExtendingSelection:NO];
}
// top left switch button
-(IBAction)togleSize:(id)sender {
   // vertical resize
   
   if ([sender value]) {
        NSSize ms;
        NSRect r=[smallSplitView frame];
        oldHeight=r.size.height;
        [smallSplitView setFrame:r];
        
        [scrollView removeFromSuperview];
        
        [bigSplitView removeFromSuperview];
        
        r=[bigSplitView frame];
        [scrollView setFrame:r];
        [[[self window] contentView] addSubview:scrollView];
        
        
        [[[self window] contentView] addSubview:popup]; 
        
        r=[[self window] frame];
        r.size.height=r.size.height-oldHeight;
        if (r.size.height<200) {
                r.size.height=200;
        }
        r.origin.y+=oldHeight;
        [[self window] setFrame: r display:YES animate:YES];
        
        
        
        ms=[[self window] minSize];
        ms.height-=oldHeight;
        if (ms.height<200) {
            ms.height=200;
        }
        [[self window] setMinSize:ms];
        

        
        
        
        
    } else {
        NSSize ms;
        NSRect r, svr=[scrollView frame];
        
        
        [popup removeFromSuperview];
        
        
        [scrollView removeFromSuperview];
        r.origin.x=0;
        r.origin.y=0;
        r.size.width=[bigSplitView frame].size.width;
        r.size.height=[bigSplitView frame].size.height-oldHeight-[bigSplitView dividerThickness];
        [scrollView setFrame:r];
        [bigSplitView addSubview:scrollView];
        r=[smallSplitView frame];
        r.size.height=oldHeight;
        [smallSplitView setFrame:r];

        
        [bigSplitView setFrame:svr];
        
         
        [[[self window] contentView] addSubview:bigSplitView];
        
    
        [[self window] display];
        
    
        r=[[self window] frame];
        r.size.height=r.size.height+oldHeight;
        r.origin.y-=oldHeight;
        [[self window] setFrame:r display:YES animate:YES];
        
                
        
        ms=[[self window] minSize];
        ms.height+=oldHeight;
        [[self window] setMinSize:ms];
    }
    
   [self adjustScrollers];
   
   
   /* horiztontal layout,  la Address Book
   
   // pour tester, changer ds MainMenu l'horientation de la splitview
   // donne un redimensionnement manuel un peu plu srapide, 
   // mais un redimensionnement automatique hideux (paske la largeur de l'historyView change)
   // aucune garantie cependant, pas tellement test
   
   
   if ([sender value]) {
        NSSize ms;
        NSRect r=[smallSplitView frame];
        oldHeight=r.size.width;
        [smallSplitView setFrame:r];
        
        [scrollView removeFromSuperview];
        
        [bigSplitView removeFromSuperview];
        
        r=[bigSplitView frame];
        [scrollView setFrame:r];
        [[[self window] contentView] addSubview:scrollView];
        
         
        r=[[self window] frame];
        r.size.width=r.size.width-oldHeight;
        if (r.size.width<430) {
                r.size.width=430;
        }
        [[self window] setFrame: r display:YES animate:YES];
        
        
        
        ms=[[self window] minSize];
        ms.width-=oldHeight;
        if (ms.width<430) {
            ms.width=430;
        }
        [[self window] setMinSize:ms];
        
        
        reduced=YES;
        
        r=[popup frame];
        r.origin.y=[search frame].origin.y-3;
        [popup setFrame:r];
        [[[self window] contentView] addSubview:popup];
        
        
        
    } else {
        NSSize ms;
        NSRect r, svr=[scrollView frame];
        
        
        [popup removeFromSuperview];
        
        
        [scrollView removeFromSuperview];
        r.origin.x=0;
        r.origin.y=0;
        r.size.width=[bigSplitView frame].size.width-oldHeight-[bigSplitView dividerThickness];
        r.size.height=[bigSplitView frame].size.height;
        [scrollView setFrame:r];
        [bigSplitView addSubview:scrollView];
        r=[smallSplitView frame];
        r.size.width=oldHeight;
        [smallSplitView setFrame:r];

        
        [bigSplitView setFrame:svr];
        
         
        [[[self window] contentView] addSubview:bigSplitView];
        
    
        [[self window] display];
        
    
        r=[[self window] frame];
        r.size.width=r.size.width+oldHeight;
        [[self window] setFrame:r display:YES animate:YES];
        
                
        
        ms=[[self window] minSize];
        ms.width+=oldHeight;
        [[self window] setMinSize:ms];
        
        
        
        reduced=NO;
    }*/
}


// observers
-(void)splitViewDidResizeSubviews:(NSNotification*)aNotification {
    if ([aNotification object]==bigSplitView) {
        [self adjustScrollers];
    } else {
        
    }
}
- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)proposedFrameSize {
    return proposedFrameSize;
}
-(void)windowDidResize:(NSNotification*)aNotification {
    [self adjustScrollers];
}

- (void)windowWillClose:(NSNotification *)aNotification {
       
    [accountPreferences setHistoryManagerPosition:[[self window] frame]];
}


- (void)newMessageReceived:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
  if ([uin isEqualToString:[accountPreferences UIN]]) {
	SMMessage *tempMsg = [[notification userInfo] objectForKey:@"message"];
    SMMessage *msg=[[SMMessage alloc] initWithData:[tempMsg data] srcUIN:[tempMsg srcUIN] destUIN:[tempMsg destUIN]];
    [msg setDate:[tempMsg date]];
        [self addMessageToHistory:msg];
        [msg release];
    }
}

- (void)newMessageSent:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMMessage *msg = [[[notification userInfo] objectForKey:@"message"] copy];
        [self addMessageToHistory:msg];
        [msg release];
    }
}

// utils


-(void)selectContactByUIN:(NSString*)uin {
    int i=0;
    while (i<[contacts count]) {
        if ([[[contacts objectAtIndex:i] UIN] isEqualToString:uin]) {
            break;
        }
        i++;
    }
    [contactList selectRow:i byExtendingSelection:NO];
    
    /*if (![fileHandles objectForKey:uin]) {
        [self openHistory:uin];
    }
    [self changeContact:contacts];*/
}


-(void)makeKeyAndOrderFront:(id)sender {
    
    [document addWindowController:self];
    [[self window] makeKeyAndOrderFront:sender];
}


- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    return [NSString stringWithFormat:NSLocalizedString(@"History Manager for UIN : %@",@"History Manager for UIN : %@"),[accountPreferences UIN]];
}

-(void)addMessageToHistory:(SMMessage*)msg {
    SMUserInfos	*contact;
    NSString *contactUIN;
    SMMessage *tmpMsg=[msg copy];
    int encoding;
    int l;
    if ([[tmpMsg srcUIN] isEqualToString:[accountPreferences UIN]]) {
        contactUIN=[tmpMsg destUIN];
    } else {
        contactUIN=[tmpMsg srcUIN];
    }	
    
    contact=[document contactOfUIN:contactUIN];
    encoding=[[[encodings objectForKey:[contact encoding]] objectForKey:@"value"] intValue];

    
            
    
    if (![fileHandles objectForKey:contactUIN]) {
        [self openHistory:contactUIN];
    }

    
    // global history
    l=[(NSFileHandle*)[fileHandles objectForKey:contactUIN] offsetInFile];
            
    [(NSFileHandle*)[fileHandles objectForKey:contactUIN] truncateFileAtOffset:l-1];
    [(NSFileHandle*)[fileHandles objectForKey:contactUIN] seekToEndOfFile];

    [(NSFileHandle*)[fileHandles objectForKey:contactUIN] writeData:[[[[tmpMsg toDictWithEncoding:encoding] description] stringByAppendingString:@", \n)"] dataUsingEncoding:NSISOLatin1StringEncoding allowLossyConversion:YES]];
    
    
        
}

// update data with current context
-(void)updateMessagesWithFilter:(NSString*)text {
    [chats release];
    [keys release];
    chats=[[NSMutableDictionary alloc] init];
    keys=[[NSMutableArray alloc] init];



    if (text) {
        int i;
        for (i=0;i<[allMessages count];i++) {
            NSRange r=[[[allMessages objectAtIndex:i] objectForKey:@"messageText"] rangeOfString:[search stringValue] options:NSCaseInsensitiveSearch];
            NSString *date=[[[allMessages objectAtIndex:i] objectForKey:@"date"] substringWithRange:NSMakeRange(0,8)];
            if (r.length>0) {
                if (![chats objectForKey:date]) {
                    [chats setObject:[NSMutableArray arrayWithCapacity:1] forKey:date];
                    [keys addObject:date];
                }
                [[chats objectForKey:date] addObject:[allMessages objectAtIndex:i]];
            }
        }

        
    } else {
        int i;
        
        for (i=0;i<[allMessages count];i++) {
            NSString *date=[[[allMessages objectAtIndex:i] objectForKey:@"date"] substringWithRange:NSMakeRange(0,8)];

            if (!date) {
                if ([keys count]) {
                    date=[keys objectAtIndex:[keys count]-1];
                } else {
                    date=NSLocalizedString(@"unknown", @"unknown");
                }
            }
            if (![chats objectForKey:date]) {
                [chats setObject:[NSMutableArray arrayWithCapacity:1] forKey:date];
                [keys addObject:date];
            }
            [[chats objectForKey:date] addObject:[allMessages objectAtIndex:i]];
        }
    }
        
    [dateList reloadData];
    
    
        
}

// select contact at index
-(void)selectContact:(int)row
{
    [allMessages release];
    if (row != -1)
    {
        NSString *uin=[[contacts objectAtIndex:row] UIN];
        NSString *path=[[[[document fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:uin] stringByAppendingPathExtension:@"plist"];

        allMessages = [[NSArray alloc] initWithContentsOfFile:path];
    }
    
    if (!allMessages) {
        allMessages = [[NSArray alloc] init];
    }
    
    
    [search setStringValue:@""];
    [historyView setFilter:NULL];
        
    [self updateMessagesWithFilter:nil];

    [self adjustScrollers];
}


// select date at index
-(void)selectChat:(int)row {
    
    NSString *str;
    [progress startAnimation:self];
    
    
    if (!row) {
        int i;
        selectedMessages=[[NSMutableArray alloc] init];
        
        for (i=0;i<[keys count];i++) {
            [selectedMessages addObjectsFromArray:[chats objectForKey:[keys objectAtIndex:i]]];
        }
        
        [historyView setMessages:selectedMessages];
        
        str=[NSString stringWithFormat:NSLocalizedString(@"%d sessions, %d messages", @"number of sessions, number of messages"),[keys count],[selectedMessages count]];
        
    } else if (row>0) {
        currentChat=row-1;
        
        selectedMessages=[chats objectForKey:[keys objectAtIndex:row-1]];
        [historyView setMessages:selectedMessages];
    
        [dateList reloadData];
    	str=[NSString stringWithFormat:NSLocalizedString(@"Session %d of %d, %d messages in this one\n",@"Session %d of %d, %d messages in this one\n"),currentChat+1, [keys count],[selectedMessages count]];
    } else {
        selectedMessages=[[NSArray alloc] init];
        [historyView setMessages:selectedMessages];
        
        str=NSLocalizedString(@"no message found", @"no message found");
    }
    [nbMessages setStringValue:str];
    [nbMessagesB setStringValue:str];
    
    
    [progress stopAnimation:self];
    
}


// ...
-(int)lastContact {
    return [contacts count];
}

// ...
-(int)lastChat {
    return [keys count];
}

// scroll history view to the end
// not used anymore
-(void)goToEnd {
    [clipView scrollToPoint:[clipView constrainScrollPoint:NSMakePoint(0,[historyView frame].size.height)]];
    [scrollView reflectScrolledClipView:clipView];
}

// update scrollers state : visibility, position
-(void)adjustScrollers {
    NSRect r=[contactList frameOfCellAtColumn:0 row:[self lastContact]-1];
    if (r.origin.y < [contactSV frame].size.height-15) {
        [contactSV setHasVerticalScroller:NO];
    } else {
        [contactSV setHasVerticalScroller:YES];
    }
    r=[dateList frameOfCellAtColumn:0 row:[self lastChat]];
    if (r.origin.y < [dateSV frame].size.height-15) {//([keys count]+2)*18.0
        [dateSV setHasVerticalScroller:NO];
    } else {
        [dateSV setHasVerticalScroller:YES];
    }
    [contactList scrollRowToVisible:[contactList selectedRow]];
    [dateList scrollRowToVisible:[dateList selectedRow]];
}

@end
