/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMHistoryView.m
//  iXQ
//
//  Created by Philippe Champeaux on Sat Aug 03 2002.
//  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
//
//
//	main view... 
//	a normal TextView with a personalized background drawn 
//	using SMHistoryBoxes (one for each message)
//
//


#import "SMHistoryView.h"

static	float 	MARGIN=150.0;


@implementation SMHistoryView

// init & such

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        bgColor=[[NSColor colorWithCalibratedRed:0.97 green:0.99 blue:0.99 alpha:1.0] retain];
        
        
        textStorage=[self textStorage];
        textContainer=[self textContainer];
        layoutManager=[self layoutManager];
        
        [self setEditable:NO];
        [self setDrawsBackground:NO];
        [self setRichText:YES];
    }
    return self;
}


-(void)dealloc {
    [bgColor release];
}


// accessors
-(void)setSelfUIN:(NSString*)uin {
    if (uin!=selfUIN) {
        [selfUIN release];
        selfUIN=[uin retain];
    }
}


// utils

// flip coordinates
-(BOOL)isFlipped {
    return YES;
}

// update displayed messages
-(void)setMessages:(NSArray*)m {
    
    if (messages!=m) {
        [messages release];
        messages=[m retain];
    }
   [self redraw];
}

-(void)setFilter:(NSString*)value {
    if (filter!=value) {
        [filter release];
        filter=[value retain];
    }
}

// fill textview and define draw boxes
-(void)redraw {
    
    
    
    NSRect vFrame=NSMakeRect(0,0,[self bounds].size.width, 100000);
    int i;
    
    totalSize=0.0;
    
    [boxes release];
    boxes=[[NSMutableArray alloc] init];
    
    [self setString:@""];
    
       
    // calc text storage
    calcTS = [[NSTextStorage alloc] init];
    
    
    // calc layout manager
    calcLM = [[NSLayoutManager alloc] init];
    [calcTS addLayoutManager:calcLM];
    [calcLM release];
    
    // calc text container
    calcTC = [[NSTextContainer alloc] initWithContainerSize:vFrame.size];
    [calcLM addTextContainer:calcTC];
    [calcTC release];

    
        
    for (i=0;i<[messages count];i++) {
            [self addMessage:[messages objectAtIndex:i]];
    }
   

    if (!totalSize) {
        totalSize=200;
    }
    [self setFrameSize:NSMakeSize([self frame].size.width, totalSize)];
        
    
}

-(void)addMessage:(NSDictionary*)dict {
        NSString *msg=[[dict objectForKey:@"messageText"] stringByAppendingString:@"\n\n"];
        
        NSAttributedString *attrMsg;
        NSRect rect;
        SMHistoryBox *box;
        NSMutableParagraphStyle *style;
        NSDictionary *attr;
        
        BOOL contact;
    

       style=[[NSMutableParagraphStyle alloc] init];
       if (floor(NSAppKitVersionNumber) > 620) {
            attr=[NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor whiteColor], NSForegroundColorAttributeName,
                style,NSParagraphStyleAttributeName,
                    nil];
        } else {
            attr=[NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor blackColor], NSForegroundColorAttributeName,
                style,NSParagraphStyleAttributeName,
                    nil];
        }
            
        if ([[dict objectForKey:@"srcUIN"] isEqualToString:selfUIN]) {
            [style setTailIndent:[self frame].size.width - MARGIN - 5.0];

            contact=2;
        } else {
            [style setFirstLineHeadIndent:MARGIN];
            [style setHeadIndent:MARGIN];
            contact=1;
        }
        
            
        attrMsg=[[NSAttributedString alloc] initWithString:msg attributes:attr];
        [textStorage appendAttributedString:attrMsg];
        
        [calcTS setAttributedString:attrMsg];
        rect=[calcLM boundingRectForGlyphRange:NSMakeRange(0,[attrMsg length]-1) inTextContainer:calcTC];
        
                
        rect.origin.y=totalSize;
        
        rect.size.width=[self frame].size.width;
        rect.origin.x=0;
        if (contact==1) {
            rect.origin.x=MARGIN;
            rect.size.width-=MARGIN;
        } else if (contact==2) {
            rect.origin.x=0;
            rect.size.width-=MARGIN;
        } 
        
        
        box=[[SMHistoryBox alloc] initWithFrame:rect type:contact];
        [boxes addObject:box];
        [box release];
                
        if (filter) {
            //NSRect r=[calcLM boundingRectForGlyphRange:[msg rangeOfString:filter options:NSCaseInsensitiveSearch] inTextContainer:calcTC];
            
            int i;
            unsigned n;
            NSRectArray r=[calcLM rectArrayForGlyphRange:[msg rangeOfString:filter options:NSCaseInsensitiveSearch] withinSelectedGlyphRange:NSMakeRange(NSNotFound,0) inTextContainer:calcTC rectCount:&n];
            for (i=0;i<n;i++) {
                    
                r[i].origin.y+=totalSize+r[i].size.height;
                r[i].size.height=1;
                if (contact==1) {
                    if (r[i].origin.x<155) {
                        r[i].origin.x=155;
                        r[i].size.width-=155;
                    }
                } else if (contact==2) {
                     if (r[i].origin.x+r[i].size.width>[self frame].size.width-150) {
                        r[i].size.width-=150;
                    }
                }
            
                box=[[SMHistoryBox alloc] initWithFrame:r[i] type:3];
                [boxes addObject:box];
                [box release];
            }
        }
            
        totalSize+=rect.size.height + 14;
        
        [attrMsg release];
        [style release];
}


- (void)drawRect:(NSRect)rect {
    int i;
    /*[bgColor set];
    NSRectFill(rect);*/
    
    if (lastWidth!=rect.size.width) {
        [self redraw];
        lastWidth=rect.size.width;
    }
    if (floor(NSAppKitVersionNumber) > 620) {        
        for (i=0;i<[boxes count];i++) {
            [(SMHistoryBox*)[boxes objectAtIndex:i] drawRect:rect];
        } 
    }
    [super drawRect:rect];
}


@end
