/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#import "SMMessageWindowController.h"
#import "SMMessagesManager.h"
#import "SMAccountDocument.h"

static NSString* 	MessageWindowToolbarIdentifier 		= @"Message Window";
static NSString*	StayOnTopToolbarItemIdentifier 	= @"Stay On Top Item Identifier";
static NSString*	AutoCloseToolbarItemIdentifier 	= @"Auto Close Item Identifier";
static NSString*	HistoryToolbarItemIdentifier 	= @"History Item Identifier";

@implementation SMMessageWindowController

//init and such

-(id)initWithManager:(SMMessagesManager*)manager document:(id)doc preferences:(SMAccountPreferences*)prefs contact:(SMUserInfos*)user;{
    self=[super init];
    if (self) {
        int i;
        NSArray *pooledMessages;
    	messagesManager=manager;
        accountPreferences=prefs;
        (SMAccountDocument*)accountDocument=doc;
        contact=[user retain];
        [NSBundle loadNibNamed:@"SMMessageWindow" owner:self];
        
        
        
        
        
        
        [history setBackgroundColor:[accountPreferences messageWindowHistoryBGColor]];
        [message setBackgroundColor:[accountPreferences messageWindowMessageBGColor]];
        [message setTextColor:[accountPreferences messageWindowSelfTextColor]];
        [[self window] setAlphaValue:[accountPreferences messageWindowAlphaLevel]];
        
        if ([contact messageWindowPosition].size.height>0) {
            [[self window] setFrame:[contact messageWindowPosition] display:NO];
        }
        
        
        if ([contact messageWindowStayOnTop]) {
            [[self window] setLevel:NSStatusWindowLevel];
        } else {
            [[self window] setLevel:NSNormalWindowLevel];
        }
        
        if ([accountPreferences checkSpelling]) {
            [message setContinuousSpellCheckingEnabled:YES];
        }
        
        
        if ([[accountPreferences messageWindowHistoryBGColor] alphaComponent]<1.0 ||[[accountPreferences messageWindowHistoryBGColor] alphaComponent]<1.0) {
            [[self window] setAlphaValue:0.99];
        }
        
        
        // read encodings
        encodings=[[NSDictionary alloc] initWithContentsOfFile:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"/Contents/Resources/encodings.txt"]];

        // setup toolbar
        [self setupToolbar];
        
        [self openAndResize];
        
        //[self openHistory];            
        // history : get received messages
        pooledMessages=[messagesManager messagesFromUIN:[contact UIN]];
        for (i=0;i<[pooledMessages count];i++) {
            [self addMessageToHistory:(SMMessage*)[pooledMessages objectAtIndex:i]];
        }

        
        
        // state of send button
        if ([accountDocument connected]) {
            [sendButton setEnabled:YES];
        } else {
            [sendButton setEnabled:NO];
        }
        
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageReceived:) name:@"newMessageReceived" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"preferencesChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactChanged:) name:@"contactChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDisconnected:) name:@"userDisconnected" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:) name:@"NSApplicationWillResignActiveNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillBecomeActive:) name:@"NSApplicationWillBecomeActiveNotification" object:nil];
    }
    return self;
}
-(void)dealloc {
    
    
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [encodings release];
    [contact release];
    [super dealloc];
}





-(void)openHistory {
// open global history
    /*    NSString *path=[[[[accountDocument fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:[contact UIN]] stringByAppendingPathExtension:@"plist"];
        NSString *tmpPath=[path stringByAppendingPathExtension:@"tmp"];
       	
        if (![[NSFileManager defaultManager] fileExistsAtPath:path]) {
            [[NSFileManager defaultManager] createFileAtPath:path contents:[[NSString stringWithString:nil] dataUsingEncoding:nil] attributes:nil];
            [[NSFileManager defaultManager] copyPath:path toPath:tmpPath handler:nil];
            globalHistory=[[NSFileHandle fileHandleForWritingAtPath:tmpPath] retain];
            [globalHistory writeData:[[NSString stringWithString:@"("] dataUsingEncoding:nil]];

        } else {
            int l;
            //NSString *test;
            [[NSFileManager defaultManager] copyPath:path toPath:tmpPath handler:nil];
            
            // check if beginning of file is ok

            
            globalHistory=[[NSFileHandle fileHandleForWritingAtPath:tmpPath] retain];
            [globalHistory seekToEndOfFile];
            l=[globalHistory offsetInFile];
            [globalHistory truncateFileAtOffset:l-1];
        }
        [globalHistory seekToEndOfFile];
*/
}
// actions
-(NSString*) convertNewlinesInString:(NSString*)string
{
    NSMutableString *mutable = [NSMutableString stringWithCapacity:[string length]];
    [mutable setString:string];
    
    if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1)
    {
        NSScanner *scanner;
        int scanLocation;
        
        if ([mutable length] > 0)
        {
            scanner = [NSScanner scannerWithString:mutable];
            while (![scanner isAtEnd])
            {
                [scanner scanUpToString:@"\n" intoString:nil];
                scanLocation = [scanner scanLocation];
                if (scanLocation<[mutable length])
                {
                    [mutable replaceCharactersInRange:NSMakeRange([scanner scanLocation], 1) withString:@"\r\n"];
                    scanner = [NSScanner scannerWithString:mutable];
                    [scanner setScanLocation:scanLocation++];
                }
            }
        }
    }
    else
        [mutable replaceOccurrencesOfString:@"\n" withString:@"\r\n" options:nil range:NSMakeRange(0, [mutable length])];

    return [NSString stringWithString:mutable];
}

- (IBAction)sendMessage:(id)sender {
    if ([[message string] length] && [accountDocument connected])
    {
        NSString *msg = [self convertNewlinesInString:[message string]];
	  SMMessage *newMessage=[[SMMessage alloc] initWithData:[msg dataUsingEncoding:[[[encodings objectForKey:[contact encoding]] objectForKey:@"value"] intValue] allowLossyConversion:YES] srcUIN:[accountPreferences UIN] destUIN:[contact UIN]];
        NSDictionary *dict;
        NSNumber *online=[NSNumber numberWithBool:[accountDocument isContactOnline:[contact UIN]]];
        [self addMessageToHistory:newMessage];
        
//        [newMessage setData:[msg dataUsingEncoding:[[[encodings objectForKey:[contact encoding]] objectForKey:@"value"] intValue] allowLossyConversion:YES]];
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN],@"uin",
                newMessage, @"message",
                [[encodings objectForKey:[contact encoding]] objectForKey:@"value"], @"encoding",
                 online,@"online",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"newMessageSent" object:self userInfo:dict];
        [message setString:@""];
        [messageSize setIntValue:0];
        if ([contact messageWindowAutoClose]) {
            [[self window] performClose:self];
        }
    }
}

- (void) stayOnTopAction:(id) sender {
    if (![contact messageWindowStayOnTop]) {
        [contact setMessageWindowStayOnTop:YES]; 
        [[self window] setLevel:NSStatusWindowLevel];
        [[[toolbar items] objectAtIndex:1] setImage:[NSImage imageNamed:@"ontop2"]];
    } else {
        [contact setMessageWindowStayOnTop:NO];
        [[self window] setLevel:NSNormalWindowLevel];
        [[[toolbar items] objectAtIndex:1]setImage:[NSImage imageNamed:@"ontop"]];
    }
}
- (void) autoCloseAction:(id) sender {
    if (![contact messageWindowAutoClose]) {
        [contact setMessageWindowAutoClose:YES]; 
        [[[toolbar items] objectAtIndex:0]setImage:[NSImage imageNamed:@"autoclose2"]];
    } else {
        [contact setMessageWindowAutoClose:NO]; 
        [[[toolbar items] objectAtIndex:0]setImage:[NSImage imageNamed:@"autoclose"]];
    }
}

- (void) historyAction:(id) sender {
     [accountDocument viewMessageArchiveForUIN:[contact UIN]]; 
}
// ============================================================
// NSToolbar Related Methods
// ============================================================

- (void) setupToolbar {
    toolbar = [[[NSToolbar alloc] initWithIdentifier: MessageWindowToolbarIdentifier] autorelease];
    
    [toolbar setAllowsUserCustomization: NO];
    [toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
    
    [toolbar setDelegate: self];
    
    
    [[self window] setToolbar: toolbar];
    [toolbar setVisible:NO];
}

- (NSToolbarItem *) toolbar: (NSToolbar *)toolbar itemForItemIdentifier: (NSString *) itemIdent willBeInsertedIntoToolbar:(BOOL) willBeInserted {
   NSToolbarItem *toolbarItem = [[[NSToolbarItem alloc] initWithItemIdentifier: itemIdent] autorelease];
    
    if ([itemIdent isEqual: StayOnTopToolbarItemIdentifier]) {
	[toolbarItem setLabel: @"Stay On Top"];
	[toolbarItem setPaletteLabel: @"Stay On Top"];
	[toolbarItem setToolTip: @"Window Stays Above Other Programs"];
	
	/*[toolbarItem setView: stayOnTopOutlet];
	[toolbarItem setMinSize:NSMakeSize(NSWidth([stayOnTopOutlet frame]), NSHeight([stayOnTopOutlet frame]))];
	[toolbarItem setMaxSize:NSMakeSize(NSWidth([stayOnTopOutlet frame]),NSHeight([stayOnTopOutlet frame]))];*/
        
        if ([contact messageWindowStayOnTop]) {
            [toolbarItem setImage:[NSImage imageNamed:@"ontop2"]];
        } else {
            [toolbarItem setImage:[NSImage imageNamed:@"ontop"]];
        }
        
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(stayOnTopAction:)];

    } else if([itemIdent isEqual: AutoCloseToolbarItemIdentifier]) {
	[toolbarItem setLabel: @"Auto Close"];
	[toolbarItem setPaletteLabel: @"Auto Close"];
	[toolbarItem setToolTip: @"Auto Close After Sending"];
	
        if ([contact messageWindowAutoClose]) {
            [toolbarItem setImage:[NSImage imageNamed:@"autoclose2"]];
        } else {
            [toolbarItem setImage:[NSImage imageNamed:@"autoclose"]];
        }
  
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(autoCloseAction:)];
    }  else if([itemIdent isEqual: HistoryToolbarItemIdentifier]) {
	[toolbarItem setLabel: @"Global History"];
	[toolbarItem setPaletteLabel: @"Global History"];
	[toolbarItem setToolTip: @"Show Previous Messages"];
	
        [toolbarItem setImage:[NSImage imageNamed:@"history"]];
        
        
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(historyAction:)];
    } else {
	toolbarItem = nil;
    }
    return toolbarItem;
}

- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar {
    return [NSArray arrayWithObjects:	
            AutoCloseToolbarItemIdentifier,
            StayOnTopToolbarItemIdentifier, 
            HistoryToolbarItemIdentifier,
            nil];
}

- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar {
    return [NSArray arrayWithObjects:	
            AutoCloseToolbarItemIdentifier,
            StayOnTopToolbarItemIdentifier, 
            HistoryToolbarItemIdentifier, 
            nil];
}







// observers

-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [sendButton setEnabled:YES];
    }
}

-(void)userDisconnected:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [sendButton setEnabled:NO];    
    }
}

- (void)newMessageReceived:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
  if ([uin isEqualToString:[accountPreferences UIN]]) {
	SMMessage *tempMsg = [[notification userInfo] objectForKey:@"message"];
    SMMessage *msg=[[SMMessage alloc] initWithData:[tempMsg data] srcUIN:[tempMsg srcUIN] destUIN:[tempMsg destUIN]];
    [msg setDate:[tempMsg date]];
        if ([[msg srcUIN] isEqualToString:[contact UIN]]) {
            [self addMessageToHistory:msg];
        }
        [msg release];
    }
}


- (void)windowDidBecomeKey:(NSNotification *)notification {
    NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN],@"uin",
                [contact UIN], @"contactUIN",
                nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"readMessageFromUIN" object:self userInfo:dict];
    [[self window] makeFirstResponder:message];
}
- (void)windowWillClose:(NSNotification *)notification {
/*    NSString	*path, *tmpPath;
    path=[[[[accountDocument fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:[contact UIN]] stringByAppendingPathExtension:@"plist"];
    tmpPath=[path stringByAppendingPathExtension:@"tmp"];
    
    [globalHistory writeData:[[NSString stringWithString:@")"] dataUsingEncoding:NSISOLatin1StringEncoding]];
    [globalHistory release];
    [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];
    [[NSFileManager defaultManager] movePath:tmpPath toPath:path handler:nil];
    
    [[[self window] toolbar] setVisible:NO];*/
    [contact setMessageWindowPosition:[[self window] frame]];
    //[messagesManager closeWindow:[contact UIN]];
    
}
/*- (void)windowDidMove:(NSNotification *)notification {
    //[contact setMessageWindowPosition:[[self window] frame]];
}
- (void)windowDidResize:(NSNotification *)notification {
    //[contact setMessageWindowPosition:[[self window] frame]];
}*/
-(void)preferencesChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [history setBackgroundColor:[accountPreferences messageWindowHistoryBGColor]];
        [message setBackgroundColor:[accountPreferences messageWindowMessageBGColor]];
        [message setTextColor:[accountPreferences messageWindowSelfTextColor]];
        if ([[NSApplication sharedApplication] isActive]) {
            [[self window] setAlphaValue:[accountPreferences messageWindowAlphaLevel]];
        } else {
            [[self window] setAlphaValue:[accountPreferences globalAlphaLevelWhenNotInFront]];
        }

        if ([accountPreferences checkSpelling]) {
            [message setContinuousSpellCheckingEnabled:YES];
        } else {
            [message setContinuousSpellCheckingEnabled:NO];
        }

        [history setNeedsDisplay:YES];
        [message setNeedsDisplay:YES];
        
    }
}
-(void)contactChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        NSString *contactUIN = [[notification userInfo] objectForKey:@"contactUIN"];
    
        if ([contactUIN isEqualToString:[contact UIN]]) {
            if ([contact messageWindowStayOnTop]) {
                [[self window] setLevel:NSStatusWindowLevel];
            } else {
                [[self window] setLevel:NSNormalWindowLevel];
            }
        }
    }
}


-(void)applicationWillResignActive:(NSNotification *)notification {
    if ([accountPreferences globalAlphaLevelWhenNotInFront]!=1.0) {
        [[self window] setAlphaValue:[accountPreferences globalAlphaLevelWhenNotInFront]];
    }
}
-(void)applicationWillBecomeActive:(NSNotification *)notification {
    [[self window] setAlphaValue:[accountPreferences contactListAlphaLevel]];
}

//utils
-(void)makeKeyAndOrderFront {
    [[self window] makeKeyAndOrderFront:self];
    
    [[self window] makeFirstResponder:message];
}


-(void)openAndResize {
    NSRect r = [[self window] frame];
    NSRect oldRect = [[self window] frame];
    
    float diff = NSHeight(r) - 38.0 - 28.0;
    r.origin.y += diff;
    r.size.height = 38.0 + 28.0;
    
    [[self window] setFrame:r display:YES];
    
    [messageScrollViewOutlet retain];
    [messageScrollViewOutlet removeFromSuperview];
    
    [[self window] makeKeyAndOrderFront:self];
    
    [[self window] setFrame:oldRect display:YES animate:YES];

    float height = [splitViewOutlet frame].size.height;

    r = [messageScrollViewOutlet frame];
    r.size.height = 32.0;
    [messageScrollViewOutlet setFrame:r];
    
    r = [historyScrollViewOutlet frame];
    r.size.height = height - 32.0;
    [historyScrollViewOutlet setFrame:r];
    
    [splitViewOutlet addSubview:messageScrollViewOutlet];
    [messageScrollViewOutlet release];
    
    [splitViewOutlet display];
    
    [[self window] makeFirstResponder:message];
}


-(void)addMessageToHistory:(SMMessage*)msg {
    NSString *stringMsg;
    // session history
    NSString *text=@"[";
    int size;
    int encoding=[[[encodings objectForKey:[contact encoding]] objectForKey:@"value"] intValue];
    if (![[msg srcUIN] isEqualToString:[accountPreferences UIN]]) {
        stringMsg=[msg messageWithEncoding:encoding];
    } else {
        stringMsg=[msg messageWithEncoding:encoding];
    }
    
    text=[text stringByAppendingString:[[msg date] descriptionWithCalendarFormat:@" %I:%M %p " timeZone:nil 
    locale:nil]];
    text=[text stringByAppendingString:@"] : "];
    text=[text stringByAppendingString:stringMsg];
    size=[text length]+1;
    [history replaceCharactersInRange:NSMakeRange([[history textStorage] length], 0) withString:[text stringByAppendingString:@"\n"]];
    
    
    if ([[msg srcUIN] isEqualToString:[accountPreferences UIN]]) {
        [history setTextColor:[accountPreferences messageWindowSelfTextColor] range:NSMakeRange([[history textStorage] length] - size, size)];
    } else {
        [history setTextColor:[accountPreferences messageWindowContactTextColor] range:NSMakeRange([[history textStorage] length] - size, size)];
    }
    
    [history scrollRangeToVisible:NSMakeRange([[history textStorage] length], 0)];
    [[self window] setViewsNeedDisplay:YES];
    
    if ([[self window] isKeyWindow]) {
        NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN],@"uin",
                [contact UIN], @"contactUIN",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"readMessageFromUIN" object:self userInfo:dict];
        [[self window] makeFirstResponder:message];
    }
    
    
    // global history
  /*  [msg setMessageText:stringMsg];
    if (![[msg srcUIN] isEqualToString:[accountPreferences UIN]]) {
        [globalHistory writeData:[[[[msg toDictWithEncoding:encoding] description] stringByAppendingString:@", \n"] dataUsingEncoding:NSISOLatin1StringEncoding]];
    } else {
        [globalHistory writeData:[[[[msg toDictWithEncoding:NSISOLatin1StringEncoding] description] stringByAppendingString:@", \n"] dataUsingEncoding:NSISOLatin1StringEncoding]];
    }*/
    
}

- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    NSString *str=@"";
    str = [[[str stringByAppendingString:[[accountDocument userInfos] nickname]] stringByAppendingString:@" & "] stringByAppendingString:[contact preferedNickname]?[contact preferedNickname]:[contact preferedNickname]];
    return str;
}

- (void)textDidChange:(NSNotification *)aNotification {
    [messageSize setIntValue:[[message string] length]];
}
@end
