/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMMessagesManager.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMMessagesManager.h"
#import "SMAccountPreferences.h"
#import "SMAccountDocument.h"


@implementation SMMessagesManager




// init and such
-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(SMAccountDocument*)document{
    self=[super init];
    
    if (self) {
        accountPreferences=prefs;
        accountDocument=document;
        messages=[[NSMutableArray alloc] init];
        windows=[[NSMutableDictionary alloc] init];
        
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageReceived:) name:@"newMessageReceived" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(openMessageWindow:) name:@"openMessageWindow" object:nil];

    }
    
    return self;
}
-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [windows release];
    [messages release];
    [super dealloc];
}


// accessors


-(NSArray*)messagesFromUIN:(NSString*)uin {

    // to do : ne pas enregistrer les messages si la fentre est ouverte...
    int i;
    NSMutableArray *result=[[NSMutableArray alloc] init];
    
    for (i=0;i<[messages count];i++) {
        if ([[[messages objectAtIndex:i] srcUIN] isEqualToString:uin]) {
            [result addObject:[messages objectAtIndex:i]];
            [messages removeObjectAtIndex:i];
            i--;
        }
    }
    return result;
}


// utils
-(void)closeWindow:(NSString*)uin {
    [windows removeObjectForKey:uin];
}

// observers

- (void)newMessageReceived:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
  if ([uin isEqualToString:[accountPreferences UIN]]) {
	SMMessage *tempMsg = [[notification userInfo] objectForKey:@"message"];
    SMMessage *msg=[[SMMessage alloc] initWithData:[tempMsg data] srcUIN:[tempMsg srcUIN] destUIN:[tempMsg destUIN]];
    [msg setDate:[tempMsg date]];
    
        if (![windows objectForKey:[msg srcUIN]]) {
            [messages addObject:msg];
        }
    }
    
}

- (void)openMessageWindow:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        NSString *contactUIN = [[notification userInfo] objectForKey:@"contactUIN"];
    
        if (![windows objectForKey:contactUIN]) {
            SMMessageWindowController *mwc=[[SMMessageWindowController alloc] initWithManager:self document:accountDocument preferences:accountPreferences contact:[accountDocument contactOfUIN:contactUIN]];
            [accountDocument addWindowController:mwc];
            [windows setObject:mwc forKey:contactUIN];
            [mwc release];
        }
    
   	[[windows objectForKey:contactUIN] makeKeyAndOrderFront];
    }
}

@end


