/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMNotifier.m
//  iXQ
//
//  Created by Philippe Champeaux on Sat Mar 09 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMNotifier.h"
#import "SMAccountDocument.h"

@implementation SMNotifier
// initialisations and such



-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(id)sender{
    self=[super init];
    if (self) {
        accountPreferences=prefs ;
        (SMAccountDocument*)document = sender;
        
        // get sound
        if ([[NSFileManager defaultManager] fileExistsAtPath:@"/System/Library/Sounds/Glass.aiff"])
            msgSound = [[NSSound alloc] initWithContentsOfFile:@"/System/Library/Sounds/Glass.aiff" byReference:YES];
        else
            NSLog(@"/System/Library/Sounds/Glass.aiff not available!");
        
        fadingMessages = [[SMFadingMessage alloc] init];
        
        // get images
        /*applicationIcon=[[accountPreferences currentTheme] imageNamed:@"ApplicationIcon"];
        applicationIconWithMessage=[[accountPreferences currentTheme] imageNamed:@"ApplicationIconWthMessage"];*/
        
        
      
        
        
        
        // register observers
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageReceived:) name:@"newMessageReceived" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidBecomeActive:) name:@"applicationDidBecomeActive" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactIsOnline:) name:@"contactIsOnline" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(statusChanged:) name:@"statusChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"preferencesChanged" object:nil];
    }
    return self;
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    if (fadingMessages) [fadingMessages release];
    [super dealloc];
}

// observers
- (void)newMessageReceived:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
	SMMessage *tempMsg = [[notification userInfo] objectForKey:@"message"];
    SMMessage *msg=[[SMMessage alloc] initWithData:[tempMsg data] srcUIN:[tempMsg srcUIN] destUIN:[tempMsg destUIN]];
    [msg setDate:[tempMsg date]];
        if (![document isContactInIgnores:[msg srcUIN]]) {
            if ([accountPreferences playSound])
			  [msgSound play];
            if (![[NSApplication sharedApplication] isActive]) {
                message=YES;
                [self redrawIcon];
            }
            if ([accountPreferences bounceWhenReceivingMessage]==1) {
                [[NSApplication sharedApplication] requestUserAttention:NSInformationalRequest];
            } else if ([accountPreferences bounceWhenReceivingMessage]==2) {
                [[NSApplication sharedApplication] requestUserAttention:NSCriticalRequest];
            }
        }
    }
}
- (void)applicationDidBecomeActive:(NSNotification *)notification {
    message=NO;
    [self redrawIcon];

}

-(void)contactIsOnline:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
         if (fabs([[NSDate date] timeIntervalSinceDate:[document loginDate]])>15 && [[[notification userInfo] objectForKey:@"SMUserInfos"] notifyWhenOnline]) {
            NSString *str;
            NSString *nick = 
            str = [NSString stringWithFormat:NSLocalizedString(@"User %@ is online",@"User %@ is online"), [[[notification userInfo] objectForKey:@"SMUserInfos"] preferedNickname]];
            
            [fadingMessages newMessage:str];
        }
    }
}

-(void)statusChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if ([[[NSDocumentController sharedDocumentController] documents] count]<2) {
            status = [[[notification userInfo] objectForKey:@"status"] intValue];
            [self redrawIcon];
        }
    }
}


-(void)redrawIcon {
    if (status) {
        if (message) {
            [[NSApplication sharedApplication] setApplicationIconImage:[self addImage:[[accountPreferences currentTheme] bigImageForStatus:status] to:[[accountPreferences currentTheme] imageNamed:@"ApplicationIconWithMessage"]]];
        } else {
            [[NSApplication sharedApplication] setApplicationIconImage:[self addImage:[[accountPreferences currentTheme] bigImageForStatus:status] to:[[accountPreferences currentTheme] imageNamed:@"ApplicationIcon"]]];
        }
    } else {
        if (message) {
            [[NSApplication sharedApplication] setApplicationIconImage:[[accountPreferences currentTheme] imageNamed:@"ApplicationIconWithMessage"]];
        } else {
            [[NSApplication sharedApplication] setApplicationIconImage:[[accountPreferences currentTheme] imageNamed:@"ApplicationIcon"]];
        }
    }
}
-(NSImage*)addImage:(NSImage*)image2 to:(NSImage*)image1 {
    NSImage *result = [[NSImage alloc] initWithSize:[image1 size]] ;

	[result lockFocus];
        
        if (image1) {
            [image1 compositeToPoint:NSMakePoint(0,0) operation:NSCompositeSourceOver];
        }

	if (image2) {
            //[image2 drawInRect:NSMakeRect(5, 5,45,45) fromRect:NSMakeRect(0, 0, 20, 20) operation:NSCompositeSourceOver fraction:1.0];
            [image2 compositeToPoint:NSMakePoint(5,5) operation:NSCompositeSourceOver];
        }
        
	[result unlockFocus];

	return [result autorelease];

}

-(void)preferencesChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [self redrawIcon];
    }
}
@end
