/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMSearchWindowController.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMSearchWindowController.h"
#import "SMAccountDocument.h"
#import "SMUserInfos.h"

@implementation SMSearchWindowController


//init and such

-(id)initWithPreferences:(SMAccountPreferences*)prefs document:(id)doc {
    self=[super init];
    if (self) {
        int i;
        NSString *applPath;
        NSArray	*keys;
        accountPreferences=prefs;
       (SMAccountDocument*)accountDocument=doc;
        [accountDocument addWindowController:self];
    
        foundUsers = [[NSMutableArray alloc] init];
        pbUsers = [[NSMutableArray alloc] init];
       
        
        [NSBundle loadNibNamed:@"SMSearchWindow" owner:self];
        
        if ([accountPreferences searchWindowPosition].size.height>0) {
            [[self window] setFrame:[accountPreferences searchWindowPosition] display:NO];
        }
                
        // init langages
        applPath=[[NSBundle mainBundle] bundlePath];
        languages=[NSDictionary dictionaryWithContentsOfFile:[applPath stringByAppendingPathComponent:@"/Contents/Resources/languages.txt"]];
        keys=[[languages allKeys] sortedArrayUsingSelector:@selector(compare:)];
        for (i=0;i<[keys count];i++) {
            [languageOutlet addItemWithObjectValue:[keys objectAtIndex:i]];
        }
        
        // init countries
        applPath=[[NSBundle mainBundle] bundlePath];
        countries=[NSDictionary dictionaryWithContentsOfFile:[applPath stringByAppendingPathComponent:@"/Contents/Resources/countries.txt"]];
        keys=[[countries allKeys] sortedArrayUsingSelector:@selector(compare:)];
        for (i=0;i<[keys count];i++) {
            [countryOutlet addItemWithObjectValue:[keys objectAtIndex:i]];
        }        
        
        
        // state of send button
        if ([accountDocument connected]) {
            [searchButton setEnabled:YES];
        } else {
            [searchButton setEnabled:NO];
        }
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(openSearchWindow:) name:@"openSearchWindow" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userFound:) name:@"userFound" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDisconnected:) name:@"userDisconnected" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactAdded:) name:@"contactAdded" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(serverSideListReceived:) name:@"serverSideListReceived" object:nil];
		[IBImportB_ setTitle:NSLocalizedString(@"Choose file...", @"")];
        [[self window] makeKeyAndOrderFront:self];
    }
    return self;
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    //[accountDocument removeWindowController:self];
    [super dealloc];
}

- (void)awakeFromNib {
        
}

// accessors
-(NSArray*)pbUsers {
    return pbUsers;
}
// actions
-(void)search:(id)sender {
    SMUserInfos *contact = [[SMUserInfos alloc] initWithUIN:[UINOutlet stringValue] nickname:[nickNameOutlet stringValue]];
    NSDictionary *dict;
    int min_age, max_age;
    [contact setFirstName:[firstNameOutlet stringValue]];
    [contact setLastName:[lastNameOutlet stringValue]];
    [contact setMail:[mailOutlet stringValue]];
    switch ([ageOutlet indexOfSelectedItem])
    {
	case 1:
	    min_age = 18;
	    max_age = 22;
	    break;
	case 2:
	    min_age = 23;
	    max_age = 29;
	    break;
	case 3:
	    min_age = 30;
	    max_age = 39;
	    break;
	case 4:
	    min_age = 40;
	    max_age = 49;
	    break;
	case 5:
	    min_age = 50;
	    max_age = 59;
	    break;
	case 6:
	    min_age = 60;
	    max_age = 0x2710;
	    break;
	default:
	    min_age = max_age = 0;
	    break;
    }
    [contact setMinimalAge:min_age];
    [contact setMaximalAge:max_age];    
    [contact setSex:[sexeOutlet indexOfSelectedItem]];
    [contact setLanguage:([languageOutlet indexOfSelectedItem]+1)];
    [contact setCountry:([countryOutlet indexOfSelectedItem]+1)];
    [contact setCity:[cityOutlet stringValue]];
    [contact setState:[stateOutlet stringValue]];
    [contact setOnlineOnly:[onlyOnlineOutlet state]];
    [contact setFromUIN:[accountPreferences UIN]];

    dict=[NSDictionary dictionaryWithObjectsAndKeys:
    	[accountPreferences UIN], @"uin",
        contact, @"SMUserInfos",
        nil];

    [foundUsers removeAllObjects];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];
    [contact release];
    [tabView selectLastTabViewItem:self];
}

- (IBAction)IBImportTypeChanged:(id)sender
{
  if ([programsOutlet selectedRow] == 4)
  {
	[IBImportB_ setTitle:NSLocalizedString(@"Import", @"")];
  }
  else
  {
	[IBImportB_ setTitle:NSLocalizedString(@"Choose file...", @"")];
  }
}

- (IBAction)import:(id)sender {
    int result;
    NSString *title;
    NSString *prompt;
    NSArray *fileTypes;
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];

    [foundUsers removeAllObjects];
    
    switch ([programsOutlet selectedRow])
    {
        case 0 :
        {
            title=NSLocalizedString(@"Mirabilis' ICQ uins list import (awaited filename : xxxxxxx.user.idx)",@"Mirabilis' ICQ uins list import (awaited filename : xxxxxxx.user.idx)");
            prompt=NSLocalizedString(@"Please select the file containing your contact list (yourUIN.user.idx)",@"Please select the file containing your contact list (yourUIN.user.idx)");
            fileTypes=[NSArray arrayWithObjects:@"idx", nil];
            break;
        }
        case 1 :
        {
            title=NSLocalizedString(@"Please select the file containing your contact list (yourUIN.user.idx)",@"Please select the file containing your contact list (yourUIN.user.idx)");
            prompt=NSLocalizedString(@"Please select the file containing your contact list (FireConfiguration.plist)",@"Please select the file containing your contact list (FireConfiguration.plist)");
            fileTypes=[NSArray arrayWithObjects:@"plist", nil];
            break;
        }
        case 2 :
        {
            title=NSLocalizedString(@"Gerry's ICQ uins list import",@"Gerry's ICQ uins list import");
            prompt=NSLocalizedString(@"Please select the file containing your contact list",@"Please select the file containing your contact list");
            fileTypes=nil;
            break;
        }
        case 3 :
        {
            title=NSLocalizedString(@"iXQ v1 uins list import",@"iXQ v1 uins list import");
            prompt=NSLocalizedString(@"Please select the file containing your contact list",@"Please select the file containing your contact list");
            fileTypes=nil; 
            break;
        }
        case 5 :
        {
            title=NSLocalizedString(@"Proteus uins list import",@"Proteus uins list import");
            prompt=NSLocalizedString(@"Please select the file containing your contact list",@"Please select the file containing your contact list");
            fileTypes=nil;
            break;
        }
		case 4 :
		{
		  NSDictionary *dict;
		  dict=[NSDictionary dictionaryWithObjectsAndKeys:
			[accountPreferences UIN], @"uin",
            nil];
		  [[NSNotificationCenter defaultCenter] postNotificationName:@"importServerSideList" object:self userInfo:dict];
		  [tabView selectLastTabViewItem:self];
		  return;
		}
        
    }
    
    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setTitle:title];
    // dialogue d'ouverture
    result = [oPanel runModalForDirectory:NSHomeDirectory() file:nil types:fileTypes];
    if (result == NSOKButton) {
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            NSString *aFile = [filesToOpen objectAtIndex:i];
            switch ([programsOutlet selectedRow]) {
                case 0 :
                    {	[self importFromICQWithPath:aFile];
                        break;
                    }
                case 1 :
                    {	[self importFromFireWithPath:aFile];
                        break;
                    }
                case 2 :
                    {	[self importFromGerryWithPath:aFile];
                        break;
                    }
                case 3 :
                    {	[self importFromV1WithPath:aFile];
                        break;  
                    }
                case 4 :
                    {	[self importFromProteusWithPath:aFile];
                        break;
                    }
	    
            }
            [pathOutlet setStringValue:aFile];
        }
    }
    [tabView selectLastTabViewItem:self];
}
// observers
- (void)openSearchWindow:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [[self window] makeKeyAndOrderFront:self];
    }
    
    
}
- (void)contactAdded:(NSNotification*)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        int i;
        for (i=0;i<[foundUsers count];i++) {
            if ([[[foundUsers objectAtIndex:i] UIN] isEqualToString:[[[notification userInfo] objectForKey:@"SMUserInfos"] UIN]]) {
                [foundUsers removeObjectAtIndex:i];
                [tableView reloadData];
                return;
            }
        }
    }
        
}

- (void)userFound:(NSNotification *)notification {
  NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
//	SMUserInfos *tempUser=[[notification userInfo] objectForKey:@"SMUserInfos"];
	SMUserInfos *user=[[SMUserInfos alloc] initFromDict:[[notification userInfo] objectForKey:@"SMUserInfos"]];
	  if (![accountDocument contactOfUIN:[user UIN]]) {
            [foundUsers addObject:user];
            [tableView reloadData];
            [user release];
        }
    }    
}

//serverSideListReceived
- (void)serverSideListReceived:(NSNotification *)notification {
  NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
    NSArray* contacts = [[notification userInfo] objectForKey:@"UINList"];
    int i;
    for (i=0;i<[contacts count];i++) {
	  NSString *uin=[contacts objectAtIndex:i];
	  SMUserInfos *contact;
	  NSDictionary *dict;
	  if (![accountDocument contactOfUIN:uin])
	  {
		contact = [[SMUserInfos alloc] initWithUIN:uin nickname:uin];
		dict=[NSDictionary dictionaryWithObjectsAndKeys:
		  [accountPreferences UIN], @"uin",
		  contact, @"SMUserInfos",
		  nil];
		[[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];
	  }
	}
  }
}

-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [searchButton setEnabled:YES];
    }
}

-(void)userDisconnected:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [searchButton setEnabled:NO];    
    }
}

- (void)windowWillClose:(NSNotification *)aNotification {
    [accountPreferences setSearchWindowPosition:[[self window] frame]];
}

// table view delegates

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {
    return [foundUsers count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {
    NSString *col = [aTableColumn identifier];
    if ([col isEqualToString:@"uin"]) {
        return [[foundUsers objectAtIndex:rowIndex] UIN];
    } else if ([col isEqualToString:@"nickname"]) {
        return [[foundUsers objectAtIndex:rowIndex] nickname];
    } else if ([col isEqualToString:@"firstname"]) {
        return [[foundUsers objectAtIndex:rowIndex] firstName];
    } else if ([col isEqualToString:@"lastname"]) {
        return [[foundUsers objectAtIndex:rowIndex] lastName];
    } else if ([col isEqualToString:@"email"]) {
        return [[foundUsers objectAtIndex:rowIndex] mail];
    } else {
        return nil;
    }
}

- (BOOL)tableView:(NSTableView *)tableView writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pboard {
    int i;
    [pbUsers release];
    pbUsers = [[NSMutableArray alloc] initWithCapacity:[rows count]];
    
    for (i=0;i<[rows count];i++) {
        [pbUsers addObject:[foundUsers objectAtIndex:[[rows objectAtIndex:i] intValue]]];
    }

    
    [pboard declareTypes:[NSArray arrayWithObjects:@"FoundUsers", @"account", nil]  owner:self];
    [pboard setString:[accountPreferences UIN] forType:@"account"];
    
    
    return YES;
}

//utils
-(void)makeKeyAndOrderFront {
    [[self window] makeKeyAndOrderFront:self];
}

- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    NSString *str = NSLocalizedString(@"Search Contact for UIN: ",@"Search Contact for UIN: ");
    return [str stringByAppendingString:[accountPreferences UIN]];
}

- (void) importFromICQWithPath:(NSString*)filePath {
    NSData* contentOfFile;
    unsigned long uin=0;
    int fileIndex=0;
    const void* theData;
    contentOfFile=[[NSFileManager defaultManager] contentsAtPath:filePath];
    theData=[contentOfFile bytes];
    for (fileIndex=18;fileIndex<[contentOfFile length];fileIndex=fileIndex+8) {
        uin=[contentOfFile deserializeIntAtIndex:fileIndex];
        if (uin>10000) {
            NSNumber *sUIN=[NSNumber numberWithLong:uin];
            SMUserInfos *contact = [[SMUserInfos alloc] initWithUIN:[sUIN stringValue] nickname:[sUIN stringValue]];
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                contact, @"SMUserInfos",
                nil];

        
            [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];

        }
    }
}

- (void) importFromFireWithPath:(NSString*)filePath
{
    NSDictionary* firePrefs;
    NSDictionary* fireGroups;
    NSEnumerator *enumerator;
    id key;
    
    firePrefs=[NSDictionary dictionaryWithContentsOfFile:filePath];
    fireGroups=[firePrefs objectForKey:@"groups"];
    enumerator = [fireGroups keyEnumerator];

    while ((key = [enumerator nextObject])) {
        NSArray* buddies=[[fireGroups objectForKey:key] objectForKey:@"buddies"];
        int counter=0;
        for (counter=0;counter<[buddies count];counter++)
        {
            NSDictionary* buddy=[buddies objectAtIndex:counter];
            unsigned int uin=[[buddy objectForKey:@"buddyname"] intValue];
            NSNumber *sUIN=[NSNumber numberWithLong:uin];
            SMUserInfos *contact = [[SMUserInfos alloc] initWithUIN:[sUIN stringValue] nickname:[sUIN stringValue]];
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                contact, @"SMUserInfos",
                nil];

        
            [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];
        }
    }
  }

- (void) importFromGerryWithPath:(NSString*)filePath
{
    NSString* gerryList=[NSString stringWithContentsOfFile:filePath];
    NSRange nextRange=[gerryList rangeOfString:@"NEXT"];
    NSCharacterSet* finalSet=[NSCharacterSet whitespaceAndNewlineCharacterSet];
    NSRange endRange;
    
    
    while (nextRange.length!=0)
    {   SMUserInfos *contact;
        NSDictionary *dict;
        unsigned int uin;
        NSNumber *sUIN;
        gerryList=[gerryList substringFromIndex:(nextRange.location+5)];
        endRange=[gerryList rangeOfCharacterFromSet:finalSet];
        uin=[[gerryList substringToIndex:endRange.location] intValue];
        sUIN=[NSNumber numberWithLong:uin];
        
            contact = [[SMUserInfos alloc] initWithUIN:[sUIN stringValue] nickname:[sUIN stringValue]];
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                contact, @"SMUserInfos",
            nil];

        
            [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];

        nextRange=[gerryList rangeOfString:@"NEXT"];
    }
}

-(void)importFromV1WithPath:(NSString*)filePath {
    NSArray* contacts = [SMUINPrefs getContactsForFile:filePath];
    int i;
    for (i=0;i<[contacts count];i++) {
        NSString *uin=[contacts objectAtIndex:i];
        SMUserInfos *contact;
        NSDictionary *dict;
        contact = [[SMUserInfos alloc] initWithUIN:uin nickname:uin];
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                contact, @"SMUserInfos",
            nil];   
        [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];
   }
}

-(void)importFromProteusWithPath:(NSString*)filePath {
  NSDictionary* proteusPrefs = [NSArray arrayWithContentsOfFile:filePath];
  NSArray* proteusContacts = [proteusPrefs objectForKey:@"Data"];
  int i;
  for (i=0;i<[proteusContacts count];i++) {
	if ([[[proteusContacts objectAtIndex:i] objectForKey:@"ServiceIdentifier"] isEqualToString:@"icq"])
	{
	  NSString *uin=[[proteusContacts objectAtIndex:i] objectForKey:@"Identifier"];
	  SMUserInfos *contact;
	  NSDictionary *dict;
	  contact = [[SMUserInfos alloc] initWithUIN:uin nickname:uin];
	  dict=[NSDictionary dictionaryWithObjectsAndKeys:
		[accountPreferences UIN], @"uin",
		contact, @"SMUserInfos",
		nil];
	  [[NSNotificationCenter defaultCenter] postNotificationName:@"searchContact" object:self userInfo:dict];
	}
  }
}


@end
