/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMThemeManager.m
//  iXQ
//
//  Created by Philippe Champeaux on Mon Jun 24 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMThemeManager.h"
#import "SMTheme.h"


@implementation SMThemeManager

static SMThemeManager *manager;

+(id)sharedManager {
    if (!manager) {
        manager=[[SMThemeManager alloc] init];
    }
    return manager;
}

-(id)init
{
    self = [super init];
    if (self)
    {
        NSEnumerator	*theLibrarysEnumerator;
        NSString	*theLibraryPath;
        
        themes = [[NSMutableDictionary alloc] init];
        
        // get default theme
        [themes setObject:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/Resources/"] forKey:@"default"]; // add default theme to themes list

        // search for other themes
        theLibrarysEnumerator = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSAllDomainsMask, YES) objectEnumerator];
        while( (theLibraryPath = [theLibrarysEnumerator nextObject]) )	// iterate over the libary directories in all domains (user, system, network, ...)
        {
            BOOL isDirectory;        
            NSDirectoryEnumerator	*theDirectoryEnumerator;
            NSString	*theFilePath, *theSearchPath;

            theSearchPath = [theLibraryPath stringByAppendingFormat:@"/Application Support/iXQ/Themes"]; // themes are stored in our "Application Support" subdirectory
            theDirectoryEnumerator = [[NSFileManager defaultManager] enumeratorAtPath:theSearchPath];
            while(theFilePath = [theDirectoryEnumerator nextObject])	// iterate over all objects in this "Themes" folder
            {
                [[NSFileManager defaultManager] fileExistsAtPath:[theSearchPath stringByAppendingFormat:@"/%@", theFilePath] isDirectory:&isDirectory];	// should always exist
                if (isDirectory == true) // themes are always folders
                    if (![themes objectForKey:theFilePath]) // only add if no item with this name (last path component) doesn't exist
                        [themes setObject:[theSearchPath stringByAppendingFormat:@"/%@", theFilePath] forKey:theFilePath]; // add to themes list
            }
	}
    }
    return self;
}

-(void)dealloc {
    [themes release];
}


// accessors
-(NSArray*)themes {
    return [themes allKeys];
}

-(SMTheme*)themeWithName:(NSString*)name {
    return [[[SMTheme alloc] initWithPath:[themes objectForKey:name]] autorelease];
}

@end
