/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMUserInfos.m
//  iXQ
//
//  Created by Philippe Champeaux on Wed Mar 27 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMUserDetailsPanelController.h"
#import "SMAccountDocument.h"

@implementation SMUserDetailsPanelController
// init and such
-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(id)doc {
self=[super init];
    if (self) {
        NSArray *keys;
        int i;
        
        accountPreferences=prefs;
        (SMAccountDocument*)accountDocument=doc;
        [accountDocument addWindowController:self];
    
        languages=[[NSDictionary alloc] initWithContentsOfFile:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"/Contents/Resources/languages.txt"]];
        countries=[[NSDictionary  alloc] initWithContentsOfFile:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"/Contents/Resources/countries.txt"]];
        
        [NSBundle loadNibNamed:@"SMUserDetailsPanel" owner:self];
        
        if ([accountPreferences userDetailsPanelPosition].x>0) {
            [[self window] setFrameOrigin:[accountPreferences userDetailsPanelPosition]];
        }
        
        [[self window] setLevel:NSNormalWindowLevel+1];
        isVisible=NO;
        
        // state of send button
        if ([accountDocument connected]) {
            [refreshOutlet setEnabled:YES];
        } else {
            [refreshOutlet setEnabled:NO];
        }
        
        
        //encoding
        encodings=[[NSDictionary alloc] initWithContentsOfFile:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"/Contents/Resources/encodings.txt"]];
        keys=[encodings allKeys];
        for (i=0;i<[keys count];i++) {
            [encodingOutlet addItemWithTitle:[keys objectAtIndex:i]];
        }  
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactListSelectionChanged:) name:@"contactListSelectionChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDetailsForUIN:) name:@"userDetailsForUIN" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactChanged:) name:@"contactChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:) name:@"NSApplicationWillResignActiveNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillBecomeActive:) name:@"NSApplicationWillBecomeActiveNotification" object:nil];
        
        
    }
    return self;
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [languages release];
    [countries release];
    [contact release];
    [super dealloc];
}


// actions
-(IBAction)refresh:(id)sender {
    [refreshOutlet setEnabled:NO];
    [progressOutlet startAnimation:self];
    [contact setIsSearchingForInfos:YES];
    
    [accountDocument requestFullInfoForUIN:[contact UIN]];
}

// observers
-(void)userDetailsForUIN:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMUserInfos *user=[[[notification userInfo] objectForKey:@"SMUserInfos"] retain];
        if (user) {
            SMUserInfos *newContact=[[accountDocument contactOfUIN:[user UIN]] retain];
            if (newContact) {
                NSDictionary *dict;
                [newContact generalFromDict:[user toDict]];
                [newContact setIsSearchingForInfos:NO];
                //[self setContact:newContact];
                //[self redisplay];
                [newContact release];
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"redisplayContactList" object:self userInfo:dict];

            }
            [self redisplay];
        }
    }
}

-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (![contact isSearchingForInfos]) {
                [refreshOutlet setEnabled:YES];
        }
    }
}
-(void)userDisconnected:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [refreshOutlet setEnabled:NO];    
    }
}
-(void)contactChanged:(NSNotification *)notification {
  NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
	[self redisplay];
  }
}

-(void)contactListSelectionChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMUserInfos *user=[[[notification userInfo] objectForKey:@"selection"] retain];
        if (user) {
            [self readChanges];
            [contact release];
            contact=[user retain];
            [self redisplay];
        }
    }
}

-(void)applicationWillResignActive:(NSNotification *)notification {
    isVisible=[[self window] isVisible];
    [[self window] orderOut:self];
}
-(void)applicationWillBecomeActive:(NSNotification *)notification {
    if (isVisible) {
        [[self window] makeKeyAndOrderFront:self];
    }
}
- (void)windowWillClose:(NSNotification *)aNotification {
    [self readChanges];
    [accountPreferences setUserDetailsPanelPosition:([[self window] frame]).origin];
}


// actions
-(IBAction)sendMail:(id)sender {
    NSString *addr=@"mailto:";
    addr=[addr stringByAppendingString:[mailOutlet stringValue]];
       
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:addr ]];

}

-(IBAction)goToHomePage:(id)sender {
    NSString *addr=@"http://";
    NSRange nextRange=[[homepageOutlet stringValue] rangeOfString:addr];
    if (nextRange.location == NSNotFound)
    {
        addr=[addr stringByAppendingString:[homepageOutlet stringValue]];
        [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:addr]];
    }
    else
        [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[homepageOutlet stringValue] ]];
}

-(IBAction)messageWindowStayOnTopAction:(id)sender {
    [contact setMessageWindowStayOnTop:[messageWindowStayOnTopOutlet state]];
    [self notifyChange];
}
-(IBAction)messageWindowAutoCloseAction:(id)sender {
    [contact setMessageWindowAutoClose:[messageWindowAutoCloseOutlet state]];
    [self notifyChange];
}
-(IBAction)notifyWhenOnlineAction:(id)sender {
    [contact setNotifyWhenOnline:[notifyWhenOnlineOutlet state]];
    [self notifyChange];
}
-(IBAction)preferedNickNameAction:(id)sender {
    if (![[preferedNickNameOutlet stringValue] isEqualToString:@""]) {
        [contact setPreferedNickname:[preferedNickNameOutlet stringValue]];
    } else {
        [contact setPreferedNickname:nil];
    }
    
    [self notifyChange];
}

	
-(IBAction)encodingAction:(id)sender {
    if ([[sender title] isEqualToString:@"Auto"]) {
        [contact setAutoEncoding:YES];
        [encodingDescOutlet setStringValue:NSLocalizedString(@"Depends on the contact's client",@"Depends on the contact's client")];
        if ([contact clientVers]) {
            if ([contact isMacClient]){
                 [contact setEncoding:@"NSMacOSRomanStringEncoding"];
                 //NSLog([[contact UIN] stringByAppendingString: @" : selected NSMacOSRomanStringEncoding encoding"]);
            } else {
                [contact setEncoding:@"NSISOLatin1StringEncoding"];
                //NSLog([[contact UIN] stringByAppendingString: @" : selected NSISOLatin1StringEncoding encoding"]);                
            }
        }
    } else {
        [contact setAutoEncoding:NO];
        [contact setEncoding:[sender title]];
        [encodingDescOutlet setStringValue:[[encodings objectForKey:[sender title]] objectForKey:@"desc"]];
    }
    [self notifyChange];
}


// utils
-(void)makeKeyAndOrderFront:(id)sender {
    [[self window] makeKeyAndOrderFront:sender];
}

-(void)setContact:(SMUserInfos*)user {
    if (contact) {
        [self readChanges];
    }
    contact=[user retain];
}

-(void)redisplay {
    
    [UINOutlet setStringValue:@""];
    [nickNameOutlet setStringValue:@""];
    [preferedNickNameOutlet setStringValue:@""];
    [firstNameOutlet setStringValue:@""];
    [lastNameOutlet setStringValue:@""];
    [mailOutlet setStringValue:@""];
    [sendMailOutlet setEnabled:NO];
    [homepageOutlet setStringValue:@""];
    [gotoHomePageOutlet setEnabled:NO];
    [ageOutlet setStringValue:@""];
    [cityOutlet setStringValue:@""];
    [streetOutlet setStringValue:@""];
    [stateOutlet setStringValue:@""];
    [sexeOutlet setStringValue:@"N/A"];
    [birthdayOutlet setStringValue:@"N/A"];
    [aboutOutlet setString:@""];
    [ipOutlet setStringValue:@"0.0.0.0"];
    [clientOutlet setStringValue:@""];
    [zipOutlet setStringValue:@""];
    
    if ([contact UIN])			[UINOutlet setStringValue:[contact UIN]];
    if ([contact firstName])		[firstNameOutlet setStringValue:[contact firstName]];
    if ([contact ip])			[ipOutlet setStringValue:[contact ip]];    
    if ([contact lastName])		[lastNameOutlet setStringValue:[contact lastName]];
    if ([contact nickname])		[nickNameOutlet setStringValue:[contact nickname]];
    if ([contact preferedNickname])                                    [preferedNickNameOutlet setStringValue:[contact preferedNickname]];
    if ([contact autoEncoding]) {
        [encodingOutlet selectItemAtIndex:0];
        [encodingDescOutlet setStringValue:NSLocalizedString(@"Depends on the contact's client",@"Depends on the contact's client")];
        
    } else if ([contact encoding]) {
        int i=0;
        NSArray *keys=[encodings allKeys];
        while (i<[keys count] && ![[keys objectAtIndex:i] isEqualToString:[contact encoding]]) {
            i++;
        }
        if (i<[keys count]) {
            [encodingOutlet selectItemAtIndex:i+1];
            [encodingDescOutlet setStringValue:[[encodings objectForKey:[keys objectAtIndex:i]] objectForKey:@"desc"]];
        }
    }
	    
    switch ([contact clientVers])
    {
        case kMacIcq3Vers:
        {
            [clientOutlet setStringValue:@"Mirabilis MacICQ 3.0.x"];
            break;
        }
        case kMacIcq31Vers:
        {
            [clientOutlet setStringValue:@"Mirabilis MacICQ 3.1"];
            break;
        }        
        case kGerryIcq45Vers:
        {
            [clientOutlet setStringValue:@"Mirabilis WinICQ 99b or Gerry's ICQ"];
            break;
        }        
        case kLibIcq2000Vers:
        {
            [clientOutlet setStringValue:@"Mirabilis WinICQ 2001 or libICQ2000 \n(Ickle, CenterICQ, IcyJuice, Fire, Proteus, ...)"];
            break;
        }     
        case kLibIcq2000Vers2:
        {
            [clientOutlet setStringValue:@"libICQ2000 \n(Ickle, CenterICQ, IcyJuice, Fire, Proteus, ...)"];
            break;
        }            
        case kWin2001bVers:
        {
            [clientOutlet setStringValue:@"Mirabilis WinICQ 2001"];
            break;
        }           
        case kWin2002aVers:
        {
            [clientOutlet setStringValue:@"Mirabilis WinICQ 2002"];
            break;
        }   
        case kiXQVers:
        {
            [clientOutlet setStringValue:@"iXQ (yeah!;)"];
            break;
        }     
        case kUnknownVers:
        {
            [clientOutlet setStringValue:NSLocalizedString(@"unknown client", @"unknown client")];
            break;
        }          
        default:
        {
            [clientOutlet setStringValue:NSLocalizedString(@"unavailable", @"unavailable (client)")];        
        }               
    }
    if ([contact mail]) {
                                        [mailOutlet setStringValue:[contact mail]];
                                        [sendMailOutlet setEnabled:YES];
    } 
    if ([contact homepage]) {
                                        [homepageOutlet setStringValue:[contact homepage]];
                                        [gotoHomePageOutlet setEnabled:YES];
    }
                                        [ageOutlet setIntValue:[contact age]];
                                        //birthday
    if ([contact sex]==2) {
        [sexeOutlet setStringValue:NSLocalizedString(@"male", @"male")];
    } else if ([contact sex]==1){
        [sexeOutlet setStringValue:NSLocalizedString(@"female", @"female")];
    }
    
    if ([contact birthyear] && [contact birthmonth] && [contact birthday]) {
        [birthdayOutlet setStringValue:[[[[[NSString 
                stringWithString:[[NSNumber numberWithInt:[contact birthmonth]] stringValue]]
                stringByAppendingString:@"/"]
                stringByAppendingString:[[NSNumber numberWithInt:[contact birthday]] stringValue]]
                stringByAppendingString:@"/"]
                stringByAppendingString:[[NSNumber numberWithInt:[contact birthyear]] stringValue]]];

    }
    
                                        [language1Outlet setStringValue:[self languageForKey:[contact lang1]]];
                                        [language2Outlet setStringValue:[self languageForKey:[contact lang2]]];
                                        [language3Outlet setStringValue:[self languageForKey:[contact lang3]]];
                                        [countryOutlet setStringValue:[self countryForKey:[contact country]]];
    if ([contact city])			[cityOutlet setStringValue:[contact city]];
    if ([contact state])		[stateOutlet setStringValue:[contact state]];
    if ([contact street])		[streetOutlet setStringValue:[contact street]];
    if ([contact zip])			[zipOutlet setStringValue:[contact zip]];
    if ([contact phone])		[homePhoneOutlet setStringValue:[contact phone]];
    if ([contact cellular])		[cellularPhoneOutlet setStringValue:[contact cellular]];
    if ([contact fax])			[faxPhoneOutlet setStringValue:[contact fax]];
    
    if ([contact about])		[aboutOutlet setString:[contact about]];
    
   
   
    if ([contact messageWindowStayOnTop]) {
       [messageWindowStayOnTopOutlet setState:1];
    } else {
        [messageWindowStayOnTopOutlet setState:0];
    }
    
    if ([contact messageWindowAutoClose]) {
       [messageWindowAutoCloseOutlet setState:1];
    } else {
        [messageWindowAutoCloseOutlet setState:0];
    }
    if ([contact notifyWhenOnline]) {
       [notifyWhenOnlineOutlet setState:1];
    } else {
        [notifyWhenOnlineOutlet setState:0];
    }
   
    if ([contact isSearchingForInfos] ) {
        [progressOutlet startAnimation:self];
        [refreshOutlet setEnabled:NO];
    } else {
        [progressOutlet stopAnimation:self];
        if ([accountDocument connected]) {
            [refreshOutlet setEnabled:YES];
        }
    }
}



- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    NSString *str = NSLocalizedString(@"Details Inspector for UIN : ",@"Details Inspector for UIN : ") ;
    return [str stringByAppendingString:[accountPreferences UIN]];
}

-(NSString*)languageForKey:(unsigned short)nb {
    NSArray *keys=[languages allKeys];
    int i=0;
    while (i<[keys count] && ![[languages objectForKey:[keys objectAtIndex:i]] isEqualToString:[[NSNumber numberWithShort:nb] stringValue]]) {
        i++;
    }  
    if (i<[keys count]) {
        return [keys objectAtIndex:i];
    } else {
        return @"N/A";
    }
}
-(NSString*)countryForKey:(unsigned short)nb {
    NSArray *keys=[countries allKeys];
    int i=0;
    while (i<[keys count] && ![[countries objectForKey:[keys objectAtIndex:i]] isEqualToString:[[NSNumber numberWithShort:nb] stringValue]]) {
        i++;
    }  
    if (i<[keys count]) {
        return [keys objectAtIndex:i];
    } else {
        return @"N/A";
    }
}

-(void)notifyChange {
    NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
        [accountPreferences UIN], @"uin",
        [contact UIN], @"contactUIN",
        nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"contactChanged" object:self userInfo:dict];
    [accountDocument saveContactList];
}

-(void)readChanges {
    if (![[preferedNickNameOutlet stringValue] isEqualToString:@""]) {
        [contact setPreferedNickname:[preferedNickNameOutlet stringValue]];
    } else if ([preferedNickNameOutlet stringValue]){
        [contact setPreferedNickname:nil];
    }
    

}
@end
