/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMVisibleListsWindowController.m
//  iXQ
//
//  Created by Philippe Champeaux on Wed Apr 10 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMListsWindowController.h"
#import "SMAccountDocument.h"


@implementation SMListsWindowController
//init and such

-(id)initWithPreferences:(SMAccountPreferences*)prefs document:(id)doc {
    self=[super init];
    if (self) {
        accountPreferences=prefs;
        (SMAccountDocument*)accountDocument=doc;
        //[accountDocument addWindowController:self];
        
        visibleKeys = [[NSMutableArray alloc] init];
        invisibleKeys = [[NSMutableArray alloc] init];
        ignoreKeys = [[NSMutableArray alloc] init];
        visibleList = [[NSMutableDictionary alloc] init];
        invisibleList = [[NSMutableDictionary alloc] init];
        ignoreList = [[NSMutableDictionary alloc] init];
        
        
        [NSBundle loadNibNamed:@"SMListsWindow" owner:self];
        
        if ([accountPreferences listsWindowPosition].size.height>0) {
            [[self window] setFrame:[accountPreferences listsWindowPosition] display:NO];
        }
               
        // register observers
       /* [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(openListsWindow:) name:@"openListsWindow" object:nil];*/
        
    }
    return self;
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    //[accountDocument removeWindowController:self];
    [super dealloc];
}

- (void)awakeFromNib {
    [visibleView registerForDraggedTypes:[NSArray arrayWithObjects:@"SMUsersInfos",nil]];
    [invisibleView registerForDraggedTypes:[NSArray arrayWithObjects:@"SMUsersInfos",nil]];
    [ignoreView registerForDraggedTypes:[NSArray arrayWithObjects:@"SMUsersInfos",nil]];
}

-(NSArray*)toDicts {
    NSMutableArray *array=[[NSMutableArray alloc] init];
    [array addObject:visibleList ];
    [array addObject:invisibleList ];
    [array addObject:ignoreList ];
    
    return [array autorelease];
}
-(void)fromDicts:(NSArray*)lists {
    if (lists) {
        [visibleList release];
        [visibleKeys release];
    
        visibleList=[[NSMutableDictionary alloc] initWithDictionary:[lists objectAtIndex:0]];
        visibleKeys=[[NSMutableArray alloc] initWithArray:[visibleList allKeys]];
        
        [invisibleList release];
        [invisibleKeys release];
    
        invisibleList=[[NSMutableDictionary alloc] initWithDictionary:[lists objectAtIndex:1]];
        invisibleKeys=[[NSMutableArray alloc] initWithArray:[invisibleList allKeys]];
        
        [ignoreList release];
        [ignoreKeys release];
    
        ignoreList=[[NSMutableDictionary alloc] initWithDictionary:[lists objectAtIndex:2]];
        ignoreKeys=[[NSMutableArray alloc] initWithArray:[ignoreList allKeys]];
        
    }
}



// accessors
-(NSArray*)visibleList {
    return [visibleKeys retain];
}
-(NSArray*)invisibleList {
    return [invisibleKeys retain];
}
-(NSArray*)ignoreList {
    return [ignoreKeys retain];
}

// table view delegates

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {
    if (aTableView==visibleView) {
        return [visibleList count];
    } else if (aTableView==invisibleView) {
        return [invisibleList count];
    } else {
        return [ignoreList count];
    }
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {
    NSString *col = [aTableColumn identifier];
    if ([col isEqualToString:@"visibleUIN"]) {
        return [visibleKeys objectAtIndex:rowIndex];
    } else if ([col isEqualToString:@"visibleNickname"]) {
        return [visibleList objectForKey:[visibleKeys objectAtIndex:rowIndex]];
    } else if ([col isEqualToString:@"invisibleUIN"]) {
        return [invisibleKeys objectAtIndex:rowIndex];
    } else if ([col isEqualToString:@"invisibleNickname"]) {
        return [invisibleList objectForKey:[invisibleKeys objectAtIndex:rowIndex]];
    } else if ([col isEqualToString:@"ignoreUIN"]) {
        return [ignoreKeys objectAtIndex:rowIndex];
    } else if ([col isEqualToString:@"ignoreNickname"]) {
        return [ignoreList objectForKey:[ignoreKeys objectAtIndex:rowIndex]];
    } else {
        return nil;
    }
}


- (unsigned int)tableView:(NSTableView*)tv validateDrop:(id<NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op {
    
    if (op==NSTableViewDropAbove) {
        NSMutableArray *users = [[accountDocument contactListWindow] pbUsers]; 
        int i;
        for (i=0;i<[users count];i++) {
            if (tv==visibleView) {
                if ([visibleKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
                    return NSDragOperationAll;
                }
            } else if (tv==invisibleView) {
                    if ([invisibleKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
                    return NSDragOperationAll;  
                }
            } else {
                if ([ignoreKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
                    return NSDragOperationAll;
                }
            }
        }
        
        return NSDragOperationNone;
    }
    return NSDragOperationNone;
}


- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id<NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op {
    int i;
    NSMutableArray *users = [[[accountDocument contactListWindow] pbUsers] retain]; 
    for (i=0;i<[users count];i++) {
        SMUserInfos *user=[[users objectAtIndex:i] copy];
        if (row<0) row=0;
        
        if (tv==visibleView&&[visibleKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
            [self setVisible:[[user UIN] copy] state:YES];
            
        } else if (tv==invisibleView&&[invisibleKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
        [self setInvisible:[[user UIN] copy] state:YES];
            
        } else if (tv==ignoreView&&[ignoreKeys indexOfObject:[[users objectAtIndex:i] UIN]]==NSNotFound) {
            [self setIgnore:[[user UIN] copy] state:YES];
        }
        
        [user release];
    }

        
    return YES;
}



//utils
-(void)makeKeyAndOrderFront:(id)sender {
    [[self window] makeKeyAndOrderFront:self];
}

- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    NSString *str = @"Lists for UIN : ";
    return [str stringByAppendingString:[accountPreferences UIN]];
}


-(BOOL)isContactInVisibles:(NSString*)uin {
    return [visibleKeys indexOfObject:uin]!=NSNotFound;
}
-(BOOL)isContactInInvisibles:(NSString*)uin {
    return [invisibleKeys indexOfObject:uin]!=NSNotFound;
}
-(BOOL)isContactInIgnores:(NSString*)uin {
    return [ignoreKeys indexOfObject:uin]!=NSNotFound;
}

-(void)setVisible:(NSString*)uin state:(BOOL)state {
    if (!state) {
        [visibleList removeObjectForKey:uin];
        [visibleKeys removeObjectAtIndex:[visibleKeys indexOfObject:uin]];
        if (![accountDocument isVisible]) {
            [(SMAccountDocument*)accountDocument removeFromVisibleList:uin];
        }
    } else {
        SMUserInfos *user=[accountDocument contactOfUIN:uin];
        [visibleKeys addObject:[[user UIN] copy]];
        [visibleList setObject:[[user preferedNickname] copy] forKey:[user UIN]];
        if (![accountDocument isVisible]) {
            [(SMAccountDocument*)accountDocument addToVisibleList:uin];
        }
    }
    [visibleView reloadData];
}
-(void)setInvisible:(NSString*)uin state:(BOOL)state{
    if (!state) {
        [invisibleList removeObjectForKey:uin];
        [invisibleKeys removeObjectAtIndex:[invisibleKeys indexOfObject:uin]];
        if ([accountDocument isVisible]) {
            [(SMAccountDocument*)accountDocument removeFromInvisibleList:uin];
        }
    } else {
        SMUserInfos *user=[accountDocument contactOfUIN:uin];
        [invisibleKeys addObject:[[user UIN] copy]];
        [invisibleList setObject:[[user preferedNickname] copy] forKey:[user UIN]];
        if ([accountDocument isVisible]) {
            [(SMAccountDocument*)accountDocument addToInvisibleList:uin];
        }
    }
    [invisibleView reloadData];
}
-(void)setIgnore:(NSString*)uin state:(BOOL)state{
   if (!state) {
        [ignoreList removeObjectForKey:uin];
        [ignoreKeys removeObjectAtIndex:[ignoreKeys indexOfObject:uin]];
    } else {
        SMUserInfos *user=[accountDocument contactOfUIN:uin];
        [ignoreKeys addObject:[[user UIN] copy]];
        [ignoreList setObject:[[user preferedNickname] copy] forKey:[user UIN]];
    }
    [ignoreView reloadData];
}


// actions
-(IBAction)remove:(id)sender {
    if ([tabView indexOfTabViewItem:[tabView selectedTabViewItem]]==0) {
        if ([visibleView selectedRow]>=0) {
            [self setVisible:[visibleKeys objectAtIndex:[visibleView selectedRow]] state:NO];
            [visibleView reloadData];
        }
    } else if ([tabView indexOfTabViewItem:[tabView selectedTabViewItem]]==1) {
        if ([invisibleView selectedRow]>=0) {
            [self setInvisible:[invisibleKeys objectAtIndex:[invisibleView selectedRow]] state:NO];
            [invisibleView reloadData];
        }
    } else if ([tabView indexOfTabViewItem:[tabView selectedTabViewItem]]==2) {
        if ([ignoreView selectedRow]>=0) {
            [self setIgnore:[ignoreKeys objectAtIndex:[ignoreView selectedRow]] state:NO];
            [ignoreView reloadData];
        }
    }
}


// observers
- (void)windowWillClose:(NSNotification *)aNotification {
    [accountPreferences setListsWindowPosition:[[self window] frame]];
}
@end
