/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  MainController.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "MainController.h"
#import "SMAccountDocument.h"
#import "SMThemeManager.h"
#import "SMAboutBox.h"

#define OPEN_DOCS 1

@implementation MainController

NSString *SMOpenDocListFlagKey  = @"OpenDocumentsFlag";
NSString *SMOpenDocListKey      = @"OpenDocuments";

// init and such
-(id)init {
    self = [super init];
    if (self) {
    }
    return self;
}

+ (void)initialize {
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];

    [defaultValues setObject:[NSNumber numberWithBool:YES]
		      forKey:SMOpenDocListFlagKey];

    [defaultValues setObject:[NSArray array]
                      forKey:SMOpenDocListKey];

    [[NSUserDefaults standardUserDefaults] registerDefaults:defaultValues];
}

-(void)dealloc {
    if (updatesWindow) {
        [updatesWindow release];
    }
}

- (BOOL)acceptsFirstResponder {
    return YES;
}
- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)sender {
	return NO;
}

-(IBAction)openReferencedBugs:(id)sender {
    NSString *path=[[NSBundle mainBundle] bundlePath];
    NSString *text=[[NSString alloc] initWithContentsOfFile:[path stringByAppendingPathComponent:@"/Contents/Resources/PublicBugs.txt"]];
    [bugs setString:text];
    [bugsWindow makeKeyAndOrderFront:self];
}
-(IBAction)openUpdates:(id)sender {
    if (!updatesWindow) {
        updatesWindow=[[SMUpdatesController alloc] init];
    }
    [updatesWindow makeKeyAndOrderFront:self];
}
-(IBAction)openCurrentPreferences:(id)sender {
    [(SMAccountDocument*)[[NSDocumentController sharedDocumentController] currentDocument] openPreferences];
}

// observers

- (void)applicationDidBecomeActive:(NSNotification *)aNotification {
    [[NSNotificationCenter defaultCenter] postNotificationName:@"applicationDidBecomeActive" object:self userInfo:nil];
}

- (void)applicationWillFinishLaunching:(NSNotification *)aNotification {
    
#if OPEN_DOCS
    NSArray *filenames = [[NSUserDefaults standardUserDefaults] arrayForKey:SMOpenDocListKey];

    NSEnumerator *e = [filenames reverseObjectEnumerator];
    NSString *fileName;

    while (fileName = [e nextObject]) {
        [[NSDocumentController sharedDocumentController]
           openDocumentWithContentsOfFile:fileName display:YES];
    }
#endif    

}

- (NSApplicationTerminateReply) applicationShouldTerminate: (NSApplication *)sender {

#if OPEN_DOCS
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSMutableArray *fileNames = [NSMutableArray array];
    NSString *fileName;
    SMAccountDocument* tDocument;
    
    NSEnumerator *e = [[[NSApplication sharedApplication] orderedDocuments]
	objectEnumerator];

    while (tDocument = [e nextObject]) {
        if ([[tDocument accountPreferences] autoOpen])
        {
            [fileNames addObject:[tDocument fileName]];
        }
    }
    [defaults setObject:[NSArray arrayWithArray:fileNames]
                 forKey:SMOpenDocListKey];

    [defaults synchronize];
#endif

    return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem {
    if ([[anItem title] isEqualToString:NSLocalizedString(@"Current Account Settings...", @"Current Account Settings Menu Item (must be the same as in the nib)")]) {
        return [[[NSDocumentController sharedDocumentController] documents] count]>0;
    }

    return YES;
}

-(IBAction)openAboutBox:(id)sender
{
  [[SMAboutBox sharedInstance] showPanel];
}
@end

