/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMAccount.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMAccountDocument.h"
#import "SMThemeManager.h"

@implementation SMAccountDocument


// init and such
-(id)init {
    self = [super init];
    if (self) {
        currentStatus=0;
        oldStatus=0;
        accountPreferences = [[SMAccountPreferences alloc] initWithParent:self];
        [self addWindowController:accountPreferences];
        
        
        userInfos = [[SMUserInfos alloc] init];
        
        messagesManager = [[SMMessagesManager alloc] initWithPreferences:accountPreferences parent:self];
        notificationManager = [[SMNotifier alloc] initWithPreferences:accountPreferences parent:self];
        
        authorizationManager = [[SMAuthorizationManager alloc] initWithPreferences:accountPreferences parent:self];
        
        
        //listsWindow = [[SMListsWindowController alloc] initWithPreferences:accountPreferences document:self];
        
        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(statusChanged:) name:@"statusChanged" object:nil];


        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewUserDetailsForUIN:) name:@"viewUserDetailsForUIN" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDetailsForUIN:) name:@"userDetailsForUIN" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(eventOccured:) name:@"eventOccured" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionLost:) name:@"connectionLost" object:nil];
        
        autoAway=NO; autoNA=NO;
        mouseInCorner=NO;
        od=[[NSDate date] retain];
        
        
        
        
        
    }
    return self;
}
/*-(id)initWithContentsOfFile:(NSString *)fileName ofType:(NSString *)type {

}*/
-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    if (timer) [timer release];
    [notificationManager release];
    [listsWindow release];
    [messagesManager release];
    [contactList release];
    [historyWindow closeHistories];
    [historyWindow release];
    [userDetailsPanel release];
    [userInfos release];
    [accountPreferences release];
    [authorizationManager release];
    


    [super dealloc];
}


- (BOOL)readFromFile:(NSString *)fileName ofType:(NSString *)type {
        NSDictionary *initData;
        // accountPreferences
        initData = [NSDictionary dictionaryWithContentsOfFile: [fileName stringByAppendingPathComponent:@"AccountPreferences.plist"]];
        [accountPreferences fromDict:initData];

        [self removeWindowController:accountPreferences];
        
        // userInfos
        initData = [NSDictionary dictionaryWithContentsOfFile: [fileName stringByAppendingPathComponent:@"UserInfos.plist"]];
        [userInfos fromDict:initData];

        



        if (contactList) [contactList release];
        contactList = [[SMContactListController alloc] initWithPreferences:accountPreferences parent:self];
       /* if (historyWindow) [historyWindow release];
        historyWindow = [[SMHistoryManagerController alloc] initWithPreferences:accountPreferences parent:self];*/
        
        [self addWindowController:contactList];
        
        
        // contactList
        [contactList fromDict:[NSDictionary dictionaryWithContentsOfFile: [fileName stringByAppendingPathComponent:@"ContactList.plist"]]];
        
        
        if (listsWindow) [listsWindow release];
        listsWindow = [[SMListsWindowController alloc] initWithPreferences:accountPreferences document:self];
        
        
        [listsWindow fromDicts:[NSArray arrayWithContentsOfFile: [fileName stringByAppendingPathComponent:@"Lists.plist"]]];
        
        [contactList setPosition:[accountPreferences contactListPosition]];

        
        // create the history folder if it does not exists
        [[NSFileManager defaultManager] createDirectoryAtPath:[fileName stringByAppendingPathComponent:@"Histories"] attributes:nil];
        
        [self restoreDocument];
        
        [[accountPreferences window] orderOut:self];
        [contactList openAndResize];
        
        if ([accountPreferences loginStatus]) {
            [contactList connect];
        }
    return YES;
}

- (BOOL)writeToFile:(NSString *)fileName ofType:(NSString *)type {
    NSFileWrapper *wrapper;
    NSString *oldFile=[[[fileName stringByDeletingPathExtension] stringByAppendingString:@"~"] stringByAppendingPathExtension:@"ixq"];
    [[NSFileManager defaultManager] createDirectoryAtPath:fileName attributes:nil];
    
    
    
    
    [[contactList toDict] writeToFile: [fileName stringByAppendingPathComponent:@"ContactList.plist"] atomically:YES];
    [[accountPreferences toDict] writeToFile:[ fileName stringByAppendingPathComponent:@"AccountPreferences.plist"] atomically:YES];
    [[userInfos toDict] writeToFile:[ fileName stringByAppendingPathComponent:@"UserInfos.plist"] atomically:YES];
    [[listsWindow toDicts] writeToFile:[ fileName stringByAppendingPathComponent:@"Lists.plist"] atomically:YES];
    
    if ([fileName isEqualToString:[self fileName]]) {
        [[NSFileManager defaultManager] movePath:[oldFile stringByAppendingPathComponent:@"Histories"] toPath:[fileName stringByAppendingPathComponent:@"Histories"] handler:nil];
    }
    
    if (![[NSFileManager defaultManager] fileExistsAtPath:[fileName stringByAppendingPathComponent:@"Histories"]]) {
        [[NSFileManager defaultManager] createDirectoryAtPath:[fileName stringByAppendingPathComponent:@"Histories"] attributes:nil];
    }
    
    wrapper=[[NSFileWrapper alloc] initWithPath:fileName];
    
    
    return [wrapper writeToFile:fileName atomically:YES updateFilenames:YES];
}

-(void)restoreDocument {
    NSArray *contacts=[self contacts];
    int i;
    for (i=0;i<[contacts count]; i++) {
        NSString *uin=[(SMUserInfos*)[contacts objectAtIndex:i] UIN];
        NSString *path=[[[[self fileName] stringByAppendingPathComponent:@"Histories"] stringByAppendingPathComponent:uin] stringByAppendingPathExtension:@"plist"];
        NSString *tmpPath=[path stringByAppendingPathExtension:@"tmp"];
       	if ([[NSFileManager defaultManager] fileExistsAtPath:tmpPath]) {
            //NSFileHandle *history;
            [[NSFileManager defaultManager] removeFileAtPath:path  handler:nil];
            [[NSFileManager defaultManager] movePath:tmpPath toPath:path handler:nil];
            
            /*history=[[NSFileHandle fileHandleForWritingAtPath:path] retain];
            [history seekToEndOfFile];
            [history writeData:[[NSString stringWithString:@")"] dataUsingEncoding:NSISOLatin1StringEncoding]];
            [history release];*/

        }
    }
}


    // NSDocument stuff

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    [super windowControllerDidLoadNib:aController];
    // Add any code here that need to be executed once the windowController has loaded the document's window.
    
    [contactList makeKeyAndOrderFront:self];
}

/*- (NSData *)dataRepresentationOfType:(NSString *)aType
{
        return nil;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
    // Insert code here to read your document from the given data.  You can also choose to override -loadFileWrapperRepresentation:ofType: or -readFromFile:ofType: instead.
    return NO;
}

*/

// accessors


// observers

-(void)userDetailsForUIN:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMUserInfos *user=[[[notification userInfo] objectForKey:@"SMUserInfos"] retain];
        if (user) {
            if ([[user UIN] isEqualToString:[accountPreferences UIN]]) {
                [userInfos fromDict:[user toDict]];
            }
        }
    }
}


-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (!historyWindow) historyWindow = [[SMHistoryManagerController alloc] initWithPreferences:accountPreferences parent:self];
        
        [loginDate release];
        loginDate=[[NSDate alloc] init];
        [connection queryOurInfo];
        timer = [[NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(testEvent) userInfo:nil repeats:YES] retain];
        
        if (!userDetailsPanel) {
            userDetailsPanel = [[SMUserDetailsPanelController alloc] initWithPreferences:accountPreferences parent:self];
        }
        
    }
}

-(void)statusChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        int status = [[[notification userInfo] objectForKey:@"status"] intValue];
        if (status==0) {
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                nil];
            [timer invalidate];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userDisconnected" object:self userInfo:dict];
        
            [[NSNotificationCenter defaultCenter] removeObserver:connection];
            [connection dealloc];
            connection=nil;
            
            
        } else if (currentStatus==0) {
            [connection requestOfflineMessages];
        }
        oldStatus=currentStatus;
        currentStatus=status;
    }
}

-(void)connectionLost:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        NSDictionary *dict;
        dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                0, @"status",
                nil];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
               
    }
}




- (void)viewUserDetailsForUIN:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        [self openUserDetails:self userInfos:[[notification userInfo] objectForKey:@"SMUserInfos"] openPanel:[[[notification userInfo] objectForKey:@"openPanel"] boolValue]];
    }
}

// events

-(void)eventOccured:(NSNotification *)notification {

    if (![self connected])
    	return;
    [od release];
    od=[[NSDate date] retain];
    if ((autoAway || autoNA) && !mouseInCorner) {
            NSDictionary *dict;
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:autoAwayOldStatus], @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            autoAway=NO; autoNA=NO;
    }
}
-(void)testEvent {
    NSPoint np=[NSEvent mouseLocation];
    if (![self connected] )
    	return;
    if (!mouseInCorner && (np.x!=mousePosition.x || np.y!=mousePosition.y)) {
        [od release];
        od=[[NSDate date] retain];
        if ((autoAway|| autoNA)  && !mouseInCorner) {
            NSDictionary *dict;
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:autoAwayOldStatus], @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            autoAway=NO; 
            autoNA=NO;
        }
     } 
     mousePosition=np;
    if ([accountPreferences autoNADelay] && fabs([[NSDate date] timeIntervalSinceDate:od])>([accountPreferences autoNADelay]*60)
            &&!autoNA) {
            NSDictionary *dict;
            if (!autoAway) {
                autoAwayOldStatus=currentStatus;
            }
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:3], @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            
            autoNA=YES;

    } else if ([accountPreferences autoAwayDelay] && fabs([[NSDate date] timeIntervalSinceDate:od])>([accountPreferences autoAwayDelay]*60)
            &&!autoAway) {
            NSDictionary *dict;
            autoAwayOldStatus=currentStatus;

            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:2], @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            
            autoAway=YES;

    }else {
        NSRect	r=[[NSScreen mainScreen] frame];
        
        if ([accountPreferences autoAwayCorner]==1 && np.x<10 && np.y>r.size.height-10) {
            if (!(autoAway|| autoNA) ) {
                NSDictionary *dict;
                autoAwayOldStatus=currentStatus;
                autoAway=YES;
                mouseInCorner=YES;
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    [NSNumber numberWithInt:2], @"status",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            }
            
        } else if ([accountPreferences autoAwayCorner]==2 && np.x>r.size.width-10 && np.y>r.size.height-10) {
            if (!(autoAway|| autoNA)) {
                NSDictionary *dict;
                autoAwayOldStatus=currentStatus;
                autoAway=YES;
                mouseInCorner=YES;
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    [NSNumber numberWithInt:2], @"status",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            }            
        } else if ([accountPreferences autoAwayCorner]==3 && np.x<10 && np.y<10) {
             if (!(autoAway|| autoNA)) {
                NSDictionary *dict;
                autoAwayOldStatus=currentStatus;
                autoAway=YES;
                mouseInCorner=YES;
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    [NSNumber numberWithInt:2], @"status",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            }            
        } else if ([accountPreferences autoAwayCorner]==4 && np.x>r.size.width-10 && np.y<10) {
             if (!(autoAway|| autoNA)) {
                NSDictionary *dict;
                autoAwayOldStatus=currentStatus;
                autoAway=YES;
                mouseInCorner=YES;
                dict=[NSDictionary dictionaryWithObjectsAndKeys:
                    [accountPreferences UIN], @"uin",
                    [NSNumber numberWithInt:2], @"status",
                    nil];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            }            
            
        } else if ((autoAway|| autoNA) && mouseInCorner) {
            NSDictionary *dict;
            dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [accountPreferences UIN], @"uin",
                [NSNumber numberWithInt:autoAwayOldStatus], @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            autoAway=NO;
            autoNA=NO;
            mouseInCorner=NO;

        }

    }
    

}

// accessors
-(NSDate*)loginDate {
    return loginDate;
}	

-(SMAccountPreferences*)accountPreferences {
    return accountPreferences;
}
-(SMUserInfos*)userInfos {
    return userInfos;
}
-(int)currentStatus {
    return currentStatus;
}
-(int)oldStatus {
    return oldStatus;
}
-(SMSearchWindowController*)searchWindow {
    return searchWindow;
}
-(SMContactListController*)contactListWindow {
    return contactList;
}



-(void)openPreferences {
    [accountPreferences showWindow];
}
-(void)changeUIN {
    if (!contactList) {
        [userInfos setUIN:[accountPreferences UIN]];
        contactList = [[SMContactListController alloc] initWithPreferences:accountPreferences parent:self];
        [self addWindowController:contactList];
        [contactList makeKeyAndOrderFront:self];
    }
    [contactList changeUIN];
}

- (NSArray*)contacts {
    return [contactList contacts];
}
-(SMUserInfos*)contactOfUIN:(NSString*)uin {
    return [contactList contactOfUIN:uin];
}
-(BOOL)isContactOnline:(NSString*)uin {
    return [contactList isContactOnline:uin];
}
-(BOOL)connected {
    return currentStatus;
}
-(BOOL)isVisible {
    return currentStatus!=7;
}
-(void)saveContactList {
    [[contactList toDict] writeToFile:[[self fileName] stringByAppendingPathComponent:@"ContactList.plist"] atomically:YES];
}
-(void)saveAccountPreferences {
    [[accountPreferences toDict] writeToFile:[[self fileName] stringByAppendingPathComponent:@"AccountPreferences.plist"] atomically:YES];
}
-(void)saveUserinfos {
    [[userInfos toDict] writeToFile:[[self fileName] stringByAppendingPathComponent:@"UserInfos.plist"] atomically:YES];

}

-(void)openHistoryManager:(id)sender uin:(NSString*)uin{
    if (!historyWindow) {
        historyWindow = [[SMHistoryManagerController alloc] initWithPreferences:accountPreferences parent:self];
    }
    [historyWindow makeKeyAndOrderFront:self];
    [historyWindow selectContactByUIN:uin];
}
-(void)openListsManager:(id)sender {
    if (!listsWindow) {
        listsWindow = [[SMListsWindowController alloc] initWithPreferences:accountPreferences document:self];
    }
    [self addWindowController:listsWindow];
     [listsWindow makeKeyAndOrderFront:self];
}
-(void)openUserDetails:(id)sender userInfos:(SMUserInfos*)user openPanel:(BOOL)open{
    if (open) {
        if (!userDetailsPanel) {
            userDetailsPanel = [[SMUserDetailsPanelController alloc] initWithPreferences:accountPreferences parent:self];
        }
        [userDetailsPanel setContact:user];
        [userDetailsPanel redisplay];
        [userDetailsPanel makeKeyAndOrderFront:self];
    } else {
        [userDetailsPanel setContact:user];
        [userDetailsPanel redisplay];
    }
}

-(void)requestLogin {
    connection = [[SMConnection alloc] initWithPreferences:accountPreferences parent:self];

}

- (void)newSearch {
    if (!searchWindow) {
            searchWindow = [[SMSearchWindowController alloc] initWithPreferences:accountPreferences document:self];// treeRoot:treeRoot];
            
    }
    [searchWindow makeKeyAndOrderFront];

}

- (void)viewMessageArchiveForUIN:(NSString*)uin {
    [self openHistoryManager:self uin:uin];
}

-(BOOL)isContactInVisibles:(NSString*)uin {
    return [listsWindow isContactInVisibles:uin];
}
-(BOOL)isContactInInvisibles:(NSString*)uin {
    return [listsWindow isContactInInvisibles:uin];
}
-(BOOL)isContactInIgnores:(NSString*)uin {
    return [listsWindow isContactInIgnores:uin];
}

-(void)setVisible:(NSString*)uin state:(BOOL)state {
    [listsWindow setVisible:uin state:state];
}
-(void)setInvisible:(NSString*)uin state:(BOOL)state{
    [listsWindow setInvisible:uin state:state];
}
-(void)setIgnore:(NSString*)uin state:(BOOL)state{
    [listsWindow setIgnore:uin state:state];
}
-(NSArray*)visibleList {
    return [listsWindow visibleList];
}
-(NSArray*)invisibleList {
    return [listsWindow invisibleList];
}
-(NSArray*)ignoreList {
    return [listsWindow ignoreList];
}
-(void)addToVisibleList:(NSString*)uin{
    [connection addToVisibleList:uin];
}
-(void)addToInvisibleList:(NSString*)uin{
    [connection addToInvisibleList:uin];
}
-(void)removeFromVisibleList:(NSString*)uin{
    [connection removeFromVisibleList:uin];
}
-(void)removeFromInvisibleList:(NSString*)uin{
    [connection removeFromInvisibleList:uin];
}
- (void) requestFullInfoForUIN:(NSString*)suin{
    [connection requestFullInfoForUIN:suin];
}
@end
