/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMConnection.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMConnection.h"
#import "SMServiceMgr.h"
#import "SMAccountPreferences.h"
#import "SMUserInfos.h"
#import "SMAccountDocument.h"

@implementation SMConnection

// init and such
-(id)initWithPreferences:(SMAccountPreferences*)prefs parent:(id)sender; {
    self = [super init];
    if (self) {
        accountPreferences = prefs;
        (SMAccountDocument*)accountDocument = sender;
        [self startICQThread];

        // register observers
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userChangedStatus:) name:@"userChangedStatus" object:nil]; 
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userConnected:) name:@"userConnected" object:nil];  
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(searchContact:) name:@"searchContact" object:nil];  
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(importServerSideList:) name:@"importServerSideList" object:nil];  
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contactAdded:) name:@"contactAdded" object:nil]; 
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newMessageSent:) name:@"newMessageSent" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(authorizationReplyTo:) name:@"authorizationReplyTo" object:nil];
    }
    return self;
}

-(void)dealloc {
    [connection setRootObject:nil];
    [connection release];
    [kernel release];
    [super dealloc];
}

- (void)startICQThread {
    NSPort *port1;
    NSPort *port2;
    NSArray *params;
    port1 = [NSPort port];
    port2 = [NSPort port];
    connection = [[NSConnection alloc] initWithReceivePort:port1 sendPort:port2];
    [connection setRootObject:self];
    /* Ports switched here. */
    params = [NSArray arrayWithObjects:port2, port1, [accountPreferences UIN], nil];
    [NSThread detachNewThreadSelector:@selector(managerWithData:) toTarget:[SMServiceMgr class] withObject:params];
}


- (void)setServer:(id)anObject {
    // NSLog(@"Kernel Thread Ready");
    [anObject setProtocolForProxy:@protocol(connectionProtocol)];
    kernel=(id <connectionProtocol>)[anObject retain];
}

-(id)serverObject {
    return kernel;
}



// accessors


// observers

- (void)userChangedStatus:(NSNotification*)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (kernel) {
            [kernel setStatus:[[[notification userInfo] objectForKey:@"status"] intValue]];
        }
    }
        
}
- (void)authorizationReplyTo:(NSNotification*)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (kernel) {
            [kernel authorizationReplyTo:[[[notification userInfo] objectForKey:@"contactUIN"]  intValue] isGiven:[[[notification userInfo] objectForKey:@"isGiven"] boolValue]];
        }
    }
        
}



- (void)searchContact:(NSNotification*)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (kernel) {
            
            [kernel searchContact:[[notification userInfo] objectForKey:@"SMUserInfos"]];
        }
    }
}

- (void)importServerSideList:(NSNotification*)notification {
  NSString *uin = [[notification userInfo] objectForKey:@"uin"];

  if ([uin isEqualToString:[accountPreferences UIN]]) {
	if (kernel) {
	  [kernel retrieveServerSideList];
	}
  }
}

- (void)contactAdded:(NSNotification*)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        if (kernel) {
            [kernel addUserToContactList:[[notification userInfo] objectForKey:@"SMUserInfos"]];
        }
    }
        
}

- (void)newMessageSent:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[accountPreferences UIN]]) {
        SMMessage *msg=[[[notification userInfo] objectForKey:@"message"] retain];
       if ([(NSNumber*)[[notification userInfo] objectForKey:@"online"] intValue]==0) {
            if (kernel) {
                [kernel sendOfflineMessage:[msg data] toUIN:[msg destUIN]];
            }
        } else {
            if (kernel) {
                [kernel sendMessage:[msg data] toUIN:[msg destUIN]];
            }
        }
        [msg release];
    }
}
- (void)requestOfflineMessages {
    if (kernel) {
            [kernel requestOfflineMessages];
        }
}



// general calls
-(void)performSelector:(SEL)aSelector target:(id)target argument:(id)anArgument {
    [target performSelector:aSelector withObject:anArgument];
}

-(oneway void)sendNotification:(NSString*) notificationName object:(id)object userInfos:(NSDictionary*)infos {
    [[NSNotificationCenter defaultCenter] postNotificationName: notificationName object:object userInfo:infos];
    
}


// special calls
- (NSDictionary*) connectionInfos {
    NSDictionary *dict;
    int status=[accountPreferences sessionLoginStatus];
    if (!status) {
        status=[accountPreferences loginStatus];
    }
    dict=[NSDictionary dictionaryWithObjectsAndKeys:
        [accountPreferences icqServerHost], @"host",
        [accountPreferences icqServerPort], @"port",
        [accountPreferences password], @"password",
        [NSNumber numberWithInt:status], @"loginstatus",
        nil];
        
    return dict;
}

-(void)queryOurInfo {
    if (kernel) {
            [kernel queryOurInfo];
        }
}

-(void)requestFullInfoForUIN:(NSString*)uin {
    if (kernel) {
        [kernel requestFullInfoForUIN:uin];
    }
}

- (void)connectionComplete {
    NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
        [accountPreferences UIN], @"uin",
        nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"userConnected" object:self userInfo:dict];
}

- (NSSet*)contacts {
    return [NSSet setWithArray:[(SMAccountDocument*)accountDocument contacts]];
}
-(NSSet*)visibleList{
    return [NSSet setWithArray:[(SMAccountDocument*)accountDocument visibleList]];
}
-(NSSet*)invisibleList{
    return [NSSet setWithArray:[(SMAccountDocument*)accountDocument invisibleList]];
}
-(NSSet*)ignoreList{
    return [NSSet setWithArray:[(SMAccountDocument*)accountDocument ignoreList]];
}

-(void)addToVisibleList:(NSString*)uin{
    if (kernel) {
            [kernel addToVisibleList:[NSSet setWithObject:uin]];
        }
}
-(void)addToInvisibleList:(NSString*)uin{
    if (kernel) {
            [kernel addToInvisibleList:[NSSet setWithObject:uin]];
        }
}
-(void)removeFromVisibleList:(NSString*)uin{
    if (kernel) {
            [kernel removeFromVisibleList:[NSSet setWithObject:uin]];
        }
}
-(void)removeFromInvisibleList:(NSString*)uin{
    if (kernel) {
            [kernel removeFromInvisibleList:[NSSet setWithObject:uin]];
        }
}
@end
