/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMDemultiplexer.m
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMServiceMgr.h"
#import "SMDemultiplexer.h"
#import "SMConnectionPackets.h"
#import "SMDataPackets.h"
#import "SMErrorPackets.h"
#import "SMSNACMgr.h"

@implementation SMDemultiplexer

- (id) init
{
    connectionReady=NO;
    lockPacket=NO;

	socket_ = nil;
    nonCompletePacket=NO;
    processing = NO;
	[awaitingData release];
    awaitingData =[[NSMutableArray alloc] init];

    return self;
}

- (void) registerConnectionPkHandler:(id)handler
{
    connectionPkHandler = (SMConnectionPackets*)handler;
}

- (void) registerErrorPkHandler:(id)handler
{
    errorPkHandler = (SMErrorPackets*)handler;
}

- (void) registerDataPkHandler:(id)handler
{
    dataPkHandler = (SMDataPackets*)handler;
}

- (void) registerSNACMgr:(id)mgr
{
    snacMgr = (SMSNACMgr*)mgr;
}

- (void) dealloc
{
  [socket_ release];
    [listeningSocket release];
    [socket release];
    [awaitingData release];
    [connectionToSocket invalidate];
    [connectionToListeningSocket invalidate];
    [connectionToSocket release];
    [connectionToListeningSocket release];
}

- (oneway void) handleData:(NSData*)data
{
  //  NSLog(@"Awaiting packets : %d", [awaitingData count]);
  [connectionPkHandler connectionAlive];
    if (!processing)
    {
      SMPackets* newPacket=[SMPackets alloc];
      processing = YES;
      if (data != nil)
      {
        newPacket = [newPacket initWithBytes:(void*)[data bytes] andSize:[data length]];
        [self handlePacket:newPacket];
      }
      else
      {
        NSData* oldData = [awaitingData objectAtIndex:0];
        newPacket = [newPacket initWithBytes:(void*)[oldData bytes] andSize:[oldData length]];
        [awaitingData removeObjectAtIndex:0];
        [self handlePacket:newPacket];
      }
      processing = NO;
      [newPacket release];
      if ([awaitingData count]>0)
        [self handleData:nil];
    }
    else
    {
      [awaitingData addObject:data];
    }
}

- (void) awaitingPacket:(NSTimer*) timer
{
  [self handlePacket:[timer userInfo]];
}

//Reassembly of multipackets data.
- (void) handlePacket:(SMPackets*)packet
{
    int command;
    int seqNum;
    [packet retain];
    seqNum = [packet get2BytesAtPosition:2];
    command = [packet getByteAtPosition:0];
	//    NSLog(@"[%02X] Nouveau paquet", seqNum);

    if (lockPacket)
    {
//      NSLog(@"[%02X] Nouveau paquet, mais on en traite deja un, la", seqNum);
      [NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(awaitingPacket:) userInfo:packet repeats:NO];
      return;
    }
    lockPacket=YES;
    //[packet debugPacket];
    //Verifying we don't have only a part of data
    // NSLog(@"command : %x", command);
	  
    if (command==0x2A&&nonCompletePacket)
    {
//      NSLog(@"[%02X] Nouvelle commande, mais on a un paquet non complet en cours de traitement", seqNum);
      [NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(awaitingPacket:) userInfo:packet repeats:NO];
      lockPacket=NO;
      return;
    }
      
    if (command==0x2A) //new data block
    {
        //Accessing important values
        unsigned int packetSize=[packet length];
        unsigned int channel=[packet getByteAtPosition:1];
        unsigned int dataSize=[packet get2BytesAtPosition:4];
	
//        NSLog(@"[%02X] Nouvelle commande", seqNum);
        //is the data complete ?
        // NSLog(@"packet infos : dataSize=%x, packetSize=%x", (dataSize+5), packetSize);
        if (dataSize+6<=packetSize) //yes
        {
          id newPacket;
          //get the data, and send it to the dedicated channel
          void* dataData; // =))
//          NSLog(@"[%02X] Le paquet est complet", seqNum);
          dataData=[packet getBytesFrom:6 length:dataSize];
          newPacket=[[SMPackets alloc] initWithBytes:dataData andSize:dataSize];
          [self distributePacket:newPacket fromChannel:channel];
          free(dataData);
          //check if there is still valid data within the packet
//          NSLog(@"[%02X] dataSize/packetSize : %02X/%02X", seqNum, dataSize, packetSize);
          if (dataSize+6!=packetSize)
          {
//            NSLog(@"[%02X] Le paquet comporte une suite avec la commande %02X%02X%02X", seqNum, [packet getByteAtPosition:(dataSize+5)], [packet getByteAtPosition:(dataSize+6)], [packet getByteAtPosition:(dataSize+7)]);
            if ([packet getByteAtPosition:(dataSize+6)]==0x2A)
            {
              id secondPart;
              void* dataData;
//              NSLog(@"[%02X] Celle ci est valide", seqNum);
              dataData=[packet getBytesFrom:(dataSize+6) length:(packetSize-(dataSize+6))];
              secondPart=[[SMPackets alloc] initWithBytes:dataData andSize:(packetSize-(dataSize+6))];
//              [secondPart debugPacket];
              lockPacket = NO;
              [self handlePacket:secondPart];
              lockPacket = YES;
              free(dataData);
            }
            else
            {
              id secondPart;
              void* dataData;
//              NSLog(@"[%02X] Suite non valide", seqNum);
              dataData=[packet getBytesFrom:(dataSize+6) length:(packetSize-(dataSize+6))];
              secondPart=[[SMPackets alloc] initWithBytes:dataData andSize:(packetSize-(dataSize+6))];
//              [secondPart debugPacket];
              free(dataData);
            }
          }
          else
          {
//            NSLog(@"[%02X] On est au bout du paquet", seqNum);
          }
        }
        else  //no -> we save it for later
        {
//          NSLog(@"[%02X] Non complete packet", seqNum);
//          NSLog(@"[%02X] dataSize/packetSize : %02X/%02X", seqNum, dataSize, packetSize);
          nonCompletePacket=YES;
          savedPacket=[packet retain];
        }
    }
    else //we reassemble the data
    {
      if (nonCompletePacket)
      {
        SMPackets* newPacket=[[SMPackets alloc] init];
//        NSLog(@"[%02X] En train de completer le paquet en stock", seqNum);
        [newPacket appendPacket:savedPacket];
        [newPacket appendPacket:packet];
        // [newPacket debugPacket];
        nonCompletePacket=NO;
        [savedPacket release];
        lockPacket = NO;
        [self handlePacket:newPacket];
        lockPacket = YES;
        [newPacket release];
      }
      else
      {
//        NSLog(@"[%02X] Fin de paquet sans debut :(", seqNum);
        [NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(awaitingPacket:) userInfo:packet repeats:NO];
      }
    }
    [packet release];
    lockPacket=NO;
}


//Repartition of the incoming packet to the upper layer of the kernel
- (void) distributePacket:(SMPackets*)packet fromChannel:(unsigned int)channel
{
    // NSLog([NSString stringWithFormat:@"Packet destinated to channel : %d", channel]);
    switch (channel)
    {
        case 1 : //connection channel
        {
            [connectionPkHandler handleIncomingConnectionPacket:packet];
            break;
        }
        case 2 : //data channel
        {
            [snacMgr handleIncomingPacket:packet];
            break;
        }
        case 3 : //error channel
        {
            //[errorPkHandler handleIncomingPacket:packet];
            break;
        }
        case 4 : //disconnection channel
        {
            [connectionPkHandler handleIncomingDisconnectionPacket:packet];
            break;
        }
    }
}

//Forward the packet to the socket through the NSConnection
//should we cut packets here if they are too big ?
- (void) sendPacket:(SMPackets*)packet
{
    [packet retain];
    // NSLog(@"Sending packet :");
    //[packet debugPacket];
//    [socket sendPacket:packet];
	if ([socket_ sendData:[packet dataBytes]])
	  [connectionPkHandler connectionAlive];
	else
	  [connectionPkHandler error];
    [packet release];
}


//Socket Management
- (void) connectSocketToHost:(NSString*)host onPort:(unsigned int)port
{
    //NSLog([NSString stringWithFormat:@"Connecting to %@:%d", host, port]);
  socket_ = [[SMSocketHandle alloc] initToConnectToTCPPort:port onHost:host];
  [socket_ setDelegate:self];
  [socket_ connect];
}

- (void) closeSocket
{
  [socket_ close];
  [socket_ release];
}

- (oneway void) error
{
  [connectionPkHandler error];
}

- (oneway void) connectionAlive
{
  [connectionPkHandler connectionAlive];
}

@end
