/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMConnectionPackets.m
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMConnectionPackets.h"
#import "SMDemultiplexer.h"
#import "SMServiceMgr.h"
#import "SMPackets.h"

@implementation SMConnectionPackets

static const unsigned char passwordEncoding[] = {
    0xF3, 0x26, 0x81, 0xC4, 0x39, 0x86, 0xDB, 0x92, 0x71, 0xA3, 0xB9, 0xE6, 0x53, 0x7A, 0x95, 0x7C
};

- (id) initWithDemultiplexer:(id)theDemultiplexer andManager:(id)theManager
{
    // NSLog(@"SMConnectionPackets init");

    manager = (SMServiceMgr*)theManager;
    demultiplexer = (SMDemultiplexer*)theDemultiplexer;

    firstLogin=YES;
    
    return self;
}

- (void) dealloc
{
    if (icqServer != nil)
      [icqServer release];
    if (cookie != nil)
      [cookie release];
}

- (void) connectUIN:(unsigned int)uin password:(NSString*)password
{
    int i;
    SMPackets* connectionPacket=[[SMPackets alloc] initForChannel:1 withParams:[manager seqNum]];
    char cEPassword[[password length]];
    const char* cPassword=[password cString];
    [password retain];

    //NSLog([NSString stringWithFormat:@"Will attempt to connect user %d with password %@", uin, password]);
    
    for (i=0;i<[password length];i++)
    {
        cEPassword[i] = cPassword[i] ^ passwordEncoding[i];
    }

    [connectionPacket set2Bytes:0 atPosition:[connectionPacket length]];
    [connectionPacket set2Bytes:1 atPosition:[connectionPacket length]];

    [connectionPacket addTLV:1 withSize:-1];
    [connectionPacket setBWString:[NSString stringWithFormat:@"%d", uin] atPosition:[connectionPacket length]];

    [connectionPacket addTLV:2 withSize:[password length]];

    for (i=0;i<[password length];i++)
    {
        [connectionPacket setByte:(unsigned char)(cEPassword[i]) atPosition:[connectionPacket length]];
    }

    [connectionPacket addTLV:3 withSize:-1];
    [connectionPacket setBWString:@"bombadil project - copyright Rapidonet" atPosition:[connectionPacket length]];

    [connectionPacket addTLV:0x16 withSize:2];
    [connectionPacket set2Bytes:0x1B0A atPosition:[connectionPacket length]];
    [connectionPacket addTLV:0x17 withSize:2];
    [connectionPacket set2Bytes:0x03EA atPosition:[connectionPacket length]];
    [connectionPacket addTLV:0x18 withSize:2];
    [connectionPacket set2Bytes:0x3C atPosition:[connectionPacket length]];
    [connectionPacket addTLV:0x19 withSize:2];
    [connectionPacket set2Bytes:1 atPosition:[connectionPacket length]];
    [connectionPacket addTLV:0x1A withSize:2];
    [connectionPacket set2Bytes:7 atPosition:[connectionPacket length]];

    [connectionPacket addTLV:0x14 withSize:4];
    [connectionPacket set4Bytes:0x55 atPosition:[connectionPacket length]];

    [connectionPacket addTLV:0x0F withSize:-1];
    [connectionPacket setBWString:@"en" atPosition:[connectionPacket length]];

    [connectionPacket addTLV:0x0E withSize:-1];
    [connectionPacket setBWString:@"us" atPosition:[connectionPacket length]];

    
    [demultiplexer sendPacket:connectionPacket];

    [connectionPacket release];
    [password release];    
    
}

- (void) connectionComplete
{
}


- (void) handleIncomingConnectionPacket:(id)packet
{
    int family;
    int type;

    family = [packet get2BytesAtPosition:0];
    type = [packet get2BytesAtPosition:2];

    //NSLog([NSString stringWithFormat:@"Connection Packet (%x/%x)", family, type]);

    //In oscar, each command has a family and a type
    //This is the only one command I know on channel 1
    switch (family)
    {
        case 0 :
        {
            switch (type)
            {
                case 1 :
                {
                    //Server is waiting for our login
                    //first connection -> login
                    if (firstLogin)
                        [manager initiateConnection];
                    //second connection -> cookie
                    else
                    {
                        [self loginWithCookie];
                        firstLogin=YES;
                    }
                    break;
                }
                default :
                {
                    NSLog(@"Unknown Connection Packet :");
                    //[packet debugPacket];
                }
            }
            break;
        }
        default :
        {
            NSLog(@"Unknown Connection Packet :");
            //[packet debugPacket];
        }
    }
}

- (void) handleIncomingDisconnectionPacket:(id)packet
{
    int i;
    int tlvValue = [packet getCurrentTLV];

    //connection error
    if (tlvValue==0x01)
    {
        tlvValue = [packet goToNextTLV];
        //   [packet debugCurrentTLV];
        if ((tlvValue==4)||(tlvValue==8))
        {
            for (i=0;i<2;i++)
            {
                if (tlvValue==4)
                    tlvValue = [packet goToNextTLV];
                if (tlvValue==8)
                {
                    NSString * errorstring;
                    //          short size = [packet getCurrentTLVSize];
                    short *errorcode = [packet getDataOfCurrentTLV];

                    switch (*errorcode)
                    {
                        case 0x0001: errorstring = [NSString stringWithFormat:NSLocalizedString(@"No Password provided or invalid UIN", @"")];
                            break;
                        case 0x0004:
                        case 0x0005: errorstring = [NSString stringWithFormat:NSLocalizedString(@"Password error", @"")];
                            break;
                        case 0x0007:
                        case 0x0008: errorstring = [NSString stringWithFormat:NSLocalizedString(@"This ICQ# doesn't exist", @"")];
                            break;
                        case 0x0015:
                        case 0x0016: errorstring = [NSString stringWithFormat:NSLocalizedString(@"Num of UINs connected from 1 IP reached maximum", @"")];
                            break;
                        case 0x0018:
                        case 0x001D: errorstring = [NSString stringWithFormat:NSLocalizedString(@"Rate limit exceeded", @"")];
                            break;
                        case 0x001B: errorstring = [NSString stringWithFormat:NSLocalizedString(@"You are using an older version of ICQ. Upgrade", @"")];
                            break;
                        case 0x001E: errorstring = [NSString stringWithFormat:NSLocalizedString(@"Can't register on ICQ network. Please try again", @"")];
                            break;
                        default: errorstring = [NSString stringWithFormat:NSLocalizedString(@"Unknown error", @"")];
                            break;
                    }
                    
                    [manager connectionError:errorstring];
                    NSLog(@"Connection Error : ");
                    NSLog(errorstring);
                    firstLogin=YES;
                    tlvValue = [packet goToNextTLV];
                }
            }
        }
        else if (tlvValue==5) //Connection ok, receiving server IP and cookie packet
        {
            void* hostData=[packet getDataOfCurrentTLV];
            NSArray* twoStrings;
            NSString* host;
            void* cookieData;
            unsigned short hostSize=[packet getCurrentTLVSize];

            //retrieving IP/Port
            host=[NSString stringWithCString:hostData length:hostSize];
            twoStrings=[host componentsSeparatedByString:@":"];

            //retrieving cookie
            [packet goToNextTLV];
            cookieData=[packet getDataOfCurrentTLV];
            cookie = (SMPackets*)[[SMPackets alloc] initWithBytes:cookieData andSize:[packet getCurrentTLVSize]];
            firstLogin=NO;

            //logout from this server, login to the new one
            [demultiplexer closeSocket];
            [demultiplexer init];
            icqServer = [twoStrings retain];
            [NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(secondConnection:) userInfo:nil repeats:NO];

            //freeing memory
            free(hostData);
            free(cookieData);
        }
    }
    else if (tlvValue==0x09)
    {
        NSString * errorstring;
        //      short size = [packet getCurrentTLVSize];
        short *errorcode = [packet getDataOfCurrentTLV];

        switch (*errorcode)
        {
            case 0x0001: errorstring = [NSString stringWithFormat:@"Another client logged int with your UIN"];
                break;
            default: errorstring = [NSString stringWithFormat:@"Unknown error"];
                break;
        }
        NSLog(@"Connection Error : ");
        NSLog(errorstring);
        [manager connectionError:errorstring];
    }
}

- (void) loginWithCookie
{
    SMPackets* cookiePacket=[[SMPackets alloc] initForChannel:1 withParams:[manager seqNum]];
    [cookiePacket set2Bytes:0 atPosition:[cookiePacket length]];
    [cookiePacket set2Bytes:1 atPosition:[cookiePacket length]];
    [cookiePacket addTLV:6 withSize:[(SMPackets*)cookie length]];
    [cookiePacket appendPacket:cookie];

    [demultiplexer sendPacket:cookiePacket];
    [cookiePacket release];
    [cookie release];
}

- (void) secondConnection:(NSTimer*) timer
{
    [demultiplexer connectSocketToHost:[icqServer objectAtIndex:0] onPort:atoi([[icqServer objectAtIndex:1] cString])];
    [icqServer release];
}


- (void) connectionAlive
{
  [manager connectionAlive];
}

- (void) error
{
  [manager error];
}

@end
