/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMDataPackets.m
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2002 iXQ-Team. All rights reserved.
//

#import "SMDataPackets.h"
#import "SMDemultiplexer.h"
#import "SMSNACMgr.h"
#import "SMServiceMgr.h"
#import <sys/types.h>
#import <sys/socket.h>
#import <netinet/in.h>
#import <netdb.h>
#import <arpa/inet.h>
#import <stdio.h>
#import <unistd.h>
#import <sys/wait.h>
#import <sys/time.h>


@implementation SMDataPackets

static const unsigned long statusTable[] = { 0, 1, 4, 0x10, 0x13, 0x20, 0x100 };

- (id) initWithDemultiplexer:(id)theDemultiplexer andManager:(id)theManager
{
  // NSLog(@"SMDataPackets init");

  manager = (SMServiceMgr*)theManager;
  demultiplexer = (SMDemultiplexer*)theDemultiplexer;
  snacMgr= (SMSNACMgr*)[[SMSNACMgr alloc] initWithDemultiplexer:demultiplexer andHandler:self];

  [demultiplexer registerSNACMgr:snacMgr];

  return self;
}

- (void) dealloc
{
  [snacMgr release];
}

- (void) sendClientTypeNotification
{
  SMPackets* clientTypePacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [clientTypePacket addSnacCommandOfFamily:1 withType:0x17];

  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x03 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x02 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x03 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x15 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x04 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x06 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x09 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x0A atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x0B atPosition:[clientTypePacket length]];
  [clientTypePacket set2Bytes:0x01 atPosition:[clientTypePacket length]];

  [demultiplexer sendPacket:clientTypePacket];

  [clientTypePacket release];
}

- (void) sendRateInfosRequest
{
  SMPackets* rateInfoRequestPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [rateInfoRequestPacket addSnacCommandOfFamily:0x01 withType:0x06];

  [demultiplexer sendPacket:rateInfoRequestPacket];
  [rateInfoRequestPacket release];
}

- (void) sendRateInfosAck
{
  SMPackets* rateInfoAckPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [rateInfoAckPacket addSnacCommandOfFamily:0x01 withType:0x08];

  [rateInfoAckPacket set2Bytes:0x01 atPosition:[rateInfoAckPacket length]];
  [rateInfoAckPacket set2Bytes:0x02 atPosition:[rateInfoAckPacket length]];
  [rateInfoAckPacket set2Bytes:0x03 atPosition:[rateInfoAckPacket length]];
  [rateInfoAckPacket set2Bytes:0x04 atPosition:[rateInfoAckPacket length]];
  [rateInfoAckPacket set2Bytes:0x05 atPosition:[rateInfoAckPacket length]];

  [demultiplexer sendPacket:rateInfoAckPacket];
  [rateInfoAckPacket release];
}

- (void) requestPersoInfos
{
  SMPackets* persoInfosPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [persoInfosPacket addSnacCommandOfFamily:0x01 withType:0x0E];

  [demultiplexer sendPacket:persoInfosPacket];
  [persoInfosPacket release];
}

- (void) requestServerSideContacts
{
  SMPackets* serverSidePacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [serverSidePacket addSnacCommandOfFamily:0x13 withType:0x04];

  [demultiplexer sendPacket:serverSidePacket];
  [serverSidePacket release];
}

- (void) requestRights
{
  SMPackets* rightsPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [rightsPacket addSnacCommandOfFamily:0x02 withType:0x02];
  [demultiplexer sendPacket:rightsPacket];

  rightsPacket=[rightsPacket initForChannel:2 withParams:[manager seqNum]];
  [rightsPacket addSnacCommandOfFamily:0x03 withType:0x02];
  [demultiplexer sendPacket:rightsPacket];

  rightsPacket=[rightsPacket initForChannel:2 withParams:[manager seqNum]];

  [rightsPacket addSnacCommandOfFamily:0x04 withType:0x04];
  [demultiplexer sendPacket:rightsPacket];

  rightsPacket=[rightsPacket initForChannel:2 withParams:[manager seqNum]];
  [rightsPacket addSnacCommandOfFamily:0x09 withType:0x02];

  [demultiplexer sendPacket:rightsPacket];

  [rightsPacket release];
}

- (void) changeStatus
{
  SMPackets* changeStatusPacket;
  
  // when going invisible we send the visible list (when going visible we send the invisible list)
  if ([manager newStatus] == 6) 	// invisible
    [manager addToVisibleList:[manager visible]];
  else					// visible	
    [manager addToInvisibleList:[manager invisible]];
  
   changeStatusPacket = [[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
    
  [changeStatusPacket addSnacCommandOfFamily:0x01 withType:0x1E];
  [changeStatusPacket addTLV:6 withSize:4];
  [changeStatusPacket set4Bytes:statusTable[(int)[manager newStatus]] atPosition:[changeStatusPacket length]];

  [changeStatusPacket addTLV:8 withSize:2];
  [changeStatusPacket set2Bytes:0x07 atPosition:[changeStatusPacket length]];

  [changeStatusPacket addTLV:0xC withSize:0x25];
  [changeStatusPacket set4Bytes:gethostid() atPosition:[changeStatusPacket length]];
  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];

  [changeStatusPacket setByte:04 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set2Bytes:0x0004 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set2Bytes:0xffff atPosition:[changeStatusPacket length]];
  [changeStatusPacket set2Bytes:0xa45f atPosition:[changeStatusPacket length]];

  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set4Bytes:0 atPosition:[changeStatusPacket length]];
  [changeStatusPacket set2Bytes:0x2100 atPosition:[changeStatusPacket length]];

  [demultiplexer sendPacket:changeStatusPacket];
  [changeStatusPacket release];
  [manager statusChanged:YES];
}

- (void) serverRequestsYourContactList
{
  NSSet* contacts = [manager contacts];
  [contacts retain];
  [self sendContactList:contacts];
  [contacts release];
}

- (void) sendContactList:(NSSet*)cl
{
  SMUserInfos* uin;
  SMPackets* contactsList=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
  NSEnumerator* enumerator = [cl objectEnumerator];

  [cl retain];
  //Adding the snac Command
  [contactsList addSnacCommandOfFamily:3 withType:4];

  //and each uin
  while ((uin = [enumerator nextObject]))
  {
    // NSLog([NSString stringWithFormat:@"Adding uin : %@", [uin UIN]]);
    [contactsList setBWString:[uin UIN] atPosition:[contactsList length]];
  }

  //then sending the packet
  [demultiplexer sendPacket:contactsList];
  [cl release];
  [contactsList release];
}


- (void) setUserInfos
{
  SMPackets* setUserInfosPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [setUserInfosPacket addSnacCommandOfFamily:0x02 withType:0x04];

  [setUserInfosPacket addTLV:5 withSize:0x40];
  //0946 1344 4C7F 11D1 8222 4445 5354 0000

  /*
   09 46 13 49 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_ICQSERVERRELAY
   09 46 13 4e 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_2002a
   97 b1 27 51 24 3c 43 34 ad 22 d6 ab f7 3f 14 92 // AIM_CAPS_ICQRTF
   09 46 13 44 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_ICQ
  //{0x09, 0x46, 0x13, 0x4d, 0x4c, 0x7f, 0x11, 0xd1,
   0x82, 0x22, 0x44, 0x45, 0x53, 0x54, 0x00, 0x00}},
   */

  /*
  //  09 46 13 49 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_ICQSERVERRELAY
  [setUserInfosPacket set2Bytes:0x0946 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x134d atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4C7F atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x11D1 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x8222 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4445 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x5354 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x0000 atPosition:[setUserInfosPacket length]];
  */
  //  09 46 13 49 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_ICQSERVERRELAY
  [setUserInfosPacket set2Bytes:0x0946 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x1349 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4C7F atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x11D1 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x8222 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4445 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x5354 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x0000 atPosition:[setUserInfosPacket length]];

  //  09 46 13 4e 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_2002a
  [setUserInfosPacket set2Bytes:0x0946 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x134E atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4C7F atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x11D1 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x8222 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4445 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x5354 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x6969 atPosition:[setUserInfosPacket length]];

  //  97 b1 27 51 24 3c 43 34 ad 22 d6 ab f7 3f 14 92 // AIM_CAPS_ICQRTF
  [setUserInfosPacket set2Bytes:0x97B1 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x2751 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x243C atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4334 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0xAD22 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0xD6AB atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0xF73F atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x1492 atPosition:[setUserInfosPacket length]];

  //	09 46 13 44 4c 7f 11 d1 82 22 44 45 53 54 00 00 // AIM_CAPS_ICQ
  [setUserInfosPacket set2Bytes:0x0946 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x1344 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4C7F atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x11D1 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x8222 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x4445 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x5354 atPosition:[setUserInfosPacket length]];
  [setUserInfosPacket set2Bytes:0x0000 atPosition:[setUserInfosPacket length]];

  [demultiplexer sendPacket:setUserInfosPacket];
  [setUserInfosPacket release];
}

- (void) sendUINList:(NSSet*)list subtype:(int)subtype
{
  NSString* uin;
  SMPackets* uinList=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
  NSEnumerator* enumerator = [list objectEnumerator];

  [list retain];

  [uinList addSnacCommandOfFamily:9 withType:subtype]; // SNAC(9,subtype)

  while ((uin = [enumerator nextObject]))
    [uinList setBWString:uin atPosition:[uinList length]];	// Add NSSet Object to UIN-LIST

  [demultiplexer sendPacket:uinList]; // send our packet
                                            // cleanup
  [uinList release];
  [list release]; 
}

- (void) clientReady
{
  SMPackets* clientReadyPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
 
  [clientReadyPacket addSnacCommandOfFamily:0x01 withType:0x02];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0003 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0002 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b01 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0003 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0015 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0004 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];

  [clientReadyPacket set2Bytes:0x0006 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];

  [clientReadyPacket set2Bytes:0x0009 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x0001 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x1b10 atPosition:[clientReadyPacket length]];
  [clientReadyPacket set2Bytes:0x03de atPosition:[clientReadyPacket length]];
  [clientReadyPacket setByte:0x00 atPosition:[clientReadyPacket length]];

  [demultiplexer sendPacket:clientReadyPacket];
  [clientReadyPacket release];
}

- (void) requestFullInfoForUIN:(int)uin withUIN:(int)localUIN
{
    SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

    lastDetailsUIN = uin;
    
    [searchUINPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];		// SNAC(15,2)
    [searchUINPacket addTLV:1 withSize:0x10];						// TLV(1)
    [searchUINPacket set2Bytes:NSSwapShort(0x0E) atPosition:[searchUINPacket length]];	// remaining bytes
    [searchUINPacket set4Bytes:NSSwapLong(localUIN) atPosition:[searchUINPacket length]];	// our UIN
    [searchUINPacket set2Bytes:(0xD007) atPosition:[searchUINPacket length]];		// message type (multifunction)
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];			// request id
    [searchUINPacket set2Bytes:(0xB204) atPosition:[searchUINPacket length]];		// message subtype (request full info)
    [searchUINPacket set4Bytes:NSSwapLong(uin) atPosition:[searchUINPacket length]];	// user to request info 

    [demultiplexer sendPacket:searchUINPacket];
    [searchUINPacket release];
}

//Contact search functions
- (void) searchForUIN:(int)uin withUIN:(int)localUIN
{
  SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [searchUINPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];		// SNAC(15,2)
  [searchUINPacket addTLV:1 withSize:0x10];						// TLV(1)
  [searchUINPacket set2Bytes:NSSwapShort(0x0E) atPosition:[searchUINPacket length]];	// remaining bytes
  [searchUINPacket set4Bytes:NSSwapLong(localUIN) atPosition:[searchUINPacket length]]; // our UIN
    [searchUINPacket set2Bytes:(0xD007) atPosition:[searchUINPacket length]];		// message type (multifunction)
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];			// request id
    [searchUINPacket set2Bytes:(0x1F05) atPosition:[searchUINPacket length]];		// message subtype (request whitepage-info by UIN)
    [searchUINPacket set4Bytes:NSSwapLong(uin) atPosition:[searchUINPacket length]];	// user to request info 

    [demultiplexer sendPacket:searchUINPacket];
    [searchUINPacket release];
}

- (void) searchForUserWithNickname:(NSString*)nick firstName:(NSString*)first lastname:(NSString*)last fromUIN:(int)localUIN
{
  SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
  int taille=0x15+[nick length]+[first length]+[last length];	// taille is the variable length of the TLV(1)

  [last retain];
  [first retain];
  [nick retain];

  [searchUINPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
  [searchUINPacket addTLV:1 withSize:taille];							// TLV(1)
  [searchUINPacket set2Bytes:NSSwapShort(taille-2) atPosition:[searchUINPacket length]];	// remaining bytes
  [searchUINPacket set4Bytes:NSSwapLong(localUIN) atPosition:[searchUINPacket length]];		// our UIN
  [searchUINPacket set2Bytes:(0xD007) atPosition:[searchUINPacket length]];			// message type (multifunction)
  [searchUINPacket set2Bytes:0x00 atPosition:[searchUINPacket length]];				// request id
  [searchUINPacket set2Bytes:(0x1505) atPosition:[searchUINPacket length]];			// message subtype (short whitepage request)

  /*
    wp-short-request
   LNTS    first
   LNTS    last
   LNTS    nick 
   */
  
  [searchUINPacket setLNTString:first atPosition:[searchUINPacket length]]; 			// first name
  [searchUINPacket setLNTString:last atPosition:[searchUINPacket length]]; 			// last name
  [searchUINPacket setLNTString:nick atPosition:[searchUINPacket length]]; 			// nick name
	

  [demultiplexer sendPacket:searchUINPacket]; // send our packet
                                              // cleanup
  [last release];
  [first release];
  [nick release];
  [searchUINPacket release];
}

- (void) searchExtendedForUser:(SMUserInfos*)ui fromUIN:(int)localUIN
{
    SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
    // taille is the variable length of the TLV(1)  
    int taille=0x15+[[ui nickname] length]+[[ui firstName] length]+[[ui lastName] length]+[[ui mail] length]+[[ui city] length]+[[ui state] length]+48;
            
    //NSLog([NSString stringWithFormat:@"extendet search: nick:'%@' first:'%@' last:'%@' mail:'%@' city:'%@' state:'%@' min:'%d' max:'%d' sex:'%d' lang:'%d' country:'%d' online:'%d'", [ui nickname], [ui firstName], [ui lastName], [ui mail], [ui city], [ui state], [ui minimalAge], [ui maximalAge], [ui sex], [ui language], [ui country], [ui onlineOnly]]);
    
                
    [searchUINPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
    [searchUINPacket addTLV:1 withSize:taille];							// TLV(1)
    [searchUINPacket set2Bytes:NSSwapShort(taille-2) atPosition:[searchUINPacket length]];	// remaining bytes
    [searchUINPacket set4Bytes:NSSwapLong(localUIN) atPosition:[searchUINPacket length]];		// our UIN
    [searchUINPacket set2Bytes:(0xD007) atPosition:[searchUINPacket length]];			// message type (multifunction)
    [searchUINPacket set2Bytes:(0x0500) atPosition:[searchUINPacket length]];			// request id
    [searchUINPacket set2Bytes:(0x3305) atPosition:[searchUINPacket length]];			// message subtype (full whitepage request)
    
    /*
        * wp-full-request
    wp-short-request-info
    LNTS    email
    WORD    (LE) minimum age, 0 if disabled
    WORD    (LE) maximum age, 0 if disabled
    BYTE    sex (0=disabled, other=see table)
    BYTE    language (0=disabled, other=see table)
    LNTS    city
    LNTS    state
    WORD    country (0=disabled, other=see table)
    LNTS    company-name
    LNTS    department
    LNTS    position
    BYTE    occupation field (0=disabled)
    WORD    past information category (0=disabled, other=see table)
    LNTS    desc
    WORD    interests-category (0=disabled, other=see table)
    LNTS    interests-specific (comma separated)
    WORD    affiliation/organization (0=disabled, other=see table)
    LNTS    desc
    WORD    homepage category
    LNTS    desc
    BYTE    only-online-users, (0=off, 1=on)  
    */

    [searchUINPacket setLNTString:[ui firstName] atPosition:[searchUINPacket length]]; 			// first name
    [searchUINPacket setLNTString:[ui lastName] atPosition:[searchUINPacket length]]; 			// last name
    [searchUINPacket setLNTString:[ui nickname] atPosition:[searchUINPacket length]]; 			// nick name
    [searchUINPacket setLNTString:[ui mail] atPosition:[searchUINPacket length]]; 			// email
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)[ui minimalAge]) atPosition:[searchUINPacket length]]; 	// minimum age
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)[ui maximalAge]) atPosition:[searchUINPacket length]];	// maximum age
    [searchUINPacket setByte:[ui sex] 	     atPosition:[searchUINPacket length]];			// sex
    [searchUINPacket setByte:[ui language]     atPosition:[searchUINPacket length]];			// language
    [searchUINPacket setLNTString:[ui city] atPosition:[searchUINPacket length]]; 			// city
    [searchUINPacket setLNTString:[ui state] atPosition:[searchUINPacket length]]; 			// state
    [searchUINPacket set2Bytes:[ui country] atPosition:[searchUINPacket length]];			// country
    
    // not supported: company, department, position
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]]; 
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];
    
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];				// occupation:  not suported, thus 0 (disabled)
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];				// pastinformation: not suported, thus 0 (disabled)
    
    // desc, what desc??
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];
    
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];				// interest: not suported, thus 0 (disabled)
    
    // not supported: interests-specific       
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];
    
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];				// organisation: not suported, thus 0 (disabled)
    
    // desc, what desc??
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];
    
    [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];				// homepagecategory: not suported, thus 0 (disabled)
    
    // desc, what desc??
    [searchUINPacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];                                                 
    
    
    
    // Julian : it crashes when it uses the line below...
    //[searchUINPacket setByte:[ui onlineOnly] atPosition:[searchUINPacket length]];
    [searchUINPacket setByte:0 atPosition:[searchUINPacket length]];				// only online users?
    
    [demultiplexer sendPacket:searchUINPacket]; 	// send our packet
    [searchUINPacket release];			// cleanup
}

- (void) testWithSearchForUIN:(int)uin withUIN:(int)localUIN
{
  SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [searchUINPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];		// SNAC(15,2)
  [searchUINPacket addTLV:1 withSize:0x10];						// TLV(1)
  [searchUINPacket set2Bytes:NSSwapShort(0x0E) atPosition:[searchUINPacket length]];	// remaining bytes
  [searchUINPacket set4Bytes:NSSwapLong(localUIN) atPosition:[searchUINPacket length]]; // our UIN
  [searchUINPacket set2Bytes:(0xD007) atPosition:[searchUINPacket length]];		// message type (multifunction)
  [searchUINPacket set2Bytes:6969 atPosition:[searchUINPacket length]];			// request id
  [searchUINPacket set2Bytes:(0x1F05) atPosition:[searchUINPacket length]];		// message subtype (request whitepage-info by UIN)
  [searchUINPacket set4Bytes:NSSwapLong(uin) atPosition:[searchUINPacket length]];	// user to request info

  [demultiplexer sendPacket:searchUINPacket];
  [searchUINPacket release];
}

- (void) testWithKeepAlive
{
  SMPackets* searchUINPacket=[[SMPackets alloc] initForChannel:5 withParams:[manager seqNum]];

  [searchUINPacket set2Bytes:0 atPosition:[searchUINPacket length]];
  [searchUINPacket set4Bytes:0 atPosition:[searchUINPacket length]];
  [searchUINPacket set4Bytes:time(0L) atPosition:[searchUINPacket length]];
  [searchUINPacket set4Bytes:0 atPosition:[searchUINPacket length]];
  
  [demultiplexer sendPacket:searchUINPacket];
  [searchUINPacket release];
}

- (void) searchResults:(SMPackets*)packet
{
    static  SMUserInfos * infos;    
    int subType=[packet get4BytesAtPosition:9]&0x00FFFFFF;
    int ID=[packet get2BytesAtPosition:8];

    if (ID == 6969)
    {
      [manager connectionAlive];
      return;
    }

    [packet retain];
    if (subType==0x9A010A||subType==0x90010A||subType==0xAE010A||subType==0xA4010A)
    {
        char auth, status; //, gender, age;
        
        int uin=NSSwapLong([packet get4BytesAtPosition:15]);
        
        unsigned short nickSize=NSSwapShort([packet get2BytesAtPosition:19])-1;
        unsigned short firstSize=NSSwapShort([packet get2BytesAtPosition:(22+nickSize)])-1;
        unsigned short lastSize=NSSwapShort([packet get2BytesAtPosition:(25+nickSize+firstSize)])-1;
        unsigned short mailSize=NSSwapShort([packet get2BytesAtPosition:(28+nickSize+firstSize+lastSize)])-1;
        
        infos = [[SMUserInfos alloc] initWithUIN:[[NSNumber numberWithInt:uin] stringValue] nickname:@""];
      
        auth = [packet getByteAtPosition:(31+nickSize+firstSize+lastSize+mailSize)];
        status = [packet getByteAtPosition:(32+nickSize+firstSize+lastSize+mailSize)];
        
        // those are in docu but dont seem to get sent ;(
        /*gender = [packet getByteAtPosition:(34+nickSize+firstSize+lastSize+mailSize)];
        if (gender != 0)
        {
            [infos setSex:gender];
            NSLog([NSString stringWithFormat:@"gender: '%d'", gender]);
        }
        age = [packet getByteAtPosition:(35+nickSize+firstSize+lastSize+mailSize)];
        if (age != 0)
        {
            [infos setAge:age];
            NSLog([NSString stringWithFormat:@"age: '%d'", age]);
        }*/
    
        [infos setNickname:[NSString stringWithCString:[packet getBytesFrom:21 length:nickSize] length:nickSize]];
        [infos setFirstName:[NSString stringWithCString:[packet getBytesFrom:(24+nickSize) length:firstSize] length:firstSize]];
        [infos setLastName:[NSString stringWithCString:[packet getBytesFrom:(27+nickSize+firstSize) length:lastSize] length:lastSize]];
        [infos setMail:[NSString stringWithCString:[packet getBytesFrom:(30+nickSize+firstSize+lastSize) length:mailSize] length:mailSize]];
    
//        NSLog([NSString stringWithFormat:@"found user: nick:'%@' first:'%@' last:'%@' mail:'%@' auth:'%d' status:'%d' gender:'%d' age:'%d'", [infos nickname], [infos firstName], [infos lastName], [infos mail], auth, status, gender, age]);
    
        [manager userFound:infos];
        [infos release];
    }
    [packet release];
}

- (void) modificationResults:(SMPackets*)packet
{
    static BOOL done [3];
    static int times = 0;
    int i;
    int subType=[packet get4BytesAtPosition:9]&0x00FFFFFF;
    
    if (times == 0)
    {
        for (i=0;i<3;i++)
            done[i] = false;
    }
    times ++;

    switch (subType)
    {
        case 0x64000A:	// ack to modify info (main/home)
        {
            done[0] = true;
            break;
        }
        case 0x78000A:	// ack to modify info (homepage/more)
        {
            done[1] = true;
            break;
        }
        case 0x82000A:	// ack to modify info (about)
        {
            done[2] = true;
            break;
        }
    }
    for (i=0;i<3;i++)
    {
        if (done[i] == false)
        {
            //NSLog(@"returning");
            return;
        }
    }

    times = 0;
    return  [manager infoModificationSucceeded];    
}

- (void) fullInfoResults:(SMPackets*)packet
{
    static BOOL done [8]; 
    static  SMUserInfos * infos;    
    static int times = 0;
    int subType=[packet get4BytesAtPosition:9]&0x00FFFFFF;
    int i;
    
    if (times == 0)
    {
        infos = [[SMUserInfos alloc] initWithUIN:[[NSNumber numberWithInt:lastDetailsUIN] stringValue] nickname:@""];
        for (i=0;i<8;i++)
            done[i] = false;
    }
    times ++;
        
    [packet retain];
    switch (subType)
    {
        case 0xC8000A:	// main-home-info
        {
            unsigned short nickSize=NSSwapShort([packet get2BytesAtPosition:13])-1;
            unsigned short firstSize=NSSwapShort([packet get2BytesAtPosition:(16+nickSize)])-1;
            unsigned short lastSize=NSSwapShort([packet get2BytesAtPosition:(19+nickSize+firstSize)])-1;
            unsigned short mailSize=NSSwapShort([packet get2BytesAtPosition:(22+nickSize+firstSize+lastSize)])-1; 
            unsigned short citySize=NSSwapShort([packet get2BytesAtPosition:(25+nickSize+firstSize+lastSize+mailSize)])-1; 
            unsigned short stateSize=NSSwapShort([packet get2BytesAtPosition:(28+nickSize+firstSize+lastSize+mailSize+citySize)])-1; 
            unsigned short phoneSize=NSSwapShort([packet get2BytesAtPosition:(31+nickSize+firstSize+lastSize+mailSize+citySize+stateSize)])-1; 
            unsigned short faxSize=NSSwapShort([packet get2BytesAtPosition:(34+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize)])-1; 
            unsigned short streetSize=NSSwapShort([packet get2BytesAtPosition:(37+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize)])-1; 
            unsigned short cellSize=NSSwapShort([packet get2BytesAtPosition:(40+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize)])-1;
             unsigned short zipSize=NSSwapShort([packet get2BytesAtPosition:(43+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize+cellSize)])-1;
                                    
            void* nickData=[packet getBytesFrom:15 length:nickSize];
            void* firstData=[packet getBytesFrom:(18+nickSize) length:firstSize];
            void* lastData=[packet getBytesFrom:(21+nickSize+firstSize) length:lastSize];
            void* mailData=[packet getBytesFrom:(24+nickSize+firstSize+lastSize) length:mailSize];     
            void* cityData=[packet getBytesFrom:(27+nickSize+firstSize+lastSize+mailSize) length:citySize];     
            void* stateData=[packet getBytesFrom:(30+nickSize+firstSize+lastSize+mailSize+citySize) length:stateSize];     
            void* phoneData=[packet getBytesFrom:(33+nickSize+firstSize+lastSize+mailSize+citySize+stateSize) length:phoneSize];     
            void* faxData=[packet getBytesFrom:(36+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize) length:faxSize];     
            void* streetData=[packet getBytesFrom:(39+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize) length:streetSize];     
            void* cellData=[packet getBytesFrom:(42+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize) length:cellSize];                 	    void* zipData=[packet getBytesFrom:(45+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize+cellSize) length:zipSize];     
            //NSLog(@"FULL Info Result 0xC8000A main-home-info");
   
           [infos setCountry:NSSwapShort([packet get2BytesAtPosition:(46+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize+cellSize+zipSize)])];
           //char gmt=[packet getByteAtPosition:(48+nickSize+firstSize+lastSize+mailSize+citySize+stateSize+phoneSize+faxSize+streetSize+cellSize+zipSize)];

                            
            [infos setNickname:[NSString stringWithCString:nickData length:nickSize]];
            [infos setFirstName:[NSString stringWithCString:firstData length:firstSize]];
            [infos setLastName:[NSString stringWithCString:lastData length:lastSize]];
            [infos setMail:[NSString stringWithCString:mailData length:mailSize]];
            [infos setCity:[NSString stringWithCString:cityData length:citySize]];
            [infos setState:[NSString stringWithCString:stateData length:stateSize]];
            [infos setPhone:[NSString stringWithCString:phoneData length:phoneSize]];
            [infos setFax:[NSString stringWithCString:faxData length:faxSize]];
            [infos setStreet:[NSString stringWithCString:streetData length:streetSize]];
            [infos setCellular:[NSString stringWithCString:cellData length:cellSize]];
            [infos setZip:[NSString stringWithCString:zipData length:zipSize]];
            
            free(nickData);free(firstData);free(lastData);free(mailData);free(cityData);
            free(stateData);free(phoneData);free(faxData);free(streetData);free(cellData);free(zipData);
            done[0] = true;         
            break;          
        }
        case 0xDC000A:	// homepage-more-info
        {
            unsigned short hpSize;
            void* hpData;

            //NSLog(@"FULL Info Result 0xDC000A homepage-more-info");
            
            [infos setAge:[packet getByteAtPosition:(13)]];
            [infos setSex:[packet getByteAtPosition:(15)]];
            hpSize=NSSwapShort([packet get2BytesAtPosition:16])-1;
            hpData=[packet getBytesFrom:18 length:hpSize];
            [infos setHomepage:[NSString stringWithCString:hpData length:hpSize]];
            //short birthyear=NSSwapShort([packet get2BytesAtPosition:(19+hpSize)]);     
            [infos setBirthYear:NSSwapShort([packet get2BytesAtPosition:(19+hpSize)])];          
            //char birthmonth=[packet getByteAtPosition:(21+hpSize)];  
            [infos setBirthMonth:[packet getByteAtPosition:(21+hpSize)]];
            //char birthday=[packet getByteAtPosition:(22+hpSize)];  
            [infos setBirthDay:[packet getByteAtPosition:(22+hpSize)]];            
            [infos setLang1:[packet getByteAtPosition:(23+hpSize)]];
            [infos setLang2:[packet getByteAtPosition:(24+hpSize)]];
            [infos setLang3:[packet getByteAtPosition:(25+hpSize)]];
            
            free(hpData);
            done[1] = true;
            break;
        }
        case 0xEB000A: // more-email-info
        {
            //NSLog(@"FULL Info Result 0xEB000A more-email-info");
        
            done[2] = true;
            break;
        }
        case 0x0E010A: //  WORD    0
        {
            //NSLog(@"FULL Info Result 0x0E010A");
        
            done[3] = true;
            break;
        }        
        case 0xD2000A: // work-info
        {
            //NSLog(@"FULL Info Result 0xD2000A");
        
            done[4] = true;
            break;
        }   
       case 0xE6000A: // LNTS    about
        {
            unsigned short aboutSize;
            void* aboutData;
            
            //NSLog(@"FULL Info Result 0xE6000A");
            
            aboutSize=NSSwapShort([packet get2BytesAtPosition:13])-1;
            aboutData=[packet getBytesFrom:15 length:aboutSize];
            
            [infos setAbout:[NSString stringWithCString:aboutData length:aboutSize]];
                
            free(aboutData);
            done[5] = true;
            break;
        }   
        case 0xF0000A: // personal-interests-info
        {
            //NSLog(@"FULL Info Result 0xF0000A");
        
            done[6] = true;
            break;
        }
        case 0xFA000A: // past-background-info
        {
            //NSLog(@"FULL Info Result 0xFA000A");
        
            done[7] = true;
            break;
        }                 
    }
    
    for (i=0;i<8;i++)
    {
        if (done[i] == false)
        { 
            //NSLog(@"returning");
            return;
        }
    }
   
    times = 0;
    [SMUserInfos release];    
    return  [manager gotFullUserInfo:infos];
}

- (void) receivedServerSideList:(SMPackets*)message
{
  NSMutableArray* tUins = [NSMutableArray array];
  NSString* tString;
  int tStringLength, tPosition, tCounter, tTag, tId, tType, tLength, tCount = [message get2BytesAtPosition:11];

  tPosition = 13;
  for (tCounter = 0 ; tCounter < tCount ; tCounter++)
  {
	tStringLength = [message get2BytesAtPosition:tPosition];
	tPosition += 2;

	if (tStringLength != 0)
	{
	  tString = [NSString stringWithCString:[message getBytesFrom:tPosition length:tStringLength] length:tStringLength];
	}
	tPosition+=tStringLength;

	tTag =[message get2BytesAtPosition:tPosition];
	tPosition+=2;
	tId =[message get2BytesAtPosition:tPosition];
	tPosition+=2;
	tType =[message get2BytesAtPosition:tPosition];
	tPosition+=2;
	tLength =[message get2BytesAtPosition:tPosition];
	tPosition+=2;
	tPosition+=tLength;

	if (tType == 0 || tType == 3 || tType == 0x000E)
	  [tUins addObject:tString];
  }
  [manager receivedServerSideList:tUins];
}

- (void) receivedMessage:(SMPackets*)message
{
    int format = [message get2BytesAtPosition:18];
    int uinSize = [message getByteAtPosition:20];
    void* uinData = [message getBytesFrom:21 length:uinSize];
    NSString* uin = [[NSString alloc] initWithCString:uinData length:uinSize];
    int tlvNb = [message get2BytesAtPosition:(uinSize+23)];
    SMPackets* realData;
    int counter;
    void* messageData;
    int tlv;

    [message retain];
    [message setTLVPosition:(uinSize+25)];

    for (counter = 0; counter < tlvNb; counter++)
    {
        tlv = [message getCurrentTLV];
        switch (tlv)
        {
            case 0x01:
            {
                //unknown
                break;
            }
            case 0x04:
            {
                //unknown
                break;
            }
            case 0x06:
            {
                //sender's status
                break;
            }
            case 0x08:
            {
			  [message debugCurrentTLV];
			  //unknown
			  break;
            }
            case 0x0F:
            {
                //time
                break;
            }
            case 0x02:
            {
                //member time
                break;
            }
            case 0x03:
            {
                //online time
                break;
            }
            default:
            {
                NSLog([NSString stringWithFormat:@"unknown TLV in packet with SNAC 4,7 (%x)", tlv]);
            }
        }
        [message goToNextTLV];
    }
    messageData = [message getDataOfCurrentTLV];
    realData = [[SMPackets alloc] initWithBytes:messageData andSize:[message getCurrentTLVSize]];
          
    switch (format)
    {
        case 1: // normal message
        {
            void* messageString;
            int offset = 6;
            int messageLength;
            NSData* stringData;            
            
            offset += [realData get2BytesAtPosition:2];
            messageLength = [realData get2BytesAtPosition:offset]-4;
            messageString = [realData getBytesFrom:(offset+6) length:messageLength];
            stringData = [NSData dataWithBytes:messageString length:messageLength];
            [manager receivedMessage:stringData fromUIN:atoi([uin cString])];
            
            free(messageString);
            free(messageData);
            [realData release];
            break;
        }        
        case 2:
        {
            break;
        }
        case 4: //many purposes
        {
            void* messageString;
            int messageLength;
            int msgSubType;
            msgSubType = [realData getByteAtPosition:4];
            messageLength = NSSwapShort([realData get2BytesAtPosition:6])-1;
            messageString = [realData getBytesFrom:8 length:messageLength];
        
            switch (msgSubType)
            {
                case 0x00: 
                case 0x01: // message
                {
                    [manager receivedMessage:[NSData dataWithBytes:messageString length:messageLength] fromUIN:atoi([uin cString])];
                    break;
                }
                case 0x04: // url
                {
                    NSString  * string = [NSString stringWithCString:messageString length:messageLength];
                    const char* cContent = [string cString];                    
                    int counter, index = 0;
                    
                    for (counter=0; counter<[string length]; counter++)
                    {
                        unsigned char aChar=cContent[counter];
                        if (aChar == 0xfe)
                            index = counter;
                    }
                    
                    [manager receivedURL:[string substringFromIndex:(index+1)] description:[string substringToIndex:index] fromUIN:atoi([uin cString])];                        
                    break;
                }
                case 0x06: // authorization request
                {
                    NSString  * string = [NSString stringWithCString:messageString length:messageLength];
                    const char* cContent = [string cString];                    
                    int counter, index = 0;
                    
                    for (counter=0; counter<[string length]; counter++)
                    {
                        unsigned char aChar=cContent[counter];
                        if (aChar == 0xfe)
                            index = counter;
                    }
                                    
                    [manager authorizationRequestFrom:atoi([uin cString]) message:[string substringFromIndex:(index+1)]];                                   		    break;
                }                
                case 0x07: // authorization denied
                {
                    break;
                }                
                case 0x08: // authorization given
                {
                    break;
                }        
                case 0x0C: // user added you
                {
                    [manager userAddedYou:atoi([uin cString])];                    
                    break;
                }                          
                default:
                {
                    NSLog([NSString stringWithFormat:@"unknown data packet with SNAC 4,7, type 4, subtype : %x", msgSubType]);
                }
            }
            free(messageData);
            [realData release];
            free(messageString);
            break;            
        }
        default:
        {
            NSLog([NSString stringWithFormat:@"unknown data packet with SNAC 4,7 : type :%x", format]);
        }
    }
    free(uinData);
    [uin release];
    [message release];
}

- (void) sendOfflineMessage:(NSData*)message toUIN:(int)uin from:(int)sendersUIN
{
  SMPackets* messagePacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
  SMPackets* text=[[SMPackets alloc] initWithBytes:(void*)[message bytes] andSize:[message length]];

  [messagePacket addSnacCommandOfFamily:4 withType:6];

  [messagePacket set4Bytes:time(0L) atPosition:[messagePacket length]];
  [messagePacket set4Bytes:time(0L) atPosition:[messagePacket length]];
  [messagePacket set2Bytes:4 atPosition:[messagePacket length]];
  [messagePacket setUINString:[NSString stringWithFormat:@"%d", uin] atPosition:[messagePacket length]];

  [messagePacket addTLV:5 withSize:(9+[message length])];
  [messagePacket set4Bytes:NSSwapLong(sendersUIN) atPosition:[messagePacket length]];
  [messagePacket set2Bytes:0x0100 atPosition:[messagePacket length]];

  [messagePacket set2Bytes:NSSwapShort([message length]+1) atPosition:[messagePacket length]];
  [messagePacket appendPacket:text];
  [messagePacket setByte:0x00 atPosition:[messagePacket length]];
  [messagePacket addTLV:6 withSize:0];

  [demultiplexer sendPacket:messagePacket];
  [messagePacket release];
  [text release];
}

- (void) sendDataMessage:(NSData*)message toUIN:(int)uin from:(int)sendersUIN
{
  SMPackets* messagePacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
  SMPackets* text=[[SMPackets alloc] initWithBytes:(void*)[message bytes] andSize:[message length]];

  [messagePacket addSnacCommandOfFamily:4 withType:6];				//  SNAC 4,06
  [messagePacket set4Bytes:time(0L) atPosition:[messagePacket length]];		// 8 BYTED  msg-id (ACKs should use same ID)
  [messagePacket set2Bytes:time(0L) atPosition:[messagePacket length]];
  [messagePacket set4Bytes:1 atPosition:[messagePacket length]];		// msg-format (1 =>  simple message)
                                                                                // recipient's UIN
  [messagePacket setUINString:[NSString stringWithFormat:@"%d", uin] atPosition:[messagePacket length]];

  [messagePacket addTLV:0x02 withSize:(13+[message length])];			// TLV(2)   msg-data-1
  [messagePacket set2Bytes:0x0501 atPosition:[messagePacket length]];		// 3 BYTE    05 01 00
  [messagePacket set2Bytes:0x01 atPosition:[messagePacket length]];
  [messagePacket setByte:0x01 atPosition:[messagePacket length]];		// BYTE      ??A (seen: 01)
  [messagePacket set2Bytes:0x0101 atPosition:[messagePacket length]];		// 2 BYTE    01 01
                                                                                // WORD      msg length + 4
  [messagePacket set2Bytes:[message length]+4 atPosition:[messagePacket length]];	
  [messagePacket set2Bytes:0x0000 atPosition:[messagePacket length]];		// 4 BYTE    0
  [messagePacket set2Bytes:time(0L) atPosition:[messagePacket length]];

  [messagePacket appendPacket:text];						// STRING    msg

  [demultiplexer sendPacket:messagePacket];
  [messagePacket release];
  [text release];
}

- (void) sendAuthorizationReplyTo:(int)uin from:(int)sendersUIN isGiven:(BOOL)given
{
  SMPackets* messagePacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [messagePacket addSnacCommandOfFamily:4 withType:6];				//  SNAC 4,06
  [messagePacket set4Bytes:time(0L) atPosition:[messagePacket length]];		// 8 BYTED  msg-id (ACKs should use same ID)
  [messagePacket set2Bytes:time(0L) atPosition:[messagePacket length]];
  [messagePacket set4Bytes:4 atPosition:[messagePacket length]];		// msg-format (4 =>  auth reply)
                                                                                // recipient's UIN
  [messagePacket setUINString:[NSString stringWithFormat:@"%d", uin] atPosition:[messagePacket length]];

  [messagePacket addTLV:0x05 withSize:(9)];					// TLV(5)   msg-data-4
                                                                                // sender's uin
  [messagePacket set4Bytes:NSSwapLong(sendersUIN) atPosition:[messagePacket length]];  
  if (given == true)	
    [messagePacket setByte:0x08 atPosition:[messagePacket length]];		// msg-type: authorization given
  else
    [messagePacket setByte:0x07 atPosition:[messagePacket length]];		// msg-type: authorization denied
  [messagePacket setByte:0x00 atPosition:[messagePacket length]];		// msg-flags
                                                                                // msg: NULL LNTS
  [messagePacket set2Bytes:NSSwapShort((unsigned short)(1)) atPosition:[messagePacket length]];
  [messagePacket setByte:0 atPosition:[messagePacket length]];  


  [demultiplexer sendPacket:messagePacket];
  [messagePacket release];
}

- (void) requestOfflineMessagesForUIN:(int)uin
{
  SMPackets* requestOfflineMessages=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [requestOfflineMessages addSnacCommandOfFamily:0x15 withType:0x02];

  [requestOfflineMessages addTLV:1 withSize:0x0A];
  [requestOfflineMessages set2Bytes:NSSwapShort(0x08) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set4Bytes:NSSwapLong(uin) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set2Bytes:(0x3C00) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set2Bytes:(0x00) atPosition:[requestOfflineMessages length]];

  [demultiplexer sendPacket:requestOfflineMessages];
  [requestOfflineMessages release];
}

- (void) ackOfflineMessagesForUIN:(int)uin
{
  SMPackets* requestOfflineMessages=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

  [requestOfflineMessages addSnacCommandOfFamily:0x15 withType:0x02];

  [requestOfflineMessages addTLV:1 withSize:0x0A];
  [requestOfflineMessages set2Bytes:NSSwapShort(0x08) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set4Bytes:NSSwapLong(uin) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set2Bytes:(0x3E00) atPosition:[requestOfflineMessages length]];
  [requestOfflineMessages set2Bytes:(0x00) atPosition:[requestOfflineMessages length]];

  [demultiplexer sendPacket:requestOfflineMessages];
  [requestOfflineMessages release];
}

- (void) decodeOfflineMessage:(SMPackets*)message
{
  int uin=NSSwapLong([message get4BytesAtPosition:0x0A]);
  int messageLength=NSSwapShort([message get2BytesAtPosition:22])-1;
 // NSString* content;
  void* messageString=[message getBytesFrom:24 length:messageLength];
  //content=[[NSString alloc] initWithCString:messageString length:messageLength];
  //[SMICQInputServices receivedMessage:content fromUIN:uin];
  [manager receivedMessage:[NSData dataWithBytes:messageString length:messageLength] fromUIN:uin];
//  [content release];
  free(messageString);
}

- (void) endOfOfflineMessages
{
  [manager endOfOfflineMessages];
}

- (void) userChangedStatus:(SMPackets*)receivedPacket
{
    char vers = kUnknownVers;
    int uin;
    int newStatus=-1;
    int uinSize;
    int tlv;
    void* uinData;
    NSString* ipAddress = @"";
    NSString* capabilities = @"";
	NSString* tLocalAddress = @"";
	unsigned short tPort = 0;
	unsigned short tConnectionType = 0;
	unsigned int tCookie = 0;

    [receivedPacket retain];

    uinSize = [receivedPacket getByteAtPosition:10];
    uinData = [receivedPacket getBytesFrom:11 length:uinSize];
    if (uinData == nil)
    {
        NSLog(@"error on contact status change");
        return;
    }
    uin = atoi([[NSString stringWithCString:uinData length:uinSize] cString]);
    [receivedPacket setTLVPosition:(15+uinSize)];
    //Looking through the packet for the correct subpart
    while ((tlv = [receivedPacket goToNextTLV])!=0)
    {
	  if (tlv == 0x0c)
	  {
		tLocalAddress = [NSString stringWithFormat:@"%d.%d.%d.%d",
		  ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+4)]),
		  ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+5)]),
		  ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+6)]),
		  ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+7)])];
		tPort=((unsigned short)([receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+8)]));
		tConnectionType=((unsigned short)([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+12)]));
		tCookie=((unsigned int)([receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+15)]));
		NSLog(@"%d changed status:\nIP=%@\nPort=%d\nConnection=%d\nCookie=%X", uin, tLocalAddress, tPort, tConnectionType, tCookie);
	  }
        if (tlv == 6) //ID of the subpart of the packet containing the informations
            newStatus=((unsigned short)([receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+4)]));
        if (tlv == 10) //extract the IP Address
        {
            ipAddress = [ipAddress stringByAppendingFormat:@"%d.%d.%d.%d",
                ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+4)]),
                ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+5)]),
                ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+6)]),
                ([receivedPacket getByteAtPosition:([receivedPacket getTLVPosition]+7)])];
           // NSLog([NSString stringWithFormat:@"uin %d ip %@", uin, ipAddress]);
        }
        if (tlv==0X0D) //extract the Capabilities
        {
            // void* messageString;
            int messageLength=[receivedPacket getCurrentTLVSize];
            int indiceBoucle = messageLength/16;
            int indice = indiceBoucle;
    
            for (;indiceBoucle>0;indiceBoucle--)
            {
                capabilities = [capabilities stringByAppendingFormat:@"%8X%8X%8X%8X", 
                    [receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+0+((indice-indiceBoucle)*16))], 
                    [receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+4+((indice-indiceBoucle)*16))], 
                    [receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+8+((indice-indiceBoucle)*16))], 
                    [receivedPacket get4BytesAtPosition:([receivedPacket getTLVPosition]+12+((indice-indiceBoucle)*16))]]; 
            }
            if ([capabilities isEqualToString:@"   D0050563FC809 B6F41BD9F794226 9DFA2F3 94613494C7F11D18222444553540000 946134E4C7F11D1822244455354000097B12751243C4334AD22D6ABF73F1492 94613444C7F11D182224445"])
			  vers = kTrillianVers;
		  else if ([capabilities isEqualToString:@"   D0050 94613494C7F11D18222444553540000 94613444C7F11D1822244455354000097B12751243C4334AD22D6ABF73F14092E7A6475FADF4DC8886FEA3595FDB6DFF2E7C7F4FEAD4DFBB2353679"])
			  vers = kTrillianVers;
		  else if ([capabilities isEqualToString:@"   D0040 94613494C7F11D1822244455354000097B12751243C4334AD22D6ABF73F14922E7A6475FADF4DC8886FEA3595FDB6DF 94613444C7F11D182224445"])
                vers = kWin2001bVers;
            else if ([capabilities isEqualToString:@"   D0040 94613494C7F11D18222444553540000 946134E4C7F11D1822244455354000097B12751243C4334AD22D6ABF73F1492 94613444C7F11D182224445"])
			  vers = kWin2002aVers;
		  else if ([capabilities isEqualToString:@"   D0040 94613494C7F11D18222444553540000 946134E4C7F11D1822244455354696997B12751243C4334AD22D6ABF73F1492 94613444C7F11D182224445"])
			vers = kiXQVers;
            else if ([capabilities isEqualToString:@"   D0020 94613494C7F11D18222444553540000 94613444C7F11D182224445"])
                vers = kLibIcq2000Vers;
            else if ([capabilities isEqualToString:@"   D0020 94613444C7F11D18222444553540000 94613494C7F11D182224445"])
			  vers = kLibIcq2000Vers2;
		  else if ([capabilities isEqualToString:@"   D0020 94613494C7F11D18222444553540000 94913494C7F11D182224445"])
			vers = kLibIcq2000Vers2;
            else if ([capabilities isEqualToString:@"   D0010 94613444C7F11D182224445"])
                vers = kGerryIcq45Vers;
            else if ([capabilities isEqualToString:@"   D0010DD16F20284E611D490DB0010"])
                vers = kMacIcq3Vers;     
            else if ([capabilities isEqualToString:@"   D0030 94613494C7F11D18222444553540000 946134E4C7F11D18222444553540000DD16F20284E611D490DB0010"])
                vers = kMacIcq31Vers;                                 
            else if ([capabilities isEqualToString:@"   D0010 94613494C7F11D182224445"])
                vers = kIcqLiteVers;
            else if ([capabilities isEqualToString:@""])
                vers = kiXQVers; 
            else 
            {    
                vers = kUnknownVers;
//                NSLog([NSString stringWithFormat:@"uin %d cap %@ vers %d", uin, capabilities, vers]);
            }
        }
    }
    //detecting the status / translating it to our notation
    //NSLog([NSString stringWithFormat:@"status change : %d", newStatus]);
    switch (newStatus)
    {
        case -1:
        {
            //NSLog(@"Error within the status change message");
            //[receivedPacket debugPacket];
            newStatus=1;
            //return;
            break;
        }
        case 0 : //Available
        {
            newStatus=1;
            break;
        }
        case 1 : //Away
        {
            newStatus=2;
            break;
        }
        case 2 : //DND
        {
            newStatus=5;
            break;
        }
        case 4 : //N/A
        {
            newStatus=3;
            break;
        }
        case 5 : //N/A
        {
            newStatus=3;
            break;
        }
        case 0x10 : //Occupied
        {
            newStatus=4;
            break;
        }
        case 0x11 : //Occupied
        {
            newStatus=4;
            break;
        }
        case 0x13 : //DND
        {
            newStatus=5;
            break;
        }
        case 0x100 : //Free for chat
        {
            newStatus=7;
            break;
        }
        case 0x20 : //Invisible
        {
            newStatus=6;
            break;
        }
        case 257 : //Invisible
        {
		  newStatus=2;
		  break;
        }
        case 261 : //Invisible
        {
		  newStatus=3;
		  break;
        }
        case 273 : //Invisible
        {
		  newStatus=4;
		  break;
        }
        case 275 : //Invisible
        {
		  newStatus=5;
		  break;
        }
        case 288 : //Invisible
        {
		  newStatus=7;
		  break;
        }
        default : //available
        {
            NSLog([NSString stringWithFormat:@"unkown status %d (uin = %d)", newStatus, uin]);
        
            newStatus=1;
        }
    }

	//NSLog(@"%d changed status:\nIP=%@\nPort=%d\nConnection=%d\nCookie=%X", uin, tLocalAddress, tPort, tConnectionType, tCookie);
    [manager userWithUIN:uin changedStatusTo:newStatus withIP:ipAddress withCap:capabilities withClient:vers localIP:tLocalAddress port:tPort cookie:tCookie];
    [receivedPacket release];
    free(uinData);
}

- (void) userHasGoneOffline:(SMPackets*)packet
{
  int uin;
  //int newStatus=0;
  int uinSize;
  void* uinData;

  [packet retain];
  uinSize=[packet getByteAtPosition:10];
  uinData=[packet getBytesFrom:11 length:uinSize];
  uin=atoi([[NSString stringWithCString:uinData length:uinSize] cString]);
  [manager userWithUIN:uin changedStatusTo:0 withIP:@"" withCap:@"" withClient:0 localIP:@"" port:0 cookie:0];

  [packet release];
  free(uinData);
}

- (void) queryOurInfo:(int)ouruin
{
    SMPackets* queryOurInfoPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];

    [queryOurInfoPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
    [queryOurInfoPacket addTLV:1 withSize:0x10];						// TLV(1)
    [queryOurInfoPacket set2Bytes:NSSwapShort(0x0E) atPosition:[queryOurInfoPacket length]];	// remaining bytes
    [queryOurInfoPacket set4Bytes:NSSwapLong(ouruin) atPosition:[queryOurInfoPacket length]];	// our UIN
    [queryOurInfoPacket set2Bytes:(0xD007) atPosition:[queryOurInfoPacket length]];		// message type (multifunction)
    [queryOurInfoPacket set2Bytes:0 atPosition:[queryOurInfoPacket length]];			// request id
    [queryOurInfoPacket set2Bytes:(0xD004) atPosition:[queryOurInfoPacket length]];		// message subtype (query my info)
    [queryOurInfoPacket set4Bytes:NSSwapLong(ouruin) atPosition:[queryOurInfoPacket length]];	// our uin

    [demultiplexer sendPacket:queryOurInfoPacket];
    [queryOurInfoPacket release];
}

- (void) modifyOurInfo:(int)ouruin newInfo:(SMUserInfos*)ui
{
    SMPackets* mainHomeInfoPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
    SMPackets* homepageMoreInfoPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
    SMPackets* aboutPacket=[[SMPackets alloc] initForChannel:2 withParams:[manager seqNum]];
    
    int length = [[ui nickname] length]+[[ui firstName] length]+[[ui lastName] length]+[[ui mail] length]+[[ui city] length]+
                [[ui state] length]+[[ui phone] length]+[[ui fax] length]+[[ui street] length]+[[ui cellular] length]+[[ui zip] length]+47;
    
    [mainHomeInfoPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
    [mainHomeInfoPacket addTLV:1 withSize:length+2];						// TLV(1)
    [mainHomeInfoPacket set2Bytes:NSSwapShort(length) atPosition:[mainHomeInfoPacket length]];	// remaining bytes
    [mainHomeInfoPacket set4Bytes:NSSwapLong(ouruin) atPosition:[mainHomeInfoPacket length]];	// our UIN
    [mainHomeInfoPacket set2Bytes:(0xD007) atPosition:[mainHomeInfoPacket length]];		// message type (multifunction)
    [mainHomeInfoPacket set2Bytes:0 atPosition:[mainHomeInfoPacket length]];			// request id
    [mainHomeInfoPacket set2Bytes:(0xEA03) atPosition:[mainHomeInfoPacket length]];		// message subtype (modify main-home-info)
  
    [mainHomeInfoPacket setLNTString:[ui nickname] atPosition:[mainHomeInfoPacket length]]; 	// nick name
    [mainHomeInfoPacket setLNTString:[ui firstName] atPosition:[mainHomeInfoPacket length]]; 	// first name
    [mainHomeInfoPacket setLNTString:[ui lastName] atPosition:[mainHomeInfoPacket length]]; 	// last name
    [mainHomeInfoPacket setLNTString:[ui mail] atPosition:[mainHomeInfoPacket length]]; 	// email
    [mainHomeInfoPacket setLNTString:[ui city] atPosition:[mainHomeInfoPacket length]]; 	// city
    [mainHomeInfoPacket setLNTString:[ui state] atPosition:[mainHomeInfoPacket length]]; 	// state
    [mainHomeInfoPacket setLNTString:[ui phone] atPosition:[mainHomeInfoPacket length]]; 	// phone
    [mainHomeInfoPacket setLNTString:[ui fax] atPosition:[mainHomeInfoPacket length]]; 		// fax
    [mainHomeInfoPacket setLNTString:[ui street] atPosition:[mainHomeInfoPacket length]]; 	// street
    [mainHomeInfoPacket setLNTString:[ui cellular] atPosition:[mainHomeInfoPacket length]]; 	// cellular
    [mainHomeInfoPacket setLNTString:[ui zip] atPosition:[mainHomeInfoPacket length]]; 		// zip

    [mainHomeInfoPacket set2Bytes:[ui country] atPosition:[mainHomeInfoPacket length]];		// country

    [mainHomeInfoPacket setByte:0 atPosition:[mainHomeInfoPacket length]]; 			// gmt  
    [mainHomeInfoPacket setByte:0 atPosition:[mainHomeInfoPacket length]]; 			// publish email (00 yes, 01 no)  

    [demultiplexer sendPacket:mainHomeInfoPacket];
    [mainHomeInfoPacket release];

    length = [[ui homepage] length]+23;

    [homepageMoreInfoPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
    [homepageMoreInfoPacket addTLV:1 withSize:length+2];						// TLV(1)
    [homepageMoreInfoPacket set2Bytes:NSSwapShort(length) atPosition:[homepageMoreInfoPacket length]];	// remaining bytes
    [homepageMoreInfoPacket set4Bytes:NSSwapLong(ouruin) atPosition:[homepageMoreInfoPacket length]];	// our UIN
    [homepageMoreInfoPacket set2Bytes:(0xD007) atPosition:[homepageMoreInfoPacket length]];		// message type (multifunction)
    [homepageMoreInfoPacket set2Bytes:0 atPosition:[homepageMoreInfoPacket length]];			// request id
    [homepageMoreInfoPacket set2Bytes:(0xFD03) atPosition:[homepageMoreInfoPacket length]];		// message subtype (modify homepage-more-info)

    [homepageMoreInfoPacket setByte:[ui age] atPosition:[homepageMoreInfoPacket length]]; 		// age
    [homepageMoreInfoPacket setByte:0 atPosition:[homepageMoreInfoPacket length]]; 			// 0
    [homepageMoreInfoPacket setByte:[ui sex] atPosition:[homepageMoreInfoPacket length]]; 		// gender
    [homepageMoreInfoPacket setLNTString:[ui homepage] atPosition:[homepageMoreInfoPacket length]]; 	// homepage
    [homepageMoreInfoPacket set2Bytes:NSSwapLong([ui birthyear]) atPosition:[homepageMoreInfoPacket length]]; 	// birth-year (LE)
    [homepageMoreInfoPacket setByte:[ui birthmonth] atPosition:[homepageMoreInfoPacket length]]; 	// birth-month
    [homepageMoreInfoPacket setByte:[ui birthday] atPosition:[homepageMoreInfoPacket length]]; 		// birth-day
    [homepageMoreInfoPacket setByte:[ui lang1] atPosition:[homepageMoreInfoPacket length]]; 		// lang1
    [homepageMoreInfoPacket setByte:[ui lang2] atPosition:[homepageMoreInfoPacket length]]; 		// lang2
    [homepageMoreInfoPacket setByte:[ui lang3] atPosition:[homepageMoreInfoPacket length]]; 		// lang3

    [demultiplexer sendPacket:homepageMoreInfoPacket];
    [homepageMoreInfoPacket release];

    length = [[ui about] length]+13;

    [aboutPacket addSnacCommandOfFamily:0x15 withType:0x02 andFlags:5];			// SNAC(15,2)
    [aboutPacket addTLV:1 withSize:length+2];						// TLV(1)
    [aboutPacket set2Bytes:NSSwapShort(length) atPosition:[aboutPacket length]];	// remaining bytes
    [aboutPacket set4Bytes:NSSwapLong(ouruin) atPosition:[aboutPacket length]];		// our UIN
    [aboutPacket set2Bytes:(0xD007) atPosition:[aboutPacket length]];			// message type (multifunction)
    [aboutPacket set2Bytes:0 atPosition:[aboutPacket length]];				// request id
    [aboutPacket set2Bytes:(0x0604) atPosition:[aboutPacket length]];			// message subtype (modify about)

    [aboutPacket setLNTString:[ui about] atPosition:[aboutPacket length]]; 		// about

    [demultiplexer sendPacket:aboutPacket];
    [aboutPacket release];
}

- (void) error
{
  [manager error];
}

@end