/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMSNACMgr.m
//  iXQnew
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import "SMSNACMgr.h"
#import "SMDemultiplexer.h"
#import "SMPackets.h"
#import "SMDataPackets.h"

@implementation SMSNACMgr

- (id) initWithDemultiplexer:(id)theDemultiplexer andHandler:(id)handler
{
  // NSLog(@"SMSNACMgr init");

  demultiplexer = (SMDemultiplexer*)theDemultiplexer;
  dataPkHandler = (SMDataPackets*)handler;

  return self;
}

- (void) dealloc
{
}

- (void) handleIncomingPacket:(id)packet;
{
  int family;
  int type;
  BOOL handled = true;  
  [packet retain];

  family = [packet get2BytesAtPosition:0];
  type = [packet get2BytesAtPosition:2];

  // NSLog([NSString stringWithFormat:@"Data Packet(%x/%x)", family, type]);

  //Errors
//  if (type == 1)
//	return [dataPkHandler error];

  //should I cut this function ?
  switch (family)
  {
    //Family = Generic Service Controls
    case 1 :
    {
      switch (type)
      {
        case 0x3 :
        {
          //Received Server is Ready (Connection Ack)
          //Send Client type = ICQ (BOS SignOn)
          [dataPkHandler sendClientTypeNotification];
          break;
        }
        case 0x7 :
        {
          //Received Rate Informations
          //Send Rate Ack
          [dataPkHandler sendRateInfosAck];
          //Set Privacy Flags
          //Request Our User Information
          [dataPkHandler requestPersoInfos];
          //Request New Service
          //Optional: Request BOS Rights
          //Optional: Request Buddy List Rights
          //Optional: Request Locate Rights
          //Optional: Request ICBM Parameter Information
          [dataPkHandler requestRights];
          break;
        }
        case 0xF :
        {
          //Received Personnal Informations
          break;
        }
        case 0x18 :
        {
          //Received Ack for Client Type (BOS HostReady)
          //Send Rate Informations Request
          [dataPkHandler sendRateInfosRequest];
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    case 2 :
    {
      switch (type)
      {
        case 0x3 :
        {
          //Received rights for location service
          //Send set user infos
          [dataPkHandler setUserInfos];
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    case 3 :
    {
      switch (type)
      {
        case 0x3 :
        {
          //Received rights for buddy list
          //Send Contact List
          [dataPkHandler serverRequestsYourContactList];
            //this should now be done in changeStatus
          //Send Status
          [dataPkHandler changeStatus];
          //Client is ready
          [dataPkHandler clientReady];
          //Connection complete
          break;
        }
        case 0xB :
        {
          //Received User Going Online
          [dataPkHandler userChangedStatus:packet];
          break;
        }
        case 0xC :
        {
          //A user has gone offline
          [dataPkHandler userHasGoneOffline:packet];
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    case 4 :
    {
      switch (type)
      {
        case 0x5 :
        {
          //Received rights for ICBM
          //Add ICBM Parameter
          //[SMICQConnectionUtilities configureICBM];
          break;
        }
        case 0x7 :
        {
          //Message recu
          [dataPkHandler receivedMessage:packet];
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    case 9 :
    {
      switch (type)
      {
        case 0x3 :
        {
          //Received rights for BOS
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    case 0x13 :
    {
      switch (type)
      {
        case 0x6 :
        {
          [dataPkHandler receivedServerSideList:packet];
          break;
        }
        default :
        {
		  handled = false;
        }
      }
      break;
    }
    case 0x15 :
    {
      switch (type)
      {
        case 0x3 :
        {
          //Received many purpose informations
          SMPackets* realData;
          int messageType;
          void* dataTLV;
          [packet setTLVPosition:10];
          dataTLV=[packet getDataOfCurrentTLV];
          realData=[[SMPackets alloc] initWithBytes:dataTLV andSize:[packet getCurrentTLVSize]];
          messageType=[realData get2BytesAtPosition:6];
          if (messageType==0xDA07)
          {
            int messageSubType;
            messageSubType=[realData get2BytesAtPosition:10];

            /*if (messageSubType == 0x9600) // SMS Reply
            {
              [SMICQMessagingFunctions decodeSMSReply:realData];
            }
            else if (messageSubType == 0x6400) // SMS Error
            {
              [SMICQMessagingFunctions decodeSMSError:realData];
            }
            else*/
            if ( messageSubType == 0x9001 || messageSubType == 0x9A01   //short search
                 || messageSubType == 0xA401 || messageSubType == 0xAE01) //full search
            {
                [dataPkHandler searchResults:realData];
            }
            
            if ( messageSubType == 0xC800 || messageSubType == 0xDC00   
                 || messageSubType == 0xEB00 || messageSubType == 0x0E01 
                 || messageSubType == 0xD200 || messageSubType == 0xE600 
                 || messageSubType == 0xF000 || messageSubType == 0xFA00 ) 
            {
                [dataPkHandler fullInfoResults:realData];
            }
            
            if ( messageSubType == 0x6400 || messageSubType == 0x7800   //ack to modify info (main/home) || (homepage/more)
                 || messageSubType == 0x8200 || messageSubType == 0x6E00) //ack to modify info (about) || (work)
            {
                [dataPkHandler modificationResults:realData];
            }            
          }
          else if (messageType==0x4100)
          {
            //Offline message
            [dataPkHandler decodeOfflineMessage:realData];
          }
          else if (messageType==0x4200)
          {
            //End of Offline message
            [dataPkHandler endOfOfflineMessages];
          }
          free(dataTLV);
          [realData release];
          break;
        }
        default :
        {
            handled = false;
        }
      }
      break;
    }
    default:
    {
            handled = false;
    }
  }
    if (handled == false)
    {
        NSLog([NSString stringWithFormat:@"Unknown Data Packet: (%d/%d)", family, type]);
        [packet debugPacket];    
    }
}


@end