/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMServiceMgr.h
//  iXQ
//
//  Created by Franois-Marie Grattepain on Wed Jan 23 2002.
//  Copyright (c) 2002 iXQ-Team. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SMUserInfos.h"
#import "connectionProtocol.h"
#import "SMMessage.h"

@interface SMServiceMgr : NSObject 
{
    // linked classes: the specialized packets classes
    id connectionPkHandler;
    id dataPkHandler;
    id errorPkHandler;

    //No direct link, the ServiceMgr just intializes the demultiplexer
    //and give the reference to its specialized linked classes
    //Added : it can connect directly to ask for socket managment
    id demultiplexer;

    // linked gui connection
    id guiConnectionRoot;

    // local saved information
    unsigned int uin;
    BOOL connected;
    unsigned int seqNum;
    unsigned int requestedStatus;
    NSTimer* kaTimer;
    BOOL connectionAlive;
}

// (de)initialization
+ (void) managerWithData:(NSArray*)ports;
- (void) dealloc;
- (id) initWithParent:(id)parent andUIN:(NSString*)userUIN;
// accessors
- (unsigned int) seqNum;
- (unsigned int) newStatus;
// gui-data
- (NSSet*) contacts;
- (NSSet*) visible;
- (NSSet*) invisible;
// gui-callbacks (outgoing)
- (void) connectionError:(NSString*)error;
- (void) userFound:(SMUserInfos*)ui;
- (void) gotFullUserInfo:(SMUserInfos*)ui;
- (void) userWithUIN:(int)UIN changedStatusTo:(int)newStatus withIP:(NSString*)ipAddress withCap:(NSString*)capabilities withClient:(char)vers;
- (void) receivedMessage:(NSData*)content fromUIN:(int)UIN;
- (void) statusChanged:(BOOL)success;
- (void) authorizationRequestFrom:(int)UIN message:(NSString*)msg;
- (void) receivedURL:(NSString*)URL description:(NSString*)desc fromUIN:(int)UIN;
- (void) userAddedYou:(int)UIN;
- (void) infoModificationSucceeded;

// kernel-functions (incoming)
- (void) initiateConnection;
- (void) connect;
- (void) queryOurInfo;
- (void) modifyOurInfo:(SMUserInfos*)ui;
- (oneway void) setStatus:(unsigned int)newStatus; 
- (void) searchContact:(SMUserInfos*)ui;
- (void) requestFullInfoForUIN:(NSString*)suin;
- (void) requestOfflineMessages;
- (void) addUserToContactList:(SMUserInfos*)ui;
- (void) sendOfflineMessage:(NSData*)message toUIN:(NSString*)UIN;
- (void) sendMessage:(NSData*)message toUIN:(NSString*)UIN;
- (void) authorizationReplyTo:(int)UIN isGiven:(BOOL)given;

- (void) endOfOfflineMessages;

- (void) addToInvisibleList:(NSSet*)list;
- (void) addToVisibleList:(NSSet*)list;
- (void) removeFromVisibleList:(NSSet*)list;
- (void) removeFromInvisibleList:(NSSet*)list;

- (void) testConnection:(NSTimer*)unused;
- (void) connectionAlive;
- (void) error;

@end
