/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMSocketHandle.m
//  
//
//  Created by Franois-Marie Grattepain on Tue Dec 03 2002.
//  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
//

#import "SMSocketHandle.h"

@implementation SMSocketHandle

+ (id) socketHandleToConnectToTCPPort:(int)inPort onHost:(NSString*)inHost
{
    return [[[SMSocketHandle alloc] initToConnectToTCPPort:inPort onHost:inHost] autorelease];
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [host_ release];
}

- (id) initToConnectToTCPPort:(int)inPort onHost:(NSString*)inHost
{
  [host_ release];
  host_ = [inHost copy];
  port_ = inPort;

  m_status_ = 0;
  m_socketDescriptor_ = -1;
  testing_ = NO;

  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(checkATaskStatus:) name:NSTaskDidTerminateNotification object:nil];

  return self;
}

- (BOOL) connect
{
	int	result;
  
	// Look up the host.
	if ((hostAddr_ = gethostbyname([host_ cString]))==NULL)
	{
	  sockAddr_.sin_addr.s_addr=inet_addr((const char*)[host_ cString]);
	}
	else
	{
	  bzero(&sockAddr_, sizeof(sockAddr_));
	  bcopy(hostAddr_->h_addr, (char *)&sockAddr_.sin_addr, hostAddr_->h_length);
	}

	sockAddr_.sin_family = AF_INET;
	sockAddr_.sin_port = htons((u_short)port_);

	// Create a local socket.
	result = socket(AF_INET, SOCK_STREAM, 0);
	if (result == -1)
	{
	  return NO;
	}
	
	m_socketDescriptor_ = result;

    if (connect(m_socketDescriptor_, (struct sockaddr *)&sockAddr_, sizeof(sockAddr_)) < 0)
    {
	  close(m_socketDescriptor_);
	  m_socketDescriptor_ = -1;
	  return NO;
    }
	m_status_ = 1;
	[self waitForDataInBackgroundAndNotify];
	
    return YES;
}

- (BOOL) sendData:(NSData*)inData
{
  int tLength = 0;
  
  tLength = write(m_socketDescriptor_, [inData bytes],[inData length]);

  if (tLength < 0)
  {
	return NO;
  }

  return YES;
}

- (oneway void) connectionClosed:(NSNotification*)notif
{
  if (m_status_ != 0)
  {
	[delegate_ error];
  }
}

- (oneway void) connectionAlive:(NSNotification*)notif
{
  if (m_status_ == 0)
	return;

  [self testConnection];
  [self waitForDataInBackgroundAndNotify];
  [delegate_ connectionAlive];
}

- (oneway void) dataAvailable:(NSData*)inData
{
	if (m_status_ == 0)
	  return;

  [self waitForDataInBackgroundAndNotify];
  [delegate_ handleData:inData];
}

- (void) setDelegate:(id)inDelegate
{
  delegate_ = inDelegate;
}

- (void) close
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [connection_ release];
  m_status_ = 0;
  if (m_socketDescriptor_ != -1)
  {
	shutdown(m_socketDescriptor_, SHUT_RDWR);
	close(m_socketDescriptor_);
  }
  m_socketDescriptor_ = -1;
}


- (void) waitForDataInBackgroundAndNotify
{
  NSPort *port1;
  NSPort *port2;
  NSArray *portArray;
  NSConnection* tConnection;

  if (m_status_ == -1)
	return;
  
  port1 = [NSPort port];
  port2 = [NSPort port];

  [connection_ release];
  connection_ = [[NSConnection alloc] initWithReceivePort:port1 sendPort:port2];
  [connection_ setRootObject:self];

  /* Ports switched here. */
  portArray = [NSArray arrayWithObjects:port2, port1, nil];

  [NSThread detachNewThreadSelector:@selector(waitForDataAndNotify:)
						   toTarget:self withObject:portArray];
  
}

- (void) waitForDataAndNotify:(id)connectionPorts
{
  NSAutoreleasePool* releasePool = [[NSAutoreleasePool alloc] init];
  fd_set fds, mask;
  struct timeval tv;
  int toto=0;
  unsigned char message[9000];
  NSConnection* tConnection = [NSConnection connectionWithReceivePort:[connectionPorts objectAtIndex:0] sendPort:[connectionPorts objectAtIndex:1]];
  SMSocketHandle* tSocketHandle = [tConnection rootProxy];
  
  // sock is an intialized socket handle
  tv.tv_sec = 10;
  tv.tv_usec = 0;

  if (m_socketDescriptor_ == -1)
	return;
  
  FD_ZERO(&fds);
  FD_SET(m_socketDescriptor_, &fds); // adds sock to the file descriptor set

  mask=fds;
  toto=select(m_socketDescriptor_+1, &mask, NULL, NULL, &tv);

  if (m_socketDescriptor_ == -1)
	return;
  
  if (toto<0)
  {
	[tSocketHandle connectionClosed:nil];
  }
  else if (FD_ISSET(m_socketDescriptor_, &mask) && toto==0)
  {
	[tSocketHandle connectionAlive:nil];
  }
  else if (FD_ISSET(m_socketDescriptor_, &mask))
  {
	int count=read(m_socketDescriptor_, message, sizeof(message));
	if (count>0)
	{
	  [tSocketHandle dataAvailable:[NSData dataWithBytes:message length:count]];
	}
	if (count<=0&&m_status_!=0)
	{
	  [tSocketHandle connectionClosed:nil];
	}
  }
  else
  {
	[tSocketHandle connectionAlive:nil];
  }
  
  [releasePool release];
}

- (void) testConnection
{
  if (testing_)
	return;

  testing_ = YES;
  currentTask_ = [[NSTask alloc] init];
  NSMutableArray *args = [NSMutableArray array];
  NSPipe* tPipe = [NSPipe pipe];

  pipe_ = [[NSPipe pipe] retain];
  readHandle_ = [[pipe_ fileHandleForReading] retain];
  
  //ping -c 1 -s 4 login.icq.com
  // write handle is closed to this process
  [currentTask_ setStandardOutput:pipe_];
  [args addObject:@"-c"];
  [args addObject:@"5"];
  [args addObject:@"-s"];
  [args addObject:@"4"];
  [args addObject:host_];
  [currentTask_ setCurrentDirectoryPath:@"/tmp"];
  [currentTask_ setLaunchPath:@"/sbin/ping"];
  [currentTask_ setArguments:args];
  [currentTask_ launch];
}

- (void)checkATaskStatus:(NSNotification *)aNotification {
  BOOL status = NO;
  NSData *inData = nil;
  NSString* tString;

  if (!testing_)
	return;
  
  if ([aNotification object] == nil || [aNotification object] != currentTask_)
  {
	testing_ = NO;
	return;
  }
  else if ([[aNotification object] terminationStatus] == 0)
  {
	  if (readHandle_ != nil)
	  {
		while ((inData = [readHandle_ availableData]) && [inData length]) {
		  tString = [[[NSString alloc] initWithData:inData encoding:NSUTF8StringEncoding] autorelease];
		  if (tString == nil || [tString rangeOfString:@"icmp_seq"].length != 0)
			status = YES;
		}
	  }
  }


  testing_ = NO;
  if (status)
  {
	[pipe_ release];
	[readHandle_ release];
	[currentTask_ release];
  }
  else
  {
	if (m_status_!=0)
	{
	  [self connectionClosed:nil];
	}
  }
}

@end
