/*
 *
 *iXQ - cocoa-based icq client for MacOS X
 *Copyright (C) 2003  Franois-Marie Grattepain, Philippe Champeaux
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//
//  SMAccountPreferences.m
//  iXQ
//
//  Created by Philippe Champeaux on Fri Jan 25 2002.
//  Copyright (c) 2002 Soamee Development. All rights reserved.
//

#import "SMAccountPreferences.h"
#import "SMAccountDocument.h"
#import "SMConversions.h"
#import "SMThemeManager.h"


static NSString* 	AccountSettingsToolbarIdentifier	= @"AccountSettingsToolbarIdentifier";
static NSString* 	ConnectionToolbarItem			= @"Connection";
static NSString* 	AppearenceToolbarItem			= @"Appearance";
static NSString* 	MiscToolbarItem				= @"Miscallaneous";


@implementation SMAccountPreferences


// init and such
-(id)initWithParent:(id)sender {
    self = [super init];
    if (self) {
        int i;
        (SMAccountDocument*)parent = sender;
        
        [NSBundle loadNibNamed:@"SMAccountPreferences" owner:self];
        [NSColor setIgnoresAlpha:NO];
        
        
        
        views = [[NSDictionary alloc] initWithObjectsAndKeys:
                        connectionView, ConnectionToolbarItem,
                        appearanceView, AppearenceToolbarItem,
                        miscView, MiscToolbarItem,
                        loadingView, @"Loading",
                        nil];
        
        
        [self setupToolbar];
        
        [self changeView:nil];
        
        // login infos
        UIN = @"";
        password=@"";
        rememberPassword=YES;
        loginStatus=0;
        sessionLoginStatus=0;
        currentStatus=0;
        
        // Connection
        icqServerHost = @"login.icq.com";
        icqServerPort = @"5190";
        
        
        // GUI
        contactListBGColor = [[NSColor colorWithCalibratedRed:0.0 green:38.0/256.0 blue:85.0/256.0 alpha:0.80] retain];
        contactListContactTextColor = [NSColor whiteColor];
        contactListGroupTextColor = [[NSColor colorWithCalibratedRed:108.0/256.0 green:157.0/256.0 blue:201.0/256.0 alpha:1.0] retain];

        
        
        contactListStayOnTop = NO;
        contactListAlphaLevel = 0.99;
        globalAlphaLevelWhenNotInFront = 1.0;
        contactListPosition = NSMakeRect(50,50,150,150);
        searchWindowPosition=NSMakeRect(0,0,0,0);
        preferenceWindowPosition=NSMakeRect(0,0,0,0);
        listsWindowPosition=NSMakeRect(0,0,0,0);
        historyManagerPosition=NSMakeRect(0,0,0,0);
        userDetailsPanelPosition=NSMakePoint(0,0);
        
        messageWindowContactTextColor = [[NSColor colorWithCalibratedRed:229.0/256.0 green:135.0/256.0 blue:0.0 alpha:1.0] retain];
        messageWindowSelfTextColor = [[NSColor colorWithCalibratedRed:0.0 green:230.0/256.0 blue:222.0/256.0 alpha:1.0] retain];
        messageWindowHistoryBGColor = [[NSColor colorWithCalibratedRed:0.0 green:38.0/256.0 blue:85.0/256.0 alpha:0.80] retain];

        messageWindowMessageBGColor = [[NSColor colorWithCalibratedRed:0.0 green:38.0/256.0 blue:85.0/256.0 alpha:0.80] retain];

        messageWindowStayOnTop = NO;
        blendOfflineContacts = YES;
        messageWindowAlphaLevel = 0.99;
        
        autoAwayCorner=1;
        [self displayValues];
        
        iconTheme=[@"default" retain];
        currentTheme=[[[SMThemeManager sharedManager] themeWithName:iconTheme] retain];
        
        [iconThemeOutlet removeAllItems];
        for (i=0;i<[[[SMThemeManager sharedManager] themes] count];i++) {
            [iconThemeOutlet addItemWithTitle:[[[SMThemeManager sharedManager] themes] objectAtIndex:i]];
        }  
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(statusChanged:) name:@"statusChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionCompleted:) name:@"connectionCompleted" object:nil];
        
    }
    return self;
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [messageWindowContactTextColor release];
    [messageWindowSelfTextColor release];
    [messageWindowHistoryBGColor release];
    [messageWindowMessageBGColor release];
    [contactListBGColor release];
    [contactListContactTextColor release];
    [contactListGroupTextColor release];
    [iconTheme release];
    [super dealloc];
}


// ============================================================
// NSToolbar Related Methods
// ============================================================

- (void) setupToolbar {
    toolbar = [[[NSToolbar alloc] initWithIdentifier: AccountSettingsToolbarIdentifier] autorelease];
    
    [toolbar setAllowsUserCustomization: NO];
    [toolbar setDisplayMode: NSToolbarDisplayModeIconAndLabel];
    
    [toolbar setDelegate: self];
    
    
    [[self window] setToolbar: toolbar];
    [toolbar setVisible:YES];
}

- (NSToolbarItem *) toolbar: (NSToolbar *)toolbar itemForItemIdentifier: (NSString *) itemIdent willBeInsertedIntoToolbar:(BOOL) willBeInserted {
   NSToolbarItem *toolbarItem = [[[NSToolbarItem alloc] initWithItemIdentifier: itemIdent] autorelease];
    
    

    if ([itemIdent isEqual: ConnectionToolbarItem]) {
	[toolbarItem setLabel: @"Connection"];
	[toolbarItem setPaletteLabel: @"Connection"];
	[toolbarItem setToolTip: @"Connection preferences"];
	
        [toolbarItem setImage:[NSImage imageNamed:@"Connection"]];
        
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(changeView:)];

    } else if ([itemIdent isEqual: AppearenceToolbarItem]) {
	[toolbarItem setLabel: @"Appearence"];
	[toolbarItem setPaletteLabel: @"Appearence"];
	[toolbarItem setToolTip: @"Appearence preferences"];
	
        [toolbarItem setImage:[NSImage imageNamed:@"Appearance"]];
        
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(changeView:)];

    } else if ([itemIdent isEqual: MiscToolbarItem]) {
	[toolbarItem setLabel: @"Miscallaneous"];
	[toolbarItem setPaletteLabel: @"Miscallaneous"];
	[toolbarItem setToolTip: @"Miscallaneous management"];
	
        [toolbarItem setImage:[NSImage imageNamed:@"misc"]];
        
	[toolbarItem setTarget: self];
	[toolbarItem setAction: @selector(changeView:)];

    } else {
	toolbarItem = nil;
    }
    return toolbarItem;
}

- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar {
    return [NSArray arrayWithObjects:	
            ConnectionToolbarItem,
            AppearenceToolbarItem,
            MiscToolbarItem,
            nil];
}

- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar {
    return [NSArray arrayWithObjects:	
            ConnectionToolbarItem,
            AppearenceToolbarItem,
            MiscToolbarItem,
            nil];
}

// display

-(void)showWindow {
    if (!opened) {
        opened=YES;
        [[self window] makeKeyAndOrderFront:self];
        
        [self changeView:nil];
    } else {
        [[self window] makeKeyAndOrderFront:self];
    }
}

-(void)changeView:(id)sender {
    NSString *ident;
    if (sender) ident = [sender itemIdentifier];
    else ident = ConnectionToolbarItem;

    if (currentView != ident) {
        NSView *old = [[self window] contentView];
        NSView *new = [views objectForKey:ident];
        
        [[self window] setContentView:loadingView];
        [[self window] setTitle:ident];
        
        NSRect r=[[self window] frame];
        float deltay=[new frame].size.height-[old frame].size.height;
        float deltax=[new frame].size.width-[old frame].size.width;
        r.size.height+=deltay;
        r.size.width+=deltax;
        r.origin.y-=deltay;
        
        [[self window] setFrame:r display:YES animate:YES];
        [[self window] setContentView:new];

        [currentView release];
        currentView=[ident retain];
    }
}


//=======================================

-(NSDictionary*)toDict {
    NSMutableDictionary	*dict = [[NSMutableDictionary alloc] init];
    
    // login infos
    [dict setObject:UIN forKey:@"UIN"];
    [dict setObject:password forKey:@"password"];
    [dict setObject:[NSNumber numberWithBool:rememberPassword] forKey:@"rememberPassword"];
    [dict setObject:[NSNumber numberWithInt:loginStatus] forKey:@"loginStatus"];
    
    
    // connection
    [dict setObject:icqServerHost forKey:@"icqServerHost"];
    [dict setObject:icqServerPort forKey:@"icqServerPort"];
    
    // GUI
    [dict setObject:iconTheme forKey:@"iconTheme"];
    [dict setObject:[SMConversions color2Dict:contactListBGColor] forKey:@"contactListBGColor"];
    [dict setObject:[SMConversions color2Dict:contactListContactTextColor] forKey:@"contactListContactTextColor"];
    [dict setObject:[SMConversions color2Dict:contactListGroupTextColor] forKey:@"contactListGroupTextColor"];
    [dict setObject:[NSNumber numberWithFloat:contactListAlphaLevel] forKey:@"contactListAlphaLevel"];
    [dict setObject:[NSNumber numberWithFloat:globalAlphaLevelWhenNotInFront] forKey:@"globalAlphaLevelWhenNotInFront"];
    [dict setObject:[NSNumber numberWithBool:contactListStayOnTop] forKey:@"contactListStayOnTop"];
    [dict setObject:[SMConversions rect2Dict:contactListPosition] forKey:@"contactListPosition"];
    [dict setObject:[SMConversions rect2Dict:searchWindowPosition] forKey:@"searchWindowPosition"];
    [dict setObject:[SMConversions rect2Dict:preferenceWindowPosition] forKey:@"preferenceWindowPosition"];
    [dict setObject:[SMConversions point2Dict:userDetailsPanelPosition] forKey:@"userDetailsPanelPosition"];
    [dict setObject:[SMConversions rect2Dict:historyManagerPosition] forKey:@"historyManagerPosition"];
    [dict setObject:[SMConversions rect2Dict:listsWindowPosition] forKey:@"listsWindowPosition"];
    [dict setObject:[SMConversions color2Dict:messageWindowContactTextColor] forKey:@"messageWindowContactTextColor"];
    [dict setObject:[SMConversions color2Dict:messageWindowSelfTextColor] forKey:@"messageWindowSelfTextColor"];
    [dict setObject:[SMConversions color2Dict:messageWindowHistoryBGColor] forKey:@"messageWindowHistoryBGColor"];
    [dict setObject:[SMConversions color2Dict:messageWindowMessageBGColor] forKey:@"messageWindowMessageBGColor"];
    [dict setObject:[NSNumber numberWithFloat:messageWindowAlphaLevel] forKey:@"messageWindowAlphaLevel"];
    [dict setObject:[NSNumber numberWithBool:messageWindowStayOnTop] forKey:@"messageWindowStayOnTop"];
    [dict setObject:[NSNumber numberWithBool:blendOfflineContacts] forKey:@"blendOfflineContacts"];
    [dict setObject:[NSNumber numberWithBool:hideOfflineContacts] forKey:@"hideOfflineContacts"];
    [dict setObject:[NSNumber numberWithBool:checkSpelling] forKey:@"checkSpelling"];
    [dict setObject:[NSNumber numberWithBool:autoOpenNewContactDetails] forKey:@"autoOpenNewContactDetails"];
    [dict setObject:[NSNumber numberWithBool:playSound] forKey:@"playSound"];
    [dict setObject:[NSNumber numberWithBool:autoOpen] forKey:@"autoOpen"];
    [dict setObject:[NSNumber numberWithBool:sortByStatus] forKey:@"sortByStatus"];
    [dict setObject:[NSNumber numberWithBool:sortByName] forKey:@"sortByName"];
    [dict setObject:[NSNumber numberWithInt:bounceWhenReceivingMessage] forKey:@"bounceWhenReceivingMessage"];
    [dict setObject:[NSNumber numberWithInt:autoAwayDelay] forKey:@"autoAwayDelay"];
    [dict setObject:[NSNumber numberWithInt:autoNADelay] forKey:@"autoNADelay"];
    [dict setObject:[NSNumber numberWithInt:autoAwayCorner] forKey:@"autoAwayCorner"];
    
    
    
    return dict;
}

-(void)fromDict:(NSDictionary*)dict {
    // login infos
    [self setUIN:[dict objectForKey:@"UIN"]];
    [self setPassword:[dict objectForKey:@"password"]];
    [self setRememberPassword:[[dict objectForKey:@"rememberPassword"] boolValue]];
    [self setLoginStatus:[[dict objectForKey:@"loginStatus"] intValue]];
    
    
    // connection
    [self setIcqServerHost:[dict objectForKey:@"icqServerHost"]];
    [self setIcqServerPort:[dict objectForKey:@"icqServerPort"]];
    
    // GUI
    if ([dict objectForKey:@"iconTheme"]) {
        [self setIconTheme:[dict objectForKey:@"iconTheme"]];
    } else {
        [self setIconTheme:@"default"];
    }

    blendOfflineContacts=YES;
    playSound=YES;
    sortByName=NO;
    sortByStatus=NO;
    autoOpen=NO;
    contactListBGColor = nil;
    contactListContactTextColor= nil;
    contactListGroupTextColor = nil;
    messageWindowContactTextColor = nil;
    messageWindowSelfTextColor = nil;
    messageWindowHistoryBGColor = nil;
    messageWindowMessageBGColor = nil;
    messageWindowMessageBGColor = nil;
    autoAwayCorner=1;
    [self setContactListBGColor:[SMConversions dict2Color:[dict objectForKey:@"contactListBGColor"]]];
    if (!contactListBGColor)  contactListBGColor = [NSColor whiteColor];
    [self setContactListContactTextColor:[SMConversions dict2Color:[dict objectForKey:@"contactListContactTextColor"]]];
    if (!contactListContactTextColor)  contactListContactTextColor = [NSColor blackColor];
    [self setContactListGroupTextColor:[SMConversions dict2Color:[dict objectForKey:@"contactListGroupTextColor"]]];
    if (!contactListGroupTextColor)  contactListGroupTextColor = [NSColor blackColor];
    [self setGlobalAlphaLevelWhenNotInFront:[[dict objectForKey:@"globalAlphaLevelWhenNotInFront"] floatValue]];
    if (![dict objectForKey:@"globalAlphaLevelWhenNotInFront"]) globalAlphaLevelWhenNotInFront=1.0;
    [self setContactListAlphaLevel:[[dict objectForKey:@"contactListAlphaLevel"] floatValue]];
    if (contactListAlphaLevel==0) contactListAlphaLevel=1.0;
    [self setContactListPosition:[SMConversions dict2Rect:[dict objectForKey:@"contactListPosition"]]];
    [self setSearchWindowPosition:[SMConversions dict2Rect:[dict objectForKey:@"searchWindowPosition"]]];
    [self setListsWindowPosition:[SMConversions dict2Rect:[dict objectForKey:@"listsWindowPosition"]]];
    [self setPreferenceWindowPosition:[SMConversions dict2Rect:[dict objectForKey:@"preferenceWindowPosition"]]];
    [self setHistoryManagerPosition:[SMConversions dict2Rect:[dict objectForKey:@"historyManagerPosition"]]];
    [self setUserDetailsPanelPosition:[SMConversions dict2Point:[dict objectForKey:@"userDetailsPanelPosition"]]];
    [self setContactListStayOnTop:[[dict objectForKey:@"contactListStayOnTop"] boolValue]];
    
    [self setMessageWindowContactTextColor:[SMConversions dict2Color:[dict objectForKey:@"messageWindowContactTextColor"]]];
    if (!messageWindowContactTextColor)  messageWindowContactTextColor = [NSColor blueColor];
    [self setMessageWindowSelfTextColor:[SMConversions dict2Color:[dict objectForKey:@"messageWindowSelfTextColor"]]];
    if (!messageWindowSelfTextColor)  messageWindowSelfTextColor = [NSColor greenColor];
    [self setMessageWindowHistoryBGColor:[SMConversions dict2Color:[dict objectForKey:@"messageWindowHistoryBGColor"]]];
    if (!messageWindowHistoryBGColor)  messageWindowHistoryBGColor = [NSColor whiteColor];
    [self setMessageWindowMessageBGColor:[SMConversions dict2Color:[dict objectForKey:@"messageWindowMessageBGColor"]]];
    if (!messageWindowMessageBGColor)  messageWindowMessageBGColor = [NSColor whiteColor];
    [self setMessageWindowAlphaLevel:[[dict objectForKey:@"messageWindowAlphaLevel"] floatValue]];
    if (messageWindowAlphaLevel==0) messageWindowAlphaLevel=1.0;
    [self setMessageWindowStayOnTop:[[dict objectForKey:@"messageWindowStayOnTop"] boolValue]];
    if ([dict objectForKey:@"blendOfflineContacts"]) {
        [self setBlendOfflineContacts:[[dict objectForKey:@"blendOfflineContacts"] boolValue]];
    }
    if ([dict objectForKey:@"checkSpelling"]) {
        [self setCheckSpelling:[[dict objectForKey:@"checkSpelling"] boolValue]];
    }
    if ([dict objectForKey:@"autoOpenNewContactDetails"]) {
        [self setAutoOpenNewContactDetails:[[dict objectForKey:@"autoOpenNewContactDetails"] boolValue]];
    }
    if ([dict objectForKey:@"playSound"]) {
	  [self setPlaySound:[[dict objectForKey:@"playSound"] boolValue]];
    }
    if ([dict objectForKey:@"sortByStatus"]) {
	  [self setSortByStatus:[[dict objectForKey:@"sortByStatus"] boolValue]];
    }
    if ([dict objectForKey:@"sortByName"]) {
	  [self setSortByName:[[dict objectForKey:@"sortByName"] boolValue]];
    }
    if ([dict objectForKey:@"autoOpen"]) {
        [self setAutoOpen:[[dict objectForKey:@"autoOpen"] boolValue]];
    }
    if ([dict objectForKey:@"hideOfflineContacts"]) {
        [self setHideOfflineContacts:[[dict objectForKey:@"hideOfflineContacts"] boolValue]];
    }
    if ([dict objectForKey:@"bounceWhenReceivingMessage"]) {
        [self setBounceWhenReceivingMessage:[[dict objectForKey:@"bounceWhenReceivingMessage"] intValue]];
    }
    
    
    if ([dict objectForKey:@"autoAwayDelay"]) {
        [self setAutoAwayDelay:[[dict objectForKey:@"autoAwayDelay"] intValue]];
    }
    if ([dict objectForKey:@"autoNADelay"]) {
        [self setAutoNADelay:[[dict objectForKey:@"autoNADelay"] intValue]];
    }
    if ([dict objectForKey:@"autoAwayCorner"]) {
        [self setAutoAwayCorner:[[dict objectForKey:@"autoAwayCorner"] intValue]];
    }
    
    [self displayValues];
    
    if (![[UINOutlet stringValue] isEqualToString:@""]) {
        [UINOutlet setEnabled:NO];
        [passwordOutlet setEnabled:NO];
        [changeOutlet setTitle:NSLocalizedString(@"Edit", @"Edit (name of edit/set button in prefs)")];
    }
    
    if ([self preferenceWindowPosition].size.height>0) {
       //[[self window] setFrame:[self preferenceWindowPosition] display:NO];
       [[self window] setFrameTopLeftPoint:NSMakePoint([self preferenceWindowPosition].origin.x,[self preferenceWindowPosition].origin.y+[self preferenceWindowPosition].size.height)];
    }
    
    [self notifyChange];
}

-(void)displayValues {
    if (iconTheme) {
        int i=0;
   	NSArray *keys=[[SMThemeManager sharedManager] themes];
        while (i<[keys count] && ![[keys objectAtIndex:i] isEqualToString:iconTheme]) {
            i++;
        }
        if (i<[keys count]) {
            [iconThemeOutlet selectItemAtIndex:i];
        }
    }
    
    [UINOutlet setStringValue:UIN];
    [passwordOutlet setStringValue:password];
    [rememberPasswordOutlet setState:rememberPassword];
    if (loginStatus) {
        [loginStatusOutlet selectItemAtIndex:loginStatus-1];
    } else {
        [loginStatusOutlet selectItemAtIndex:7];
    }

    
    [icqServerHostOutlet setStringValue:icqServerHost];
    [icqServerPortOutlet setStringValue:icqServerPort];
    
    [contactListBGColorOutlet setColor:contactListBGColor];
    [contactListContactTextColorOutlet setColor:contactListContactTextColor];
    [contactListGroupTextColorOutlet setColor:contactListGroupTextColor];
    [contactListAlphaLevelOutlet setFloatValue:contactListAlphaLevel];
    [globalAlphaLevelWhenNotInFrontOutlet setFloatValue:globalAlphaLevelWhenNotInFront];
    [contactListStayOnTopOutlet setState:contactListStayOnTop];
    [messageWindowContactTextColorOutlet setColor:messageWindowContactTextColor];
    [messageWindowSelfTextColorOutlet setColor:messageWindowSelfTextColor];
    [messageWindowHistoryBGColorOutlet setColor:messageWindowHistoryBGColor];
    [messageWindowMessageBGColorOutlet setColor:messageWindowMessageBGColor];
    [messageWindowAlphaLevelOutlet setFloatValue:messageWindowAlphaLevel];
    [messageWindowStayOnTopOutlet setState:messageWindowStayOnTop];
    [blendOfflineContactsOutlet setState:blendOfflineContacts];
    [hideOfflineContactsOutlet setState:hideOfflineContacts];
    [autoOpenNewContactDetailsOutlet setState:autoOpenNewContactDetails];
    [playSoundOutlet setState:playSound];
    [autoOpenOutlet setState:autoOpen];
    [sortByNameOutlet setState:sortByName];
    [sortByStatusOutlet setState:sortByStatus];
    [autoAwayCornerOutlet selectItemAtIndex:autoAwayCorner];
    [autoAwayDelayOutlet setStringValue:[[NSNumber numberWithInt:autoAwayDelay] stringValue]];
    [autoNADelayOutlet setStringValue:[[NSNumber numberWithInt:autoNADelay] stringValue]];
    [bounceWhenReceivingMessageOutlet selectItemAtIndex:bounceWhenReceivingMessage];
    
    [checkSpellingOutlet setState:checkSpelling];
    
    
}


// IB actions : 



-(void)setUINAction:(id)sender {
    if ([[changeOutlet title] isEqualToString:NSLocalizedString(@"Set", @"Set (name of edit/set button in prefs)")]) {
        [self setUIN:[UINOutlet stringValue]];
        [self setPassword:[passwordOutlet stringValue]];
        [parent changeUIN];
        [parent saveDocument:self];
        [changeOutlet setTitle:NSLocalizedString(@"Edit", @"Edit (name of edit/set button in prefs)")];
        [UINOutlet setEnabled:NO];
        [passwordOutlet setEnabled:NO];
    } else {
         
         if ([self currentStatus]>0) {
            NSBeginAlertSheet(	NSLocalizedString(@"Change login/pass",@"Change login/pass") ,
                                NSLocalizedString(@"Cancel",@"Cancel"),
                                NSLocalizedString(@"Ok", @"Ok"),
                                nil, 
                                [self window], 
                                self, 
                                nil,
                                @selector(doChangeUIN:returnCode:contextInfo:), 
                                nil,
                                NSLocalizedString(@"Are you sure you want to modify theses informations? This will disconnect you from the icq Network...",@"Are you sure you want to modify theses informations? This will disconnect you from the icq Network..."));
        } else {
            [self doChangeUIN:nil returnCode:!NSAlertDefaultReturn contextInfo:nil];
        }
    }
}
-(void)registerAction:(id)sender {
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://go.icq.com/register/" ]];
}
-(void)setPasswordAction:(id)sender {
    [self setPassword:[passwordOutlet stringValue]];
}
-(void)setRememberPasswordAction:(id)sender {
    [self setRememberPassword:[[NSNumber numberWithInt:[rememberPasswordOutlet state]] boolValue]];
}
-(void)setLoginStatusAction:(id)sender {
    int status=[loginStatusOutlet indexOfSelectedItem]+1;
    if (status==8) status=0;
    [self setLoginStatus:status];
}
-(void)setIcqServerHostAction:(id)sender {
    [self setIcqServerHost:[icqServerHostOutlet stringValue]];
}
-(void)setIcqServerPortAction:(id)sender {
    [self setIcqServerPort:[icqServerPortOutlet stringValue]];
}
-(void)setContactListBGColorAction:(id)sender {
    if ([[contactListBGColorOutlet color] alphaComponent]<1.0 && (contactListAlphaLevel==1.0 || globalAlphaLevelWhenNotInFront==1.0 )) {
        contactListAlphaLevel=0.99;
        globalAlphaLevelWhenNotInFront=0.99;
    }
    [self setContactListBGColor:[contactListBGColorOutlet color]];
}
-(void)setContactListContactTextColorAction:(id)sender {
    [self setContactListContactTextColor:[contactListContactTextColorOutlet color]];
}
-(void)setContactListGroupTextColorAction:(id)sender {
    [self setContactListGroupTextColor:[contactListGroupTextColorOutlet color]];
}
-(void)setContactListAlphaLevelAction:(id)sender {
    if ([[contactListBGColorOutlet color] alphaComponent]<1.0 && [contactListAlphaLevelOutlet floatValue]==1.0) {
        [self setContactListAlphaLevel:0.99];
    } else {
        [self setContactListAlphaLevel:[contactListAlphaLevelOutlet floatValue]];
    }
}
-(void)setGlobalAlphaLevelWhenNotInFrontAction:(id)sender {
    if ([[contactListBGColorOutlet color] alphaComponent]<1.0 && [globalAlphaLevelWhenNotInFrontOutlet floatValue]==1.0) {
        [self setContactListAlphaLevel:0.99];
    } else {
        [self setGlobalAlphaLevelWhenNotInFront:[globalAlphaLevelWhenNotInFrontOutlet floatValue]];
    }
    
}
-(void)setIconThemeAction:(id)sender {
    [self setIconTheme:[sender titleOfSelectedItem]];
 
}
-(void)setContactListStayOnTopAction:(id)sender {
    [self setContactListStayOnTop:[contactListStayOnTopOutlet state]];
    
}
-(void)setHideOfflineContactsAction:(id)sender {
    [self setHideOfflineContacts:[hideOfflineContactsOutlet state]];
    
}
-(void)setCheckSpellingAction:(id)sender {
    [self setCheckSpelling:[checkSpellingOutlet state]];
    
}
-(void)setMessageWindowContactTextColorAction:(id)sender {
    [self setMessageWindowContactTextColor:[messageWindowContactTextColorOutlet color]];
    
}
-(void)setMessageWindowSelfTextColorAction:(id)sender {
    [self setMessageWindowSelfTextColor:[messageWindowSelfTextColorOutlet color]];
    
}
-(void)setMessageWindowHistoryBGColorAction:(id)sender {
    if ([[messageWindowHistoryBGColorOutlet color] alphaComponent]<1.0 && messageWindowAlphaLevel==1.0) {
        messageWindowAlphaLevel=0.99;
    }
    [self setMessageWindowHistoryBGColor:[messageWindowHistoryBGColorOutlet color]];
}
-(void)setMessageWindowMessageBGColorAction:(id)sender {
    if ([[messageWindowMessageBGColorOutlet color] alphaComponent]<1.0 && messageWindowAlphaLevel==1.0) {
        messageWindowAlphaLevel=0.99;
    }
    [self setMessageWindowMessageBGColor:[messageWindowMessageBGColorOutlet color]];
}
-(void)setMessageWindowAlphaLevelAction:(id)sender {
    if (([[messageWindowMessageBGColorOutlet color] alphaComponent]<1.0 || [[messageWindowHistoryBGColorOutlet color] alphaComponent]<1.0) && [messageWindowAlphaLevelOutlet floatValue]==1.0) {
        [self setMessageWindowAlphaLevel:0.99];
    } else {
        [self setMessageWindowAlphaLevel:[messageWindowAlphaLevelOutlet floatValue]];
    }
}
-(void)setMessageWindowStayOnTopAction:(id)sender {
    [self setMessageWindowStayOnTop:[messageWindowStayOnTopOutlet state]];
    
}
-(void)setBlendOfflineContactsAction:(id)sender {
    [self setBlendOfflineContacts:[blendOfflineContactsOutlet state]];
    
}
-(void)setAutoAwayDelayAction:(id)sender {
    [self setAutoAwayDelay:[autoAwayDelayOutlet intValue]];
}
-(void)setAutoNADelayAction:(id)sender {
    [self setAutoNADelay:[autoNADelayOutlet intValue]];
}
-(void)setAutoAwayCornerAction:(id)sender {
    int c=[autoAwayCornerOutlet indexOfSelectedItem];
    [self setAutoAwayCorner:c];
}
-(void)setAutoOpenNewContactDetailsAction:(id)sender {
    [self setAutoOpenNewContactDetails:[autoOpenNewContactDetailsOutlet intValue]];
}
-(void)setPlaySoundAction:(id)sender {
  [self setPlaySound:[playSoundOutlet intValue]];
}
-(void)setSortByStatusAction:(id)sender {
  [self setSortByStatus:[sortByStatusOutlet intValue]];
}
-(void)setSortByNameAction:(id)sender {
  [self setSortByName:[sortByNameOutlet intValue]];
}
-(void)setAutoOpenAction:(id)sender {
    [self setAutoOpen:[autoOpenOutlet intValue]];
}
-(void)setBounceWhenReceivingMessageAction:(id)sender{
    [self setBounceWhenReceivingMessage:[bounceWhenReceivingMessageOutlet indexOfSelectedItem]];
}
// accessors

// session
-(void)setCurrentStatus:(int)status {
    currentStatus=status;
}
-(int)currentStatus {
    return currentStatus;
}
-(void)setSessionLoginStatus:(int)status {
    sessionLoginStatus=status;
}
-(int)sessionLoginStatus {
    return sessionLoginStatus;
}

// login infos
-(void)setUIN:(NSString*)uin{
    if (UIN!=uin) {
        [UIN release];
        UIN=[uin retain];
        [self notifyChange];
    }
}
-(NSString*)UIN{
    return [UIN retain];
}

-(void)setPassword:(NSString*)pass{
    if (password!=pass) {
        [password release];
        password=[pass retain];
        [self notifyChange];
    }
}
-(NSString*)password {
    return [password retain];
}
-(void)setRememberPassword:(BOOL)remember {
    rememberPassword = remember;
    [self notifyChange];
}

-(BOOL)rememberPassword {
    return rememberPassword;
}

// Connection
-(void)setIcqServerHost:(NSString*)host {
    if (icqServerHost!=host) {
        [icqServerHost release];
        icqServerHost=[host retain];
        [self notifyChange];
    }
}
-(NSString*)icqServerHost {
    return icqServerHost;
}
-(void)setIcqServerPort:(NSString*)port {
    if (icqServerPort!=port) {
        [icqServerPort release];
        icqServerPort=[port retain];
        [self notifyChange];
    }
}
-(NSString*)icqServerPort {
    return icqServerPort;
}
-(void)setLoginStatus:(int)status {
    loginStatus = status;
    [self notifyChange];
}
-(int)loginStatus {
    return loginStatus;
}


// GUI
-(SMTheme*)currentTheme {
    return currentTheme;
}
-(void)setCurrentTheme:(NSString*)name {
    [currentTheme release];
    currentTheme=[[[SMThemeManager sharedManager] themeWithName:name] retain];
    
}

-(void)setIconTheme:(NSString*)value{
    if (value!=iconTheme) {
        [iconTheme release];
        iconTheme=[value copy];
        [self setCurrentTheme:iconTheme];
        [self notifyChange];
    }
}
-(NSString*)iconTheme{
    return [iconTheme retain];
}

-(void)setAutoAwayDelay:(int)value {
    autoAwayDelay = value;
    [self notifyChange];
}
-(int)autoAwayDelay{
    return autoAwayDelay;
}
-(void)setAutoNADelay:(int)value {
    autoNADelay = value;
    [self notifyChange];
}
-(int)autoNADelay{
    return autoNADelay;
}
-(void)setAutoAwayCorner:(int)value {
    autoAwayCorner = value;
    [self notifyChange];
}
-(int)autoAwayCorner{
    return autoAwayCorner;
}

-(void)setContactListBGColor:(NSColor*)color {
    if (contactListBGColor!=color) {
        [contactListBGColor release];
        contactListBGColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)contactListBGColor {
    return contactListBGColor;
}
-(void)setContactListContactTextColor:(NSColor*)color {
    if (contactListContactTextColor!=color) {
        [contactListContactTextColor release];
        contactListContactTextColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)contactListContactTextColor {
    return contactListContactTextColor;
}
-(void)setContactListGroupTextColor:(NSColor*)color {
    if (contactListGroupTextColor!=color) {
        [contactListGroupTextColor release];
        contactListGroupTextColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)contactListGroupTextColor {
    return contactListGroupTextColor;
}
-(void)setContactListAlphaLevel:(float)level {
    contactListAlphaLevel = level;
    [self notifyChange];
}
-(float)contactListAlphaLevel {
    return contactListAlphaLevel;
}
-(void)setGlobalAlphaLevelWhenNotInFront:(float)level {
    globalAlphaLevelWhenNotInFront = level;
}
-(float)globalAlphaLevelWhenNotInFront {
    return globalAlphaLevelWhenNotInFront;
}
-(void)setContactListStayOnTop:(BOOL)state {
    contactListStayOnTop = state;
    [self notifyChange];
}
-(BOOL)contactListStayOnTop {
    return contactListStayOnTop;
}
-(void)setCheckSpelling:(BOOL)state {
    checkSpelling = state;
    [self notifyChange];
}
-(BOOL)checkSpelling {
    return checkSpelling;
}

-(void)setHideOfflineContacts:(BOOL)state {
    hideOfflineContacts = state;
    [self notifyChange];
}
-(BOOL)hideOfflineContacts {
    return hideOfflineContacts;
}
-(void)setBlendOfflineContacts:(BOOL)state {
    blendOfflineContacts = state;
    [self notifyChange];
}
-(BOOL)blendOfflineContacts {
    return blendOfflineContacts;
}
-(void)setContactListPosition:(NSRect)position {
    contactListPosition = position;
    [self notifyChange];
}
-(NSRect)contactListPosition {
    return contactListPosition;
}
-(void)setMessageWindowContactTextColor:(NSColor*)color {
    if (messageWindowContactTextColor!=color) {
        [messageWindowContactTextColor release];
        messageWindowContactTextColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)messageWindowContactTextColor {
    return messageWindowContactTextColor;
}
-(void)setMessageWindowSelfTextColor:(NSColor*)color {
    if (messageWindowSelfTextColor!=color) {
        [messageWindowSelfTextColor release];
        messageWindowSelfTextColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)messageWindowSelfTextColor {
    return messageWindowSelfTextColor;
}
-(void)setMessageWindowHistoryBGColor:(NSColor*)color {
    if (messageWindowHistoryBGColor!=color) {
        [messageWindowHistoryBGColor release];
        messageWindowHistoryBGColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)messageWindowHistoryBGColor {
    return messageWindowHistoryBGColor;
}
-(void)setMessageWindowMessageBGColor:(NSColor*)color {
    if (messageWindowMessageBGColor!=color) {
        [messageWindowMessageBGColor release];
        messageWindowMessageBGColor=[color retain];
        [self notifyChange];
    }
}
-(NSColor*)messageWindowMessageBGColor {
    return messageWindowMessageBGColor;
}
-(void)setMessageWindowAlphaLevel:(float)level {
    messageWindowAlphaLevel = level;
    [self notifyChange];
}
-(float)messageWindowAlphaLevel {
    return messageWindowAlphaLevel;
}
-(void)setMessageWindowStayOnTop:(BOOL)state {
    messageWindowStayOnTop = state;
    [self notifyChange];
}
-(BOOL)messageWindowStayOnTop {
    return messageWindowStayOnTop;
}
-(void)setAutoOpenNewContactDetails:(BOOL)state {
    autoOpenNewContactDetails = state;
    [self notifyChange];
}
-(void)setPlaySound:(BOOL)state {
  playSound = state;
  [self notifyChange];
}
-(void)setSortByStatus:(BOOL)state {
  sortByStatus = state;
  [self notifyChange];
}
-(void)setSortByName:(BOOL)state {
  sortByName = state;
  [self notifyChange];
}
-(void)setAutoOpen:(BOOL)state {
    autoOpen = state;
    [self notifyChange];
}
-(BOOL)autoOpenNewContactDetails {
    return autoOpenNewContactDetails;
}
-(BOOL)playSound {
  return playSound;
}
-(BOOL)autoOpen {
    return autoOpen;
}
-(BOOL)sortByName {
  return sortByName;
}
-(BOOL)sortByStatus {
  return sortByStatus;
}
-(void)setBounceWhenReceivingMessage:(int)state {
    bounceWhenReceivingMessage = state;
    [self notifyChange];
}
-(int)bounceWhenReceivingMessage {
    return bounceWhenReceivingMessage;
}
-(void)setSearchWindowPosition:(NSRect)rect {
    searchWindowPosition=rect;
    [self notifyChange];
}
-(NSRect)searchWindowPosition {
    return searchWindowPosition;
}
-(void)setPreferenceWindowPosition:(NSRect)rect {
    preferenceWindowPosition=rect;
    [self notifyChange];
}
-(NSRect)preferenceWindowPosition {
    return preferenceWindowPosition;
}
-(void)setListsWindowPosition:(NSRect)rect {
    listsWindowPosition=rect;
    [self notifyChange];
}
-(NSRect)listsWindowPosition {
    return listsWindowPosition;
}
-(void)setHistoryManagerPosition:(NSRect)rect {
    historyManagerPosition=rect;
    [self notifyChange];
}
-(NSRect)historyManagerPosition {
    return historyManagerPosition;
}
-(void)setUserDetailsPanelPosition:(NSPoint)point {
    userDetailsPanelPosition=point;
    [self notifyChange];
}
-(NSPoint)userDetailsPanelPosition {
    return userDetailsPanelPosition;
}

// observers
-(void)statusChanged:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
     

    if ([uin isEqualToString:[self UIN]]) {
        int status = [[[notification userInfo] objectForKey:@"status"] intValue];
        [self  setCurrentStatus:status];
    }
}

-(void)connectionCompleted:(NSNotification *)notification {
    NSString *uin = [[notification userInfo] objectForKey:@"uin"];
    
    if ([uin isEqualToString:[self UIN]]) {
        [self setSessionLoginStatus:0];
    }
}

// observers
- (void)windowWillClose:(NSNotification *)aNotification {
    [self setPreferenceWindowPosition:[[self window] frame]];
    [parent saveAccountPreferences];
}

// utils

-(void)makeKeyAndOrderFront:(id)sender {
    [self showWindow];
}
-(void)notifyChange {
    NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
        UIN, @"uin",
        nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"preferencesChanged" object:self userInfo:dict];
    
}

- (void)doChangeUIN:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo 
{
    if (returnCode != NSAlertDefaultReturn) {
            // disconnect
            NSDictionary *dict=[NSDictionary dictionaryWithObjectsAndKeys:
                [self UIN], @"uin",
                0, @"status",
                nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"userChangedStatus" object:self userInfo:dict];
            
            
            [UINOutlet setEnabled:YES];
            [passwordOutlet setEnabled:YES];
            [changeOutlet setTitle:@"Set"];
    }
}

- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
    NSString *str = NSLocalizedString(@"Preferences for UIN: ",@"Preferences for UIN: ");
    return [str stringByAppendingString:[self UIN]];
}

@end
